@@ -12,23 +12,25 @@
 #ifndef _DDS_RHC_DEFAULT_H_
 #define _DDS_RHC_DEFAULT_H_
 
+#include "dds/features.h"
+
 #if defined (__cplusplus)
 extern "C" {
 #endif
 
 struct dds_rhc;
 struct dds_reader;
-struct ddsi_sertopic;
+struct ddsi_sertype;
 struct ddsi_domaingv;
 struct dds_rhc_default;
 struct rhc_sample;
 
-DDS_EXPORT struct dds_rhc *dds_rhc_default_new_xchecks (dds_reader *reader, struct ddsi_domaingv *gv, const struct ddsi_sertopic *topic, bool xchecks);
-DDS_EXPORT struct dds_rhc *dds_rhc_default_new (struct dds_reader *reader, const struct ddsi_sertopic *topic);
-#ifdef DDSI_INCLUDE_LIFESPAN
+DDS_EXPORT struct dds_rhc *dds_rhc_default_new_xchecks (dds_reader *reader, struct ddsi_domaingv *gv, const struct ddsi_sertype *type, bool xchecks);
+DDS_EXPORT struct dds_rhc *dds_rhc_default_new (struct dds_reader *reader, const struct ddsi_sertype *type);
+#ifdef DDS_HAS_LIFESPAN
 DDS_EXPORT ddsrt_mtime_t dds_rhc_default_sample_expired_cb(void *hc, ddsrt_mtime_t tnow);
 #endif
-#ifdef DDSI_INCLUDE_DEADLINE_MISSED
+#ifdef DDS_HAS_DEADLINE_MISSED
 DDS_EXPORT ddsrt_mtime_t dds_rhc_default_deadline_missed_cb(void *hc, ddsrt_mtime_t tnow);
 #endif
 
