@@ -21,16 +21,19 @@
 #include "dds__listener.h"
 #include "dds__qos.h"
 #include "dds__topic.h"
+#include "dds__builtin.h"
+#include "dds__subscriber.h" // for non-materialized DATA_ON_READERS
 #include "dds/version.h"
 #include "dds/ddsi/ddsi_pmd.h"
 #include "dds/ddsi/ddsi_xqos.h"
 #include "dds/ddsi/q_transmit.h"
 #include "dds/ddsi/q_bswap.h"
 
-extern inline dds_entity *dds_entity_from_handle_link (struct dds_handle_link *hdllink);
-extern inline bool dds_entity_is_enabled (const dds_entity *e);
-extern inline void dds_entity_status_reset (dds_entity *e, status_mask_t t);
-extern inline dds_entity_kind_t dds_entity_kind (const dds_entity *e);
+DDS_EXPORT extern inline dds_entity *dds_entity_from_handle_link (struct dds_handle_link *hdllink);
+DDS_EXPORT extern inline bool dds_entity_is_enabled (const dds_entity *e);
+DDS_EXPORT extern inline void dds_entity_status_reset (dds_entity *e, status_mask_t t);
+DDS_EXPORT extern inline uint32_t dds_entity_status_reset_ov (dds_entity *e, status_mask_t t);
+DDS_EXPORT extern inline dds_entity_kind_t dds_entity_kind (const dds_entity *e);
 
 const struct dds_entity_deriver *dds_entity_deriver_table[] = {
   [DDS_KIND_TOPIC] = &dds_entity_deriver_topic,
@@ -69,15 +72,15 @@ void dds_entity_deriver_dummy_refresh_statistics (const struct dds_entity *e, st
   (void) e; (void) s;
 }
 
-extern inline void dds_entity_deriver_interrupt (struct dds_entity *e);
-extern inline void dds_entity_deriver_close (struct dds_entity *e);
-extern inline dds_return_t dds_entity_deriver_delete (struct dds_entity *e);
-extern inline dds_return_t dds_entity_deriver_set_qos (struct dds_entity *e, const dds_qos_t *qos, bool enabled);
-extern inline dds_return_t dds_entity_deriver_validate_status (struct dds_entity *e, uint32_t mask);
-extern inline bool dds_entity_supports_set_qos (struct dds_entity *e);
-extern inline bool dds_entity_supports_validate_status (struct dds_entity *e);
-extern inline struct dds_statistics *dds_entity_deriver_create_statistics (const struct dds_entity *e);
-extern inline void dds_entity_deriver_refresh_statistics (const struct dds_entity *e, struct dds_statistics *s);
+DDS_EXPORT extern inline void dds_entity_deriver_interrupt (struct dds_entity *e);
+DDS_EXPORT extern inline void dds_entity_deriver_close (struct dds_entity *e);
+DDS_EXPORT extern inline dds_return_t dds_entity_deriver_delete (struct dds_entity *e);
+DDS_EXPORT extern inline dds_return_t dds_entity_deriver_set_qos (struct dds_entity *e, const dds_qos_t *qos, bool enabled);
+DDS_EXPORT extern inline dds_return_t dds_entity_deriver_validate_status (struct dds_entity *e, uint32_t mask);
+DDS_EXPORT extern inline bool dds_entity_supports_set_qos (struct dds_entity *e);
+DDS_EXPORT extern inline bool dds_entity_supports_validate_status (struct dds_entity *e);
+DDS_EXPORT extern inline struct dds_statistics *dds_entity_deriver_create_statistics (const struct dds_entity *e);
+DDS_EXPORT extern inline void dds_entity_deriver_refresh_statistics (const struct dds_entity *e, struct dds_statistics *s);
 
 static int compare_instance_handle (const void *va, const void *vb)
 {
@@ -88,12 +91,22 @@ static int compare_instance_handle (const void *va, const void *vb)
 
 const ddsrt_avl_treedef_t dds_entity_children_td = DDSRT_AVL_TREEDEF_INITIALIZER (offsetof (struct dds_entity, m_avlnode_child), offsetof (struct dds_entity, m_iid), compare_instance_handle, 0);
 
-static void dds_entity_observers_signal (dds_entity *observed, uint32_t status);
 static void dds_entity_observers_signal_delete (dds_entity *observed);
 
 static dds_return_t dds_delete_impl (dds_entity_t entity, enum delete_impl_state delstate);
 static dds_return_t really_delete_pinned_closed_locked (struct dds_entity *e, enum delete_impl_state delstate);
 
+static bool entity_is_builtin_topic (const struct dds_entity *entity)
+{
+  if (dds_entity_kind (entity) != DDS_KIND_TOPIC)
+    return false;
+  else
+  {
+    const dds_topic *tp = (dds_topic *) entity;
+    return builtintopic_is_builtintopic (&tp->m_entity.m_domain->btif, tp->m_stype);
+  }
+}
+
 void dds_entity_add_ref_locked (dds_entity *e)
 {
   dds_handle_add_ref (&e->m_hdllink);
@@ -196,7 +209,7 @@ static bool entity_kind_has_qos (dds_entity_kind_t kind)
 }
 #endif
 
-dds_entity_t dds_entity_init (dds_entity *e, dds_entity *parent, dds_entity_kind_t kind, bool implicit, dds_qos_t *qos, const dds_listener_t *listener, status_mask_t mask)
+dds_entity_t dds_entity_init (dds_entity *e, dds_entity *parent, dds_entity_kind_t kind, bool implicit, bool user_access, dds_qos_t *qos, const dds_listener_t *listener, status_mask_t mask)
 {
   dds_handle_t handle;
 
@@ -216,7 +229,10 @@ dds_entity_t dds_entity_init (dds_entity *e, dds_entity *parent, dds_entity_kind
   if (implicit)
     e->m_flags |= DDS_ENTITY_IMPLICIT;
 
-  /* set the status enable based on kind */
+  /* set the status enable based on kind
+     DATA_ON_READERS in mask on a reader, is cleared now, will be set from subscriber in
+     reader-specific init */
+  assert (kind != DDS_KIND_READER || (mask & DDS_DATA_ON_READERS_STATUS) == 0);
   if (entity_has_status (e))
     ddsrt_atomic_st32 (&e->m_status.m_status_and_mask, (uint32_t) mask << SAM_ENABLED_SHIFT);
   else
@@ -259,7 +275,7 @@ dds_entity_t dds_entity_init (dds_entity *e, dds_entity *parent, dds_entity_kind
   {
     /* for topics, refc counts readers/writers, for all others, it counts children (this we can get away with
        as long as topics can't have children) */
-    if ((handle = dds_handle_create (&e->m_hdllink, implicit, entity_may_have_children (e))) <= 0)
+    if ((handle = dds_handle_create (&e->m_hdllink, implicit, entity_may_have_children (e), user_access)) <= 0)
       return (dds_entity_t) handle;
   }
 
@@ -364,7 +380,7 @@ static void print_delete (const dds_entity *e, enum delete_impl_state delstate ,
 
 dds_return_t dds_delete (dds_entity_t entity)
 {
-  return dds_delete_impl (entity, DIS_EXPLICIT);
+  return dds_delete_impl (entity, DIS_USER);
 }
 
 void dds_entity_final_deinit_before_free (dds_entity *e)
@@ -380,7 +396,7 @@ static dds_return_t dds_delete_impl (dds_entity_t entity, enum delete_impl_state
 {
   dds_entity *e;
   dds_return_t ret;
-  if ((ret = dds_entity_pin_for_delete (entity, (delstate != DIS_IMPLICIT), &e)) == DDS_RETCODE_OK)
+  if ((ret = dds_entity_pin_for_delete (entity, (delstate != DIS_IMPLICIT), (delstate == DIS_USER), &e)) == DDS_RETCODE_OK)
     return dds_delete_impl_pinned (e, delstate);
   else if (ret == DDS_RETCODE_TRY_AGAIN) /* non-child refs exist */
     return DDS_RETCODE_OK;
@@ -424,22 +440,15 @@ static dds_return_t really_delete_pinned_closed_locked (struct dds_entity *e, en
      continued cleanup -- while that's quite safe given that GUIDs don't get
      reused quickly, it needs an update) */
   dds_entity_deriver_interrupt (e);
-
-  /* Prevent further listener invocations; dds_set_status_mask locks m_mutex and
-     checks for a pending delete to guarantee that once we clear the mask here,
-     no new listener invocations will occur beyond those already in progress.
-     (FIXME: or committed to?  I think in-progress only, better check.) */
-  ddsrt_mutex_lock (&e->m_observers_lock);
-  if (entity_has_status (e))
-    ddsrt_atomic_and32 (&e->m_status.m_status_and_mask, SAM_STATUS_MASK);
-
   ddsrt_mutex_unlock (&e->m_mutex);
 
-  /* wait for all listeners to complete - FIXME: rely on pincount instead?
-     that would require all listeners to pin the entity instead, but it
-     would prevent them from doing much. */
+  /* - Wait for listeners currently in-progress to complete their thing
+     - Reset all listeners so no new listener invocations will occur
+     - Wait for all pending ones ones to end as well */
+  ddsrt_mutex_lock (&e->m_observers_lock);
   while (e->m_cb_pending_count > 0)
     ddsrt_cond_wait (&e->m_observers_cond, &e->m_observers_lock);
+  dds_reset_listener (&e->m_listener);
   ddsrt_mutex_unlock (&e->m_observers_lock);
 
   /* Wait for all other threads to unpin the entity */
@@ -447,8 +456,18 @@ static dds_return_t really_delete_pinned_closed_locked (struct dds_entity *e, en
 
   /* Pin count dropped to one with CLOSING flag set: no other threads still
      in operations involving this entity */
-  dds_entity_observers_signal_delete (e);
+  if (dds_entity_kind (e) == DDS_KIND_WAITSET)
+  {
+    /* This takes care of the rare and deprecated case of a waitset attached
+       to itself. Those only ever get triggered by pinning the waitset, and
+       so this call should be safe, even if it isn't for, e.g., read
+       conditions which get signalled from deep inside the RHC code.  The
+       list is empty after this call, which turns the "normal" call after the
+       close() into a no-op. */
+    dds_entity_observers_signal_delete (e);
+  }
   dds_entity_deriver_close (e);
+  dds_entity_observers_signal_delete (e);
 
   /*
    * Recursively delete children.
@@ -609,9 +628,14 @@ dds_return_t dds_get_children (dds_entity_t entity, dds_entity_t *children, size
       /* Attempt at pinning the entity will fail if it is still pending */
       if (dds_entity_pin (c->m_hdllink.hdl, &tmp) == DDS_RETCODE_OK)
       {
-        if (n < size)
-          children[n] = c->m_hdllink.hdl;
-        n++;
+        /* Hide built-in topics to keep up the pretense that they are identified by their
+           pseudo handles and really do exist */
+        if (!entity_is_builtin_topic (tmp))
+        {
+          if (n < size)
+            children[n] = c->m_hdllink.hdl;
+          n++;
+        }
         dds_entity_unpin (tmp);
       }
     }
@@ -651,6 +675,18 @@ static uint64_t entity_kind_qos_mask (dds_entity_kind_t kind)
   return 0;
 }
 
+static dds_return_t dds_get_qos_builtin_topic (dds_qos_t *qos)
+{
+  // It is surprisingly hard to get the topic QoS used for built-in
+  // topics without having a reference to a domain. Some changes in
+  // this area might be a good idea.
+  dds_reset_qos (qos);
+  dds_qos_t *bq = dds__create_builtin_qos ();
+  ddsi_xqos_mergein_missing (qos, bq, DDS_TOPIC_QOS_MASK);
+  dds_delete_qos (bq);
+  return DDS_RETCODE_OK;
+}
+
 dds_return_t dds_get_qos (dds_entity_t entity, dds_qos_t *qos)
 {
   dds_entity *e;
@@ -660,7 +696,12 @@ dds_return_t dds_get_qos (dds_entity_t entity, dds_qos_t *qos)
     return DDS_RETCODE_BAD_PARAMETER;
 
   if ((ret = dds_entity_lock (entity, DDS_KIND_DONTCARE, &e)) != DDS_RETCODE_OK)
-    return ret;
+  {
+    if (dds__get_builtin_topic_name_typename (entity, NULL, NULL) == 0)
+      return dds_get_qos_builtin_topic (qos);
+    else
+      return ret;
+  }
 
   if (!dds_entity_supports_set_qos (e))
     ret = DDS_RETCODE_ILLEGAL_OPERATION;
@@ -679,7 +720,7 @@ dds_return_t dds_get_qos (dds_entity_t entity, dds_qos_t *qos)
     }
 
     dds_reset_qos (qos);
-    ddsi_xqos_mergein_missing (qos, entity_qos, ~(QP_TOPIC_NAME | QP_TYPE_NAME));
+    ddsi_xqos_mergein_missing (qos, entity_qos, ~(QP_TOPIC_NAME | QP_TYPE_NAME | QP_CYCLONE_TYPE_INFORMATION));
     ret = DDS_RETCODE_OK;
   }
   dds_entity_unlock(e);
@@ -940,41 +981,6 @@ dds_return_t dds_get_listener (dds_entity_t entity, dds_listener_t *listener)
   }
 }
 
-static void clear_status_with_listener (struct dds_entity *e)
-{
-  const struct dds_listener *lst = &e->m_listener;
-  status_mask_t mask = 0;
-  if (lst->on_inconsistent_topic)
-    mask |= DDS_INCONSISTENT_TOPIC_STATUS;
-  if (lst->on_liveliness_lost)
-    mask |= DDS_LIVELINESS_LOST_STATUS;
-  if (lst->on_offered_deadline_missed)
-    mask |= DDS_OFFERED_DEADLINE_MISSED_STATUS;
-  if (lst->on_offered_deadline_missed_arg)
-    mask |= DDS_OFFERED_DEADLINE_MISSED_STATUS;
-  if (lst->on_offered_incompatible_qos)
-    mask |= DDS_OFFERED_INCOMPATIBLE_QOS_STATUS;
-  if (lst->on_data_on_readers)
-    mask |= DDS_DATA_ON_READERS_STATUS;
-  if (lst->on_sample_lost)
-    mask |= DDS_SAMPLE_LOST_STATUS;
-  if (lst->on_data_available)
-    mask |= DDS_DATA_AVAILABLE_STATUS;
-  if (lst->on_sample_rejected)
-    mask |= DDS_SAMPLE_REJECTED_STATUS;
-  if (lst->on_liveliness_changed)
-    mask |= DDS_LIVELINESS_CHANGED_STATUS;
-  if (lst->on_requested_deadline_missed)
-    mask |= DDS_REQUESTED_DEADLINE_MISSED_STATUS;
-  if (lst->on_requested_incompatible_qos)
-    mask |= DDS_REQUESTED_INCOMPATIBLE_QOS_STATUS;
-  if (lst->on_publication_matched)
-    mask |= DDS_PUBLICATION_MATCHED_STATUS;
-  if (lst->on_subscription_matched)
-    mask |= DDS_SUBSCRIPTION_MATCHED_STATUS;
-  ddsrt_atomic_and32 (&e->m_status.m_status_and_mask, ~(uint32_t)mask);
-}
-
 static void pushdown_listener (dds_entity *e)
 {
   /* Note: e is claimed, no mutexes held */
@@ -997,7 +1003,6 @@ static void pushdown_listener (dds_entity *e)
       dds_override_inherited_listener (&c->m_listener, &e->m_listener);
       ddsrt_mutex_unlock (&e->m_observers_lock);
 
-      clear_status_with_listener (c);
       ddsrt_mutex_unlock (&c->m_observers_lock);
 
       pushdown_listener (c);
@@ -1035,7 +1040,6 @@ dds_return_t dds_set_listener (dds_entity_t entity, const dds_listener_t *listen
     dds_inherit_listener (&e->m_listener, &x->m_listener);
     ddsrt_mutex_unlock (&x->m_observers_lock);
   }
-  clear_status_with_listener (e);
   ddsrt_mutex_unlock (&e->m_observers_lock);
   pushdown_listener (e);
   dds_entity_unpin (e);
@@ -1060,29 +1064,6 @@ dds_return_t dds_enable (dds_entity_t entity)
   return DDS_RETCODE_OK;
 }
 
-dds_return_t dds_get_status_changes (dds_entity_t entity, uint32_t *status)
-{
-  dds_entity *e;
-  dds_return_t ret;
-
-  if (status == NULL)
-    return DDS_RETCODE_BAD_PARAMETER;
-
-  if ((ret = dds_entity_lock (entity, DDS_KIND_DONTCARE, &e)) != DDS_RETCODE_OK)
-    return ret;
-
-  if (!dds_entity_supports_validate_status (e))
-    ret = DDS_RETCODE_ILLEGAL_OPERATION;
-  else
-  {
-    assert (entity_has_status (e));
-    *status = ddsrt_atomic_ld32 (&e->m_status.m_status_and_mask) & SAM_STATUS_MASK;
-    ret = DDS_RETCODE_OK;
-  }
-  dds_entity_unlock(e);
-  return ret;
-}
-
 dds_return_t dds_get_status_mask (dds_entity_t entity, uint32_t *mask)
 {
   dds_entity *e;
@@ -1100,6 +1081,9 @@ dds_return_t dds_get_status_mask (dds_entity_t entity, uint32_t *mask)
   {
     assert (entity_has_status (e));
     *mask = ddsrt_atomic_ld32 (&e->m_status.m_status_and_mask) >> SAM_ENABLED_SHIFT;
+    // don't leak abuse of DATA_ON_READERS in readers' status masks
+    if (dds_entity_kind (e) == DDS_KIND_READER)
+      *mask &= ~(uint32_t)DDS_DATA_ON_READERS_STATUS;
     ret = DDS_RETCODE_OK;
   }
   dds_entity_unpin(e);
@@ -1119,15 +1103,12 @@ dds_return_t dds_set_status_mask (dds_entity_t entity, uint32_t mask)
   if ((mask & ~SAM_STATUS_MASK) != 0)
     return DDS_RETCODE_BAD_PARAMETER;
 
+  /* Lock rather than pin so this is can be done atomically with respect to dds_delete */
   if ((ret = dds_entity_lock (entity, DDS_KIND_DONTCARE, &e)) != DDS_RETCODE_OK)
     return ret;
 
   if (dds_handle_is_closed (&e->m_hdllink))
   {
-    /* The sole reason for locking the mutex was so we can do this atomically with
-       respect to dds_delete (which in turn requires checking whether the handle
-       is still open), because delete relies on the mask to shut down all listener
-       invocations.  */
     dds_entity_unlock (e);
     return DDS_RETCODE_PRECONDITION_NOT_MET;
   }
@@ -1139,10 +1120,14 @@ dds_return_t dds_set_status_mask (dds_entity_t entity, uint32_t mask)
     while (e->m_cb_pending_count > 0)
       ddsrt_cond_wait (&e->m_observers_cond, &e->m_observers_lock);
 
+    // readers: don't touch DATA_ON_READERS_STATUS in mask
+    if (dds_entity_kind (e) == DDS_KIND_READER)
+      mask |= DDS_DATA_ON_READERS_STATUS;
     uint32_t old, new;
     do {
       old = ddsrt_atomic_ld32 (&e->m_status.m_status_and_mask);
-      new = (mask << SAM_ENABLED_SHIFT) | (old & mask);
+      assert (!(old & DDS_DATA_ON_READERS_STATUS) || dds_entity_kind (e) != DDS_KIND_READER);
+      new = (mask << SAM_ENABLED_SHIFT) | (old & SAM_STATUS_MASK);
     } while (!ddsrt_atomic_cas32 (&e->m_status.m_status_and_mask, old, new));
     ddsrt_mutex_unlock (&e->m_observers_lock);
   }
@@ -1172,10 +1157,28 @@ static dds_return_t dds_readtake_status (dds_entity_t entity, uint32_t *status,
   {
     uint32_t s;
     assert (entity_has_status (e));
+    if (mask == 0)
+      mask = SAM_STATUS_MASK;
     if (reset)
       s = ddsrt_atomic_and32_ov (&e->m_status.m_status_and_mask, ~mask) & mask;
     else
       s = ddsrt_atomic_ld32 (&e->m_status.m_status_and_mask) & mask;
+
+    // non-materialized DATA_ON_READERS requires a fix-up
+    if (dds_entity_kind (e) == DDS_KIND_SUBSCRIBER)
+    {
+      dds_subscriber * const sub = (dds_subscriber *) e;
+      ddsrt_mutex_lock (&sub->m_entity.m_observers_lock);
+      if (!(sub->materialize_data_on_readers & DDS_SUB_MATERIALIZE_DATA_ON_READERS_FLAG))
+      {
+        if (dds_subscriber_compute_data_on_readers_locked (sub))
+          s |= DDS_DATA_ON_READERS_STATUS;
+        else
+          s &= ~(uint32_t)DDS_DATA_ON_READERS_STATUS;
+      }
+      ddsrt_mutex_unlock (&sub->m_entity.m_observers_lock);
+    }
+
     *status = s;
   }
   dds_entity_unlock (e);
@@ -1192,6 +1195,11 @@ dds_return_t dds_take_status (dds_entity_t entity, uint32_t *status, uint32_t ma
   return dds_readtake_status (entity, status, mask, true);
 }
 
+dds_return_t dds_get_status_changes (dds_entity_t entity, uint32_t *status)
+{
+  return dds_read_status (entity, status, 0);
+}
+
 dds_return_t dds_get_domainid (dds_entity_t entity, dds_domainid_t *id)
 {
   dds_entity *e;
@@ -1237,7 +1245,8 @@ dds_return_t dds_get_guid (dds_entity_t entity, dds_guid_t *guid)
   {
     case DDS_KIND_PARTICIPANT:
     case DDS_KIND_READER:
-    case DDS_KIND_WRITER: {
+    case DDS_KIND_WRITER:
+    case DDS_KIND_TOPIC: {
       DDSRT_STATIC_ASSERT (sizeof (dds_guid_t) == sizeof (ddsi_guid_t));
       ddsi_guid_t tmp = nn_ntoh_guid (e->m_guid);
       memcpy (guid, &tmp, sizeof (*guid));
@@ -1253,11 +1262,11 @@ dds_return_t dds_get_guid (dds_entity_t entity, dds_guid_t *guid)
   return ret;
 }
 
-dds_return_t dds_entity_pin (dds_entity_t hdl, dds_entity **eptr)
+dds_return_t dds_entity_pin_with_origin (dds_entity_t hdl, bool from_user, dds_entity **eptr)
 {
   dds_return_t hres;
   struct dds_handle_link *hdllink;
-  if ((hres = dds_handle_pin (hdl, &hdllink)) < 0)
+  if ((hres = dds_handle_pin_with_origin (hdl, from_user, &hdllink)) < 0)
     return hres;
   else
   {
@@ -1266,11 +1275,16 @@ dds_return_t dds_entity_pin (dds_entity_t hdl, dds_entity **eptr)
   }
 }
 
-dds_return_t dds_entity_pin_for_delete (dds_entity_t hdl, bool explicit, dds_entity **eptr)
+dds_return_t dds_entity_pin (dds_entity_t hdl, dds_entity **eptr)
+{
+  return dds_entity_pin_with_origin (hdl, true, eptr);
+}
+
+dds_return_t dds_entity_pin_for_delete (dds_entity_t hdl, bool explicit, bool from_user, dds_entity **eptr)
 {
   dds_return_t hres;
   struct dds_handle_link *hdllink;
-  if ((hres = dds_handle_pin_for_delete (hdl, explicit, &hdllink)) < 0)
+  if ((hres = dds_handle_pin_for_delete (hdl, explicit, from_user, &hdllink)) < 0)
     return hres;
   else
   {
@@ -1317,10 +1331,13 @@ dds_return_t dds_triggered (dds_entity_t entity)
 
   if ((ret = dds_entity_lock (entity, DDS_KIND_DONTCARE, &e)) != DDS_RETCODE_OK)
     return ret;
-  if (entity_has_status (e))
-    ret = ((ddsrt_atomic_ld32 (&e->m_status.m_status_and_mask) & SAM_STATUS_MASK) != 0);
-  else
+  if (!entity_has_status (e))
     ret = (ddsrt_atomic_ld32 (&e->m_status.m_trigger) != 0);
+  else
+  {
+    const uint32_t sm = ddsrt_atomic_ld32 (&e->m_status.m_status_and_mask);
+    ret = ((sm & (sm >> SAM_ENABLED_SHIFT)) != 0);
+  }
   dds_entity_unlock (e);
   return ret;
 }
@@ -1387,7 +1404,7 @@ dds_return_t dds_entity_observer_unregister (dds_entity *observed, dds_waitset *
   return rc;
 }
 
-static void dds_entity_observers_signal (dds_entity *observed, uint32_t status)
+void dds_entity_observers_signal (dds_entity *observed, uint32_t status)
 {
   for (dds_entity_observer *idx = observed->m_observers; idx; idx = idx->m_next)
     idx->m_cb (idx->m_observer, observed->m_hdllink.hdl, status);
@@ -1414,32 +1431,17 @@ void dds_entity_status_signal (dds_entity *e, uint32_t status)
   ddsrt_mutex_unlock (&e->m_observers_lock);
 }
 
-void dds_entity_status_set (dds_entity *e, status_mask_t status)
+bool dds_entity_status_set (dds_entity *e, status_mask_t status)
 {
+  // returns true if waitsets need triggering
   assert (entity_has_status (e));
-  uint32_t old, delta, new;
-  do {
-    old = ddsrt_atomic_ld32 (&e->m_status.m_status_and_mask);
-    delta = ((uint32_t) status & (old >> SAM_ENABLED_SHIFT));
-    if (delta == 0)
-      return;
-    new = old | delta;
-  } while (!ddsrt_atomic_cas32 (&e->m_status.m_status_and_mask, old, new));
-  if (delta)
-    dds_entity_observers_signal (e, status);
-}
-
-void dds_entity_trigger_set (dds_entity *e, uint32_t t)
-{
-  assert (! entity_has_status (e));
-  uint32_t oldst;
-  ddsrt_mutex_lock (&e->m_observers_lock);
-  do {
-    oldst = ddsrt_atomic_ld32 (&e->m_status.m_trigger);
-  } while (!ddsrt_atomic_cas32 (&e->m_status.m_trigger, oldst, t));
-  if (oldst == 0 && t != 0)
-    dds_entity_observers_signal (e, t);
-  ddsrt_mutex_unlock (&e->m_observers_lock);
+  uint32_t old = ddsrt_atomic_or32_ov (&e->m_status.m_status_and_mask, status);
+  if (old & status)
+    return false; // already set, no need to trigger waitsets
+  else if (!(status & (old >> SAM_ENABLED_SHIFT)))
+    return false; // masked
+  else
+    return true;
 }
 
 dds_entity_t dds_get_topic (dds_entity_t entity)
@@ -1454,11 +1456,13 @@ dds_entity_t dds_get_topic (dds_entity_t entity)
   {
     case DDS_KIND_READER: {
       dds_reader *rd = (dds_reader *) e;
-      hdl = rd->m_topic->m_entity.m_hdllink.hdl;
+      if ((hdl = dds__get_builtin_topic_pseudo_handle_from_typename (rd->m_topic->m_stype->type_name)) < 0)
+        hdl = rd->m_topic->m_entity.m_hdllink.hdl;
       break;
     }
     case DDS_KIND_WRITER: {
       dds_writer *wr = (dds_writer *) e;
+      assert (dds__get_builtin_topic_pseudo_handle_from_typename (wr->m_topic->m_stype->type_name) < 0);
       hdl = wr->m_topic->m_entity.m_hdllink.hdl;
       break;
     }
@@ -1466,7 +1470,8 @@ dds_entity_t dds_get_topic (dds_entity_t entity)
     case DDS_KIND_COND_QUERY: {
       assert (dds_entity_kind (e->m_parent) == DDS_KIND_READER);
       dds_reader *rd = (dds_reader *) e->m_parent;
-      hdl = rd->m_topic->m_entity.m_hdllink.hdl;
+      if ((hdl = dds__get_builtin_topic_pseudo_handle_from_typename (rd->m_topic->m_stype->type_name)) < 0)
+        hdl = rd->m_topic->m_entity.m_hdllink.hdl;
       break;
     }
     default: {
@@ -1536,3 +1541,61 @@ dds_return_t dds_assert_liveliness (dds_entity_t entity)
   dds_entity_unpin (e);
   return rc;
 }
+
+dds_return_t dds_return_loan (dds_entity_t entity, void **buf, int32_t bufsz)
+{
+  dds_entity *p_entity;
+  dds_return_t ret;
+
+  // bufsz <= 0 is accepted because it allows one to write:
+  //
+  // if (dds_return_loan(rd, buf, dds_take(rd, buf, ...)) < 0)
+  //   abort();
+  //
+  // with abort only being called if there is a real problem.
+  //
+  // The wisdom of such code may be debatable, but it has been allowed for a long
+  // time and changing it may well break existing application code.
+  if (buf == NULL || (bufsz > 0 && buf[0] == NULL))
+    return DDS_RETCODE_BAD_PARAMETER;
+
+  if ((ret = dds_entity_pin (entity, &p_entity)) < 0)
+    return ret;
+
+  switch (dds_entity_kind (p_entity))
+  {
+    case DDS_KIND_READER: {
+      dds_reader *rd = (dds_reader *) p_entity;
+      ret = dds_return_reader_loan (rd, buf, bufsz);
+      break;
+    }
+    case DDS_KIND_COND_READ:
+    case DDS_KIND_COND_QUERY: {
+      dds_readcond *rdcond = (dds_readcond *) p_entity;
+      dds_reader *rd = (dds_reader *) rdcond->m_entity.m_parent;
+      assert (dds_entity_kind (&rd->m_entity) == DDS_KIND_READER);
+      ret = dds_return_reader_loan (rd, buf, bufsz);
+      break;
+    }
+    case DDS_KIND_WRITER: {
+      dds_writer *wr = (dds_writer *) p_entity;
+      ret = dds_return_writer_loan (wr, buf, bufsz);
+      break;
+    }
+    case DDS_KIND_DONTCARE:
+    case DDS_KIND_CYCLONEDDS:
+    case DDS_KIND_DOMAIN:
+    case DDS_KIND_WAITSET:
+    case DDS_KIND_COND_GUARD:
+    case DDS_KIND_PARTICIPANT:
+    case DDS_KIND_TOPIC:
+    case DDS_KIND_PUBLISHER:
+    case DDS_KIND_SUBSCRIBER: {
+      ret = DDS_RETCODE_ILLEGAL_OPERATION;
+      break;
+    }
+  }
+  dds_entity_unpin (p_entity);
+  return ret;
+}
+
