@@ -14,6 +14,7 @@
 #include "dds/dds.h"
 #include "dds/version.h"
 #include "dds/ddsrt/static_assert.h"
+#include "dds/ddsrt/heap.h"
 #include "dds__participant.h"
 #include "dds__subscriber.h"
 #include "dds__reader.h"
@@ -29,12 +30,24 @@
 #include "dds/ddsi/ddsi_domaingv.h"
 #include "dds__builtin.h"
 #include "dds__statistics.h"
-#include "dds/ddsi/ddsi_sertopic.h"
+#include "dds__data_allocator.h"
+#include "dds/ddsi/ddsi_sertype.h"
 #include "dds/ddsi/ddsi_entity_index.h"
 #include "dds/ddsi/ddsi_security_omg.h"
 #include "dds/ddsi/ddsi_statistics.h"
 
-DECL_ENTITY_LOCK_UNLOCK (extern inline, dds_reader)
+#ifdef DDS_HAS_SHM
+#include "shm__monitor.h"
+#include "dds/ddsi/q_receive.h"
+#include "dds/ddsi/ddsi_tkmap.h"
+#include "iceoryx_binding_c/wait_set.h"
+#include "dds/ddsrt/threads.h"
+#include "dds/ddsrt/sync.h"
+#include "dds/ddsrt/md5.h"
+#include "dds/ddsi/shm_sync.h"
+#endif
+
+DECL_ENTITY_LOCK_UNLOCK (dds_reader)
 
 #define DDS_READER_STATUS_MASK                                   \
                         (DDS_SAMPLE_REJECTED_STATUS              |\
@@ -52,6 +65,15 @@ static void dds_reader_close (dds_entity *e)
   struct dds_reader * const rd = (struct dds_reader *) e;
   assert (rd->m_rd != NULL);
 
+#ifdef DDS_HAS_SHM
+  if (rd->m_iox_sub)
+  {
+  //will wait for any runing callback using the iceoryx subscriber of this reader
+    shm_monitor_detach_reader(&rd->m_entity.m_domain->m_shm_monitor, rd);
+  //from now on no callbacks on this reader will run
+  }
+#endif
+
   thread_state_awake (lookup_thread_state (), &e->m_domain->gv);
   (void) delete_reader (&e->m_domain->gv, &e->m_guid);
   thread_state_asleep (lookup_thread_state ());
@@ -67,17 +89,37 @@ static dds_return_t dds_reader_delete (dds_entity *e) ddsrt_nonnull_all;
 static dds_return_t dds_reader_delete (dds_entity *e)
 {
   dds_reader * const rd = (dds_reader *) e;
-  dds_free (rd->m_loan);
+
+  if (rd->m_loan)
+  {
+    void **ptrs = ddsrt_malloc (rd->m_loan_size * sizeof (*ptrs));
+    ddsi_sertype_realloc_samples (ptrs, rd->m_topic->m_stype, rd->m_loan, rd->m_loan_size, rd->m_loan_size);
+    ddsi_sertype_free_samples (rd->m_topic->m_stype, ptrs, rd->m_loan_size, DDS_FREE_ALL);
+    ddsrt_free (ptrs);
+  }
+
   thread_state_awake (lookup_thread_state (), &e->m_domain->gv);
   dds_rhc_free (rd->m_rhc);
   thread_state_asleep (lookup_thread_state ());
+
+#ifdef DDS_HAS_SHM
+  if (rd->m_iox_sub)
+  {
+    // deletion must happen at the very end after the reader cache is not used anymore
+    // since the mutex is needed and the data needs to be released using the iceoryx subscriber
+    DDS_CLOG (DDS_LC_SHM, &e->m_domain->gv.logconfig, "Release iceoryx's subscriber\n");
+    iox_sub_deinit(rd->m_iox_sub);
+    iox_sub_storage_extension_fini(&rd->m_iox_sub_stor);
+  }
+#endif
+
   dds_entity_drop_ref (&rd->m_topic->m_entity);
   return DDS_RETCODE_OK;
 }
 
 static dds_return_t validate_reader_qos (const dds_qos_t *rqos)
 {
-#ifndef DDSI_INCLUDE_DEADLINE_MISSED
+#ifndef DDS_HAS_DEADLINE_MISSED
   if (rqos != NULL && (rqos->present & QP_DEADLINE) && rqos->deadline.deadline != DDS_INFINITY)
     return DDS_RETCODE_BAD_PARAMETER;
 #else
@@ -108,6 +150,71 @@ static dds_return_t dds_reader_status_validate (uint32_t mask)
   return (mask & ~DDS_READER_STATUS_MASK) ? DDS_RETCODE_BAD_PARAMETER : DDS_RETCODE_OK;
 }
 
+static void data_avail_cb_enter_listener_exclusive_access (dds_entity *e)
+{
+  // assumes e->m_observers_lock held on entry
+  // possibly unlocks and relocks e->m_observers_lock
+  // afterward e->m_listener is stable
+  e->m_cb_pending_count++;
+  while (e->m_cb_count > 0)
+    ddsrt_cond_wait (&e->m_observers_cond, &e->m_observers_lock);
+  e->m_cb_count++;
+}
+
+static void data_avail_cb_leave_listener_exclusive_access (dds_entity *e)
+{
+  // assumes e->m_observers_lock held on entry
+  e->m_cb_count--;
+  e->m_cb_pending_count--;
+  ddsrt_cond_broadcast (&e->m_observers_cond);
+}
+
+static void data_avail_cb_invoke_dor (dds_entity *sub, const struct dds_listener *lst)
+{
+  // assumes sub->m_observers_lock held on entry
+  // unlocks and relocks sub->m_observers_lock
+  data_avail_cb_enter_listener_exclusive_access (sub);
+  ddsrt_mutex_unlock (&sub->m_observers_lock);
+  lst->on_data_on_readers (sub->m_hdllink.hdl, lst->on_data_on_readers_arg);
+  ddsrt_mutex_lock (&sub->m_observers_lock);
+  data_avail_cb_leave_listener_exclusive_access (sub);
+}
+
+static uint32_t data_avail_cb_set_status (dds_entity *rd, uint32_t status_and_mask)
+{
+  uint32_t ret = 0;
+  if (dds_entity_status_set (rd, DDS_DATA_AVAILABLE_STATUS))
+    ret |= DDS_DATA_AVAILABLE_STATUS;
+  if (status_and_mask & (DDS_DATA_ON_READERS_STATUS << SAM_ENABLED_SHIFT))
+  {
+    if (dds_entity_status_set (rd->m_parent, DDS_DATA_ON_READERS_STATUS))
+      ret |= DDS_DATA_ON_READERS_STATUS;
+  }
+  return ret;
+}
+
+static void data_avail_cb_trigger_waitsets (dds_entity *rd, uint32_t signal)
+{
+  if (signal == 0)
+    return;
+
+  if (signal & DDS_DATA_ON_READERS_STATUS)
+  {
+    dds_entity * const sub = rd->m_parent;
+    ddsrt_mutex_lock (&sub->m_observers_lock);
+    const uint32_t sm = ddsrt_atomic_ld32 (&sub->m_status.m_status_and_mask);
+    if ((sm & (sm >> SAM_ENABLED_SHIFT)) & DDS_DATA_ON_READERS_STATUS)
+      dds_entity_observers_signal (sub, DDS_DATA_ON_READERS_STATUS);
+    ddsrt_mutex_unlock (&sub->m_observers_lock);
+  }
+  if (signal & DDS_DATA_AVAILABLE_STATUS)
+  {
+    const uint32_t sm = ddsrt_atomic_ld32 (&rd->m_status.m_status_and_mask);
+    if ((sm & (sm >> SAM_ENABLED_SHIFT)) & DDS_DATA_AVAILABLE_STATUS)
+      dds_entity_observers_signal (rd, DDS_DATA_AVAILABLE_STATUS);
+  }
+}
+
 void dds_reader_data_available_cb (struct dds_reader *rd)
 {
   /* DATA_AVAILABLE is special in two ways: firstly, it should first try
@@ -115,66 +222,44 @@ void dds_reader_data_available_cb (struct dds_reader *rd)
      status on the subscriber; secondly it is the only one for which
      overhead really matters.  Otherwise, it is pretty much like
      dds_reader_status_cb. */
-
-  const uint32_t data_av_enabled = (ddsrt_atomic_ld32 (&rd->m_entity.m_status.m_status_and_mask) & (DDS_DATA_AVAILABLE_STATUS << SAM_ENABLED_SHIFT));
-  if (data_av_enabled == 0)
-    return;
+  struct dds_listener const * const lst = &rd->m_entity.m_listener;
+  uint32_t signal = 0;
 
   ddsrt_mutex_lock (&rd->m_entity.m_observers_lock);
-  rd->m_entity.m_cb_pending_count++;
-
-  /* FIXME: why wait if no listener is set? */
-  while (rd->m_entity.m_cb_count > 0)
-    ddsrt_cond_wait (&rd->m_entity.m_observers_cond, &rd->m_entity.m_observers_lock);
-  rd->m_entity.m_cb_count++;
-
-  struct dds_listener const * const lst = &rd->m_entity.m_listener;
-  dds_entity * const sub = rd->m_entity.m_parent;
-  if (lst->on_data_on_readers)
-  {
-    ddsrt_mutex_unlock (&rd->m_entity.m_observers_lock);
-    ddsrt_mutex_lock (&sub->m_observers_lock);
-    const uint32_t data_on_rds_enabled = (ddsrt_atomic_ld32 (&sub->m_status.m_status_and_mask) & (DDS_DATA_ON_READERS_STATUS << SAM_ENABLED_SHIFT));
-    if (data_on_rds_enabled)
-    {
-      sub->m_cb_pending_count++;
-      while (sub->m_cb_count > 0)
-        ddsrt_cond_wait (&sub->m_observers_cond, &sub->m_observers_lock);
-      sub->m_cb_count++;
-      ddsrt_mutex_unlock (&sub->m_observers_lock);
-
-      lst->on_data_on_readers (sub->m_hdllink.hdl, lst->on_data_on_readers_arg);
-
-      ddsrt_mutex_lock (&sub->m_observers_lock);
-      sub->m_cb_count--;
-      sub->m_cb_pending_count--;
-      ddsrt_cond_broadcast (&sub->m_observers_cond);
-    }
-    ddsrt_mutex_unlock (&sub->m_observers_lock);
-    ddsrt_mutex_lock (&rd->m_entity.m_observers_lock);
-  }
-  else if (rd->m_entity.m_listener.on_data_available)
-  {
-    ddsrt_mutex_unlock (&rd->m_entity.m_observers_lock);
-    lst->on_data_available (rd->m_entity.m_hdllink.hdl, lst->on_data_available_arg);
-    ddsrt_mutex_lock (&rd->m_entity.m_observers_lock);
-  }
+  const uint32_t status_and_mask = ddsrt_atomic_ld32 (&rd->m_entity.m_status.m_status_and_mask);
+  if (lst->on_data_on_readers == 0 && lst->on_data_available == 0)
+    signal = data_avail_cb_set_status (&rd->m_entity, status_and_mask);
   else
   {
-    dds_entity_status_set (&rd->m_entity, DDS_DATA_AVAILABLE_STATUS);
-    ddsrt_mutex_lock (&sub->m_observers_lock);
-    dds_entity_status_set (sub, DDS_DATA_ON_READERS_STATUS);
-    ddsrt_mutex_unlock (&sub->m_observers_lock);
+    // "lock" listener object so we can look at "lst" without holding m_observers_lock
+    data_avail_cb_enter_listener_exclusive_access (&rd->m_entity);
+    if (lst->on_data_on_readers)
+    {
+      dds_entity * const sub = rd->m_entity.m_parent;
+      ddsrt_mutex_unlock (&rd->m_entity.m_observers_lock);
+      ddsrt_mutex_lock (&sub->m_observers_lock);
+      if (!(lst->reset_on_invoke & DDS_DATA_ON_READERS_STATUS))
+        signal = data_avail_cb_set_status (&rd->m_entity, status_and_mask);
+      data_avail_cb_invoke_dor (sub, lst);
+      ddsrt_mutex_unlock (&sub->m_observers_lock);
+      ddsrt_mutex_lock (&rd->m_entity.m_observers_lock);
+    }
+    else
+    {
+      assert (rd->m_entity.m_listener.on_data_available);
+      if (!(lst->reset_on_invoke & DDS_DATA_AVAILABLE_STATUS))
+        signal = data_avail_cb_set_status (&rd->m_entity, status_and_mask);
+      ddsrt_mutex_unlock (&rd->m_entity.m_observers_lock);
+      lst->on_data_available (rd->m_entity.m_hdllink.hdl, lst->on_data_available_arg);
+      ddsrt_mutex_lock (&rd->m_entity.m_observers_lock);
+    }
+    data_avail_cb_leave_listener_exclusive_access (&rd->m_entity);
   }
-
-  rd->m_entity.m_cb_count--;
-  rd->m_entity.m_cb_pending_count--;
-
-  ddsrt_cond_broadcast (&rd->m_entity.m_observers_cond);
+  data_avail_cb_trigger_waitsets (&rd->m_entity, signal);
   ddsrt_mutex_unlock (&rd->m_entity.m_observers_lock);
 }
 
-static void update_requested_deadline_missed (struct dds_requested_deadline_missed_status * __restrict st, struct dds_requested_deadline_missed_status * __restrict lst, const status_cb_data_t *data)
+static void update_requested_deadline_missed (struct dds_requested_deadline_missed_status * __restrict st, const status_cb_data_t *data)
 {
   st->last_instance_handle = data->handle;
   st->total_count++;
@@ -184,51 +269,31 @@ static void update_requested_deadline_missed (struct dds_requested_deadline_miss
   //
   // (same line of reasoning for all of them)
   st->total_count_change++;
-  if (lst != NULL)
-  {
-    *lst = *st;
-    st->total_count_change = 0;
-  }
 }
 
-static void update_requested_incompatible_qos (struct dds_requested_incompatible_qos_status * __restrict st, struct dds_requested_incompatible_qos_status * __restrict lst, const status_cb_data_t *data)
+static void update_requested_incompatible_qos (struct dds_requested_incompatible_qos_status * __restrict st, const status_cb_data_t *data)
 {
   st->last_policy_id = data->extra;
   st->total_count++;
   st->total_count_change++;
-  if (lst != NULL)
-  {
-    *lst = *st;
-    st->total_count_change = 0;
-  }
 }
 
-static void update_sample_lost (struct dds_sample_lost_status * __restrict st, struct dds_sample_lost_status * __restrict lst, const status_cb_data_t *data)
+static void update_sample_lost (struct dds_sample_lost_status * __restrict st, const status_cb_data_t *data)
 {
   (void) data;
   st->total_count++;
   st->total_count_change++;
-  if (lst != NULL)
-  {
-    *lst = *st;
-    st->total_count_change = 0;
-  }
 }
 
-static void update_sample_rejected (struct dds_sample_rejected_status * __restrict st, struct dds_sample_rejected_status * __restrict lst, const status_cb_data_t *data)
+static void update_sample_rejected (struct dds_sample_rejected_status * __restrict st, const status_cb_data_t *data)
 {
   st->last_reason = data->extra;
   st->last_instance_handle = data->handle;
   st->total_count++;
   st->total_count_change++;
-  if (lst != NULL)
-  {
-    *lst = *st;
-    st->total_count_change = 0;
-  }
 }
 
-static void update_liveliness_changed (struct dds_liveliness_changed_status * __restrict st, struct dds_liveliness_changed_status * __restrict lst, const status_cb_data_t *data)
+static void update_liveliness_changed (struct dds_liveliness_changed_status * __restrict st, const status_cb_data_t *data)
 {
   DDSRT_STATIC_ASSERT ((uint32_t) LIVELINESS_CHANGED_ADD_ALIVE == 0 &&
                        LIVELINESS_CHANGED_ADD_ALIVE < LIVELINESS_CHANGED_ADD_NOT_ALIVE &&
@@ -270,15 +335,9 @@ static void update_liveliness_changed (struct dds_liveliness_changed_status * __
       st->alive_count_change++;
       break;
   }
-  if (lst != NULL)
-  {
-    *lst = *st;
-    st->alive_count_change = 0;
-    st->not_alive_count_change = 0;
-  }
 }
 
-static void update_subscription_matched (struct dds_subscription_matched_status * __restrict st, struct dds_subscription_matched_status * __restrict lst, const status_cb_data_t *data)
+static void update_subscription_matched (struct dds_subscription_matched_status * __restrict st, const status_cb_data_t *data)
 {
   st->last_publication_handle = data->handle;
   if (data->add) {
@@ -290,20 +349,24 @@ static void update_subscription_matched (struct dds_subscription_matched_status
     st->current_count--;
     st->current_count_change--;
   }
-  if (lst != NULL)
-  {
-    *lst = *st;
-    st->total_count_change = 0;
-    st->current_count_change = 0;
-  }
 }
 
-STATUS_CB_IMPL (reader, requested_deadline_missed, REQUESTED_DEADLINE_MISSED)
-STATUS_CB_IMPL (reader, requested_incompatible_qos, REQUESTED_INCOMPATIBLE_QOS)
-STATUS_CB_IMPL (reader, sample_lost, SAMPLE_LOST)
-STATUS_CB_IMPL (reader, sample_rejected, SAMPLE_REJECTED)
-STATUS_CB_IMPL (reader, liveliness_changed, LIVELINESS_CHANGED)
-STATUS_CB_IMPL (reader, subscription_matched, SUBSCRIPTION_MATCHED)
+/* Reset sets everything (type) 0, including the reason field, verify that 0 is correct */
+DDSRT_STATIC_ASSERT ((int) DDS_NOT_REJECTED == 0);
+
+DDS_GET_STATUS (reader, subscription_matched,       SUBSCRIPTION_MATCHED,       total_count_change, current_count_change)
+DDS_GET_STATUS (reader, liveliness_changed,         LIVELINESS_CHANGED,         alive_count_change, not_alive_count_change)
+DDS_GET_STATUS (reader, sample_rejected,            SAMPLE_REJECTED,            total_count_change)
+DDS_GET_STATUS (reader, sample_lost,                SAMPLE_LOST,                total_count_change)
+DDS_GET_STATUS (reader, requested_deadline_missed,  REQUESTED_DEADLINE_MISSED,  total_count_change)
+DDS_GET_STATUS (reader, requested_incompatible_qos, REQUESTED_INCOMPATIBLE_QOS, total_count_change)
+
+STATUS_CB_IMPL (reader, subscription_matched,       SUBSCRIPTION_MATCHED,       total_count_change, current_count_change)
+STATUS_CB_IMPL (reader, liveliness_changed,         LIVELINESS_CHANGED,         alive_count_change, not_alive_count_change)
+STATUS_CB_IMPL (reader, sample_rejected,            SAMPLE_REJECTED,            total_count_change)
+STATUS_CB_IMPL (reader, sample_lost,                SAMPLE_LOST,                total_count_change)
+STATUS_CB_IMPL (reader, requested_deadline_missed,  REQUESTED_DEADLINE_MISSED,  total_count_change)
+STATUS_CB_IMPL (reader, requested_incompatible_qos, REQUESTED_INCOMPATIBLE_QOS, total_count_change)
 
 void dds_reader_status_cb (void *ventity, const status_cb_data_t *data)
 {
@@ -331,30 +394,31 @@ void dds_reader_status_cb (void *ventity, const status_cb_data_t *data)
      are stable */
   /* FIXME: why do this if no listener is set? */
   ddsrt_mutex_lock (&rd->m_entity.m_observers_lock);
+  rd->m_entity.m_cb_pending_count++;
   while (rd->m_entity.m_cb_count > 0)
     ddsrt_cond_wait (&rd->m_entity.m_observers_cond, &rd->m_entity.m_observers_lock);
+  rd->m_entity.m_cb_count++;
 
   const enum dds_status_id status_id = (enum dds_status_id) data->raw_status_id;
-  const bool enabled = (ddsrt_atomic_ld32 (&rd->m_entity.m_status.m_status_and_mask) & ((1u << status_id) << SAM_ENABLED_SHIFT)) != 0;
   switch (status_id)
   {
     case DDS_REQUESTED_DEADLINE_MISSED_STATUS_ID:
-      status_cb_requested_deadline_missed (rd, data, enabled);
+      status_cb_requested_deadline_missed (rd, data);
       break;
     case DDS_REQUESTED_INCOMPATIBLE_QOS_STATUS_ID:
-      status_cb_requested_incompatible_qos (rd, data, enabled);
+      status_cb_requested_incompatible_qos (rd, data);
       break;
     case DDS_SAMPLE_LOST_STATUS_ID:
-      status_cb_sample_lost (rd, data, enabled);
+      status_cb_sample_lost (rd, data);
       break;
     case DDS_SAMPLE_REJECTED_STATUS_ID:
-      status_cb_sample_rejected (rd, data, enabled);
+      status_cb_sample_rejected (rd, data);
       break;
     case DDS_LIVELINESS_CHANGED_STATUS_ID:
-      status_cb_liveliness_changed (rd, data, enabled);
+      status_cb_liveliness_changed (rd, data);
       break;
     case DDS_SUBSCRIPTION_MATCHED_STATUS_ID:
-      status_cb_subscription_matched (rd, data, enabled);
+      status_cb_subscription_matched (rd, data);
       break;
     case DDS_DATA_ON_READERS_STATUS_ID:
     case DDS_DATA_AVAILABLE_STATUS_ID:
@@ -366,6 +430,8 @@ void dds_reader_status_cb (void *ventity, const status_cb_data_t *data)
       assert (0);
   }
 
+  rd->m_entity.m_cb_count--;
+  rd->m_entity.m_cb_pending_count--;
   ddsrt_cond_broadcast (&rd->m_entity.m_observers_cond);
   ddsrt_mutex_unlock (&rd->m_entity.m_observers_lock);
 }
@@ -401,6 +467,29 @@ const struct dds_entity_deriver dds_entity_deriver_reader = {
   .refresh_statistics = dds_reader_refresh_statistics
 };
 
+#ifdef DDS_HAS_SHM
+#define DDS_READER_QOS_CHECK_FIELDS (QP_LIVELINESS|QP_DEADLINE|QP_RELIABILITY|QP_DURABILITY|QP_HISTORY)
+static bool dds_reader_support_shm(const struct ddsi_config* cfg, const dds_qos_t *qos, const struct dds_topic *tp)
+{
+  if (NULL == cfg ||
+      false == cfg->enable_shm)
+    return false;
+
+  if (!tp->m_stype->fixed_size)
+    return false;
+
+  uint32_t sub_history_req = cfg->sub_history_request;
+
+  return (NULL != qos &&
+    DDS_READER_QOS_CHECK_FIELDS == (qos->present & DDS_READER_QOS_CHECK_FIELDS) &&
+    DDS_LIVELINESS_AUTOMATIC == qos->liveliness.kind &&
+    DDS_INFINITY == qos->deadline.deadline &&
+    DDS_DURABILITY_VOLATILE == qos->durability.kind &&
+    DDS_HISTORY_KEEP_LAST == qos->history.kind &&
+    (int)sub_history_req >= (int)qos->history.depth);
+}
+#endif
+
 static dds_entity_t dds_create_reader_int (dds_entity_t participant_or_subscriber, dds_entity_t topic, const dds_qos_t *qos, const dds_listener_t *listener, struct dds_rhc *rhc)
 {
   dds_qos_t *rqos;
@@ -414,9 +503,9 @@ static dds_entity_t dds_create_reader_int (dds_entity_t participant_or_subscribe
   switch (topic)
   {
     case DDS_BUILTIN_TOPIC_DCPSTOPIC:
-      /* not implemented yet */
-      return DDS_RETCODE_BAD_PARAMETER;
-
+#ifndef DDS_HAS_TOPIC_DISCOVERY
+      return DDS_RETCODE_UNSUPPORTED;
+#endif
     case DDS_BUILTIN_TOPIC_DCPSPARTICIPANT:
     case DDS_BUILTIN_TOPIC_DCPSPUBLICATION:
     case DDS_BUILTIN_TOPIC_DCPSSUBSCRIPTION:
@@ -454,9 +543,11 @@ static dds_entity_t dds_create_reader_int (dds_entity_t participant_or_subscribe
     }
   }
 
-  if ((rc = dds_topic_pin (topic, &tp)) < 0)
+  /* If pseudo_topic != 0, topic didn't didn't originate from the application and we allow pinning
+     it despite it being marked as NO_USER_ACCESS */
+  if ((rc = dds_topic_pin_with_origin (topic, pseudo_topic ? false : true, &tp)) < 0)
     goto err_pin_topic;
-  assert (tp->m_stopic);
+  assert (tp->m_stype);
   if (dds_entity_participant (&sub->m_entity) != dds_entity_participant (&tp->m_entity))
   {
     rc = DDS_RETCODE_BAD_PARAMETER;
@@ -483,7 +574,7 @@ static dds_entity_t dds_create_reader_int (dds_entity_t participant_or_subscribe
     ddsi_xqos_mergein_missing (rqos, sub->m_entity.m_qos, ~(uint64_t)0);
   if (tp->m_ktopic->qos)
     ddsi_xqos_mergein_missing (rqos, tp->m_ktopic->qos, ~(uint64_t)0);
-  ddsi_xqos_mergein_missing (rqos, &gv->default_xqos_rd, ~(uint64_t)0);
+  ddsi_xqos_mergein_missing (rqos, &ddsi_default_qos_reader, ~(uint64_t)0);
 
   if ((rc = ddsi_xqos_valid (&gv->logconfig, rqos)) < 0 || (rc = validate_reader_qos(rqos)) != DDS_RETCODE_OK)
     goto err_bad_qos;
@@ -506,12 +597,12 @@ static dds_entity_t dds_create_reader_int (dds_entity_t participant_or_subscribe
      the subscriber lock, we can assert that the participant exists. */
   assert (pp != NULL);
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   /* Check if DDS Security is enabled */
   if (q_omg_participant_is_secure (pp))
   {
     /* ask to access control security plugin for create reader permissions */
-    if (!q_omg_security_check_create_reader (pp, gv->config.domainId, tp->m_stopic->name, rqos))
+    if (!q_omg_security_check_create_reader (pp, gv->config.domainId, tp->m_name, rqos))
     {
       rc = DDS_RETCODE_NOT_ALLOWED_BY_SECURITY;
       goto err_not_allowed;
@@ -521,11 +612,17 @@ static dds_entity_t dds_create_reader_int (dds_entity_t participant_or_subscribe
 
   /* Create reader and associated read cache (if not provided by caller) */
   struct dds_reader * const rd = dds_alloc (sizeof (*rd));
-  const dds_entity_t reader = dds_entity_init (&rd->m_entity, &sub->m_entity, DDS_KIND_READER, false, rqos, listener, DDS_READER_STATUS_MASK);
+  const dds_entity_t reader = dds_entity_init (&rd->m_entity, &sub->m_entity, DDS_KIND_READER, false, true, rqos, listener, DDS_READER_STATUS_MASK);
+  // assume DATA_ON_READERS is materialized in the subscriber:
+  // - changes to it won't be propagated to this reader until after it has been added to the subscriber's children
+  // - data can arrive once `new_reader` is called, requiring raising DATA_ON_READERS if materialized
+  // - setting DATA_ON_READERS on subscriber if it is not actually materialized is no problem
+  ddsrt_atomic_or32 (&rd->m_entity.m_status.m_status_and_mask, DDS_DATA_ON_READERS_STATUS << SAM_ENABLED_SHIFT);
   rd->m_sample_rejected_status.last_reason = DDS_NOT_REJECTED;
   rd->m_topic = tp;
-  rd->m_rhc = rhc ? rhc : dds_rhc_default_new (rd, tp->m_stopic);
-  if (dds_rhc_associate (rd->m_rhc, rd, tp->m_stopic, rd->m_entity.m_domain->gv.m_tkmap) < 0)
+  rd->m_wrapped_sertopic = (tp->m_stype->wrapped_sertopic != NULL) ? 1 : 0;
+  rd->m_rhc = rhc ? rhc : dds_rhc_default_new (rd, tp->m_stype);
+  if (dds_rhc_associate (rd->m_rhc, rd, tp->m_stype, rd->m_entity.m_domain->gv.m_tkmap) < 0)
   {
     /* FIXME: see also create_querycond, need to be able to undo entity_init */
     abort ();
@@ -537,19 +634,60 @@ static dds_entity_t dds_create_reader_int (dds_entity_t participant_or_subscribe
      it; and then invoke those listeners that are in the pending set */
   dds_entity_init_complete (&rd->m_entity);
 
-  rc = new_reader (&rd->m_rd, &rd->m_entity.m_guid, NULL, pp, tp->m_stopic, rqos, &rd->m_rhc->common.rhc, dds_reader_status_cb, rd);
+#ifdef DDS_HAS_SHM
+  assert(rqos->present & QP_LOCATOR_MASK);
+  if (!dds_reader_support_shm(&gv->config, rqos, tp))
+    rqos->ignore_locator_type |= NN_LOCATOR_KIND_SHEM;
+#endif
+
+  rc = new_reader (&rd->m_rd, &rd->m_entity.m_guid, NULL, pp, tp->m_name, tp->m_stype, rqos, &rd->m_rhc->common.rhc, dds_reader_status_cb, rd);
   assert (rc == DDS_RETCODE_OK); /* FIXME: can be out-of-resources at the very least */
   thread_state_asleep (lookup_thread_state ());
 
+#ifdef DDS_HAS_SHM
+  if (0x0 == (rqos->ignore_locator_type & NN_LOCATOR_KIND_SHEM))
+  {
+    DDS_CLOG (DDS_LC_SHM, &rd->m_entity.m_domain->gv.logconfig, "Reader's topic name will be DDS:Cyclone:%s\n", rd->m_topic->m_name);
+
+    iox_sub_options_t opts;
+    iox_sub_options_init(&opts);
+
+    // ICEORYX TODO: handle failure (how should the system behave if resources are insufficient?)
+    iox_sub_storage_extension_init(&rd->m_iox_sub_stor);
+
+    assert (rqos->durability.kind == DDS_DURABILITY_VOLATILE);
+    opts.queueCapacity = rd->m_entity.m_domain->gv.config.sub_queue_capacity;
+    opts.historyRequest = 0;
+    rd->m_iox_sub = iox_sub_init(&rd->m_iox_sub_stor.storage, gv->config.iceoryx_service, rd->m_topic->m_stype->type_name, rd->m_topic->m_name, &opts);
+    shm_monitor_attach_reader(&rd->m_entity.m_domain->m_shm_monitor, rd);
+
+    // those are set once and never changed
+    // they are used to access reader and monitor from the callback when data is received
+    rd->m_iox_sub_stor.monitor = &rd->m_entity.m_domain->m_shm_monitor;
+    rd->m_iox_sub_stor.parent_reader = rd;
+  }
+#endif
+
   rd->m_entity.m_iid = get_entity_instance_id (&rd->m_entity.m_domain->gv, &rd->m_entity.m_guid);
   dds_entity_register_child (&sub->m_entity, &rd->m_entity);
 
+  // After including the reader amongst the subscriber's children, the subscriber will start
+  // propagating whether data_on_readers is materialised or not.  That doesn't cater for the cases
+  // where pessimistically set it to materialized here, nor for the race where the it actually was
+  // materialized but no longer so prior to `dds_entity_register_child`.
+  ddsrt_mutex_lock (&rd->m_entity.m_observers_lock);
+  ddsrt_mutex_lock (&sub->m_entity.m_observers_lock);
+  if (sub->materialize_data_on_readers == 0)
+    ddsrt_atomic_and32 (&rd->m_entity.m_status.m_status_and_mask, ~(uint32_t)(DDS_DATA_ON_READERS_STATUS << SAM_ENABLED_SHIFT));
+  ddsrt_mutex_unlock (&sub->m_entity.m_observers_lock);
+  ddsrt_mutex_unlock (&rd->m_entity.m_observers_lock);
+
   dds_topic_allow_set_qos (tp);
   dds_topic_unpin (tp);
   dds_subscriber_unlock (sub);
   return reader;
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
 err_not_allowed:
   thread_state_asleep (lookup_thread_state ());
 #endif
@@ -690,13 +828,35 @@ dds_entity_t dds_get_subscriber (dds_entity_t entity)
   }
 }
 
-/* Reset sets everything (type) 0, including the reason field, verify that 0 is correct */
-DDSRT_STATIC_ASSERT ((int) DDS_NOT_REJECTED == 0);
-
-DDS_GET_STATUS (reader, subscription_matched,       SUBSCRIPTION_MATCHED,       total_count_change, current_count_change)
-DDS_GET_STATUS (reader, liveliness_changed,         LIVELINESS_CHANGED,         alive_count_change, not_alive_count_change)
-DDS_GET_STATUS (reader, sample_rejected,            SAMPLE_REJECTED,            total_count_change)
-DDS_GET_STATUS (reader, sample_lost,                SAMPLE_LOST,                total_count_change)
-DDS_GET_STATUS (reader, requested_deadline_missed,  REQUESTED_DEADLINE_MISSED,  total_count_change)
-DDS_GET_STATUS (reader, requested_incompatible_qos, REQUESTED_INCOMPATIBLE_QOS, total_count_change)
+dds_return_t dds__reader_data_allocator_init (const dds_reader *rd, dds_data_allocator_t *data_allocator)
+{
+#ifdef DDS_HAS_SHM
+  dds_iox_allocator_t *d = (dds_iox_allocator_t *) data_allocator->opaque.bytes;
+  if (NULL != rd->m_iox_sub)
+  {
+    d->kind = DDS_IOX_ALLOCATOR_KIND_SUBSCRIBER;
+    d->ref.sub = rd->m_iox_sub;
+  }
+  else
+  {
+    d->kind = DDS_IOX_ALLOCATOR_KIND_NONE;
+  }
+  return DDS_RETCODE_OK;
+#else
+  (void) rd;
+  (void) data_allocator;
+  return DDS_RETCODE_OK;
+#endif
+}
 
+dds_return_t dds__reader_data_allocator_fini (const dds_reader *rd, dds_data_allocator_t *data_allocator)
+{
+#ifdef DDS_HAS_SHM
+  dds_iox_allocator_t *d = (dds_iox_allocator_t *) data_allocator->opaque.bytes;
+  d->kind = DDS_IOX_ALLOCATOR_KIND_FINI;
+#else
+  (void) data_allocator;
+#endif
+  (void) rd;
+  return DDS_RETCODE_OK;
+}
