@@ -10,6 +10,7 @@
  * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
  */
 #include <assert.h>
+#include <string.h>
 
 #include "dds/dds.h"
 #include "dds/version.h"
@@ -31,9 +32,10 @@
 #include "dds/ddsi/ddsi_tkmap.h"
 #include "dds__whc.h"
 #include "dds__statistics.h"
+#include "dds__data_allocator.h"
 #include "dds/ddsi/ddsi_statistics.h"
 
-DECL_ENTITY_LOCK_UNLOCK (extern inline, dds_writer)
+DECL_ENTITY_LOCK_UNLOCK (dds_writer)
 
 #define DDS_WRITER_STATUS_MASK                                   \
                         (DDS_LIVELINESS_LOST_STATUS              |\
@@ -46,7 +48,7 @@ static dds_return_t dds_writer_status_validate (uint32_t mask)
   return (mask & ~DDS_WRITER_STATUS_MASK) ? DDS_RETCODE_BAD_PARAMETER : DDS_RETCODE_OK;
 }
 
-static void update_offered_deadline_missed (struct dds_offered_deadline_missed_status * __restrict st, struct dds_offered_deadline_missed_status * __restrict lst, const status_cb_data_t *data)
+static void update_offered_deadline_missed (struct dds_offered_deadline_missed_status * __restrict st, const status_cb_data_t *data)
 {
   st->last_instance_handle = data->handle;
   st->total_count++;
@@ -56,38 +58,23 @@ static void update_offered_deadline_missed (struct dds_offered_deadline_missed_s
   //
   // (same line of reasoning for all of them)
   st->total_count_change++;
-  if (lst != NULL)
-  {
-    *lst = *st;
-    st->total_count_change = 0;
-  }
 }
 
-static void update_offered_incompatible_qos (struct dds_offered_incompatible_qos_status * __restrict st, struct dds_offered_incompatible_qos_status * __restrict lst, const status_cb_data_t *data)
+static void update_offered_incompatible_qos (struct dds_offered_incompatible_qos_status * __restrict st, const status_cb_data_t *data)
 {
   st->last_policy_id = data->extra;
   st->total_count++;
   st->total_count_change++;
-  if (lst != NULL)
-  {
-    *lst = *st;
-    st->total_count_change = 0;
-  }
 }
 
-static void update_liveliness_lost (struct dds_liveliness_lost_status * __restrict st, struct dds_liveliness_lost_status * __restrict lst, const status_cb_data_t *data)
+static void update_liveliness_lost (struct dds_liveliness_lost_status * __restrict st, const status_cb_data_t *data)
 {
   (void) data;
   st->total_count++;
   st->total_count_change++;
-  if (lst != NULL)
-  {
-    *lst = *st;
-    st->total_count_change = 0;
-  }
 }
 
-static void update_publication_matched (struct dds_publication_matched_status * __restrict st, struct dds_publication_matched_status * __restrict lst, const status_cb_data_t *data)
+static void update_publication_matched (struct dds_publication_matched_status * __restrict st, const status_cb_data_t *data)
 {
   st->last_subscription_handle = data->handle;
   if (data->add) {
@@ -99,18 +86,17 @@ static void update_publication_matched (struct dds_publication_matched_status *
     st->current_count--;
     st->current_count_change--;
   }
-  if (lst != NULL)
-  {
-    *lst = *st;
-    st->total_count_change = 0;
-    st->current_count_change = 0;
-  }
 }
 
-STATUS_CB_IMPL (writer, offered_deadline_missed, OFFERED_DEADLINE_MISSED)
-STATUS_CB_IMPL (writer, offered_incompatible_qos, OFFERED_INCOMPATIBLE_QOS)
-STATUS_CB_IMPL (writer, liveliness_lost, LIVELINESS_LOST)
-STATUS_CB_IMPL (writer, publication_matched, PUBLICATION_MATCHED)
+DDS_GET_STATUS(writer, publication_matched, PUBLICATION_MATCHED, total_count_change, current_count_change)
+DDS_GET_STATUS(writer, liveliness_lost, LIVELINESS_LOST, total_count_change)
+DDS_GET_STATUS(writer, offered_deadline_missed, OFFERED_DEADLINE_MISSED, total_count_change)
+DDS_GET_STATUS(writer, offered_incompatible_qos, OFFERED_INCOMPATIBLE_QOS, total_count_change)
+
+STATUS_CB_IMPL(writer, publication_matched, PUBLICATION_MATCHED, total_count_change, current_count_change)
+STATUS_CB_IMPL(writer, liveliness_lost, LIVELINESS_LOST, total_count_change)
+STATUS_CB_IMPL(writer, offered_deadline_missed, OFFERED_DEADLINE_MISSED, total_count_change)
+STATUS_CB_IMPL(writer, offered_incompatible_qos, OFFERED_INCOMPATIBLE_QOS, total_count_change)
 
 void dds_writer_status_cb (void *entity, const struct status_cb_data *data)
 {
@@ -130,24 +116,25 @@ void dds_writer_status_cb (void *entity, const struct status_cb_data *data)
 
   /* FIXME: why wait if no listener is set? */
   ddsrt_mutex_lock (&wr->m_entity.m_observers_lock);
+  wr->m_entity.m_cb_pending_count++;
   while (wr->m_entity.m_cb_count > 0)
     ddsrt_cond_wait (&wr->m_entity.m_observers_cond, &wr->m_entity.m_observers_lock);
+  wr->m_entity.m_cb_count++;
 
   const enum dds_status_id status_id = (enum dds_status_id) data->raw_status_id;
-  const bool enabled = (ddsrt_atomic_ld32 (&wr->m_entity.m_status.m_status_and_mask) & ((1u << status_id) << SAM_ENABLED_SHIFT)) != 0;
   switch (status_id)
   {
     case DDS_OFFERED_DEADLINE_MISSED_STATUS_ID:
-      status_cb_offered_deadline_missed (wr, data, enabled);
+      status_cb_offered_deadline_missed (wr, data);
       break;
     case DDS_LIVELINESS_LOST_STATUS_ID:
-      status_cb_liveliness_lost (wr, data, enabled);
+      status_cb_liveliness_lost (wr, data);
       break;
     case DDS_OFFERED_INCOMPATIBLE_QOS_STATUS_ID:
-      status_cb_offered_incompatible_qos (wr, data, enabled);
+      status_cb_offered_incompatible_qos (wr, data);
       break;
     case DDS_PUBLICATION_MATCHED_STATUS_ID:
-      status_cb_publication_matched (wr, data, enabled);
+      status_cb_publication_matched (wr, data);
       break;
     case DDS_DATA_AVAILABLE_STATUS_ID:
     case DDS_INCONSISTENT_TOPIC_STATUS_ID:
@@ -161,14 +148,16 @@ void dds_writer_status_cb (void *entity, const struct status_cb_data *data)
       assert (0);
   }
 
+  wr->m_entity.m_cb_count--;
+  wr->m_entity.m_cb_pending_count--;
   ddsrt_cond_broadcast (&wr->m_entity.m_observers_cond);
   ddsrt_mutex_unlock (&wr->m_entity.m_observers_lock);
 }
 
 static uint32_t get_bandwidth_limit (dds_transport_priority_qospolicy_t transport_priority)
 {
-#ifdef DDSI_INCLUDE_NETWORK_CHANNELS
-  struct config_channel_listelem *channel = find_channel (&config, transport_priority);
+#ifdef DDS_HAS_NETWORK_CHANNELS
+  struct ddsi_config_channel_listelem *channel = find_channel (&config, transport_priority);
   return channel->data_bandwidth_limit;
 #else
   (void) transport_priority;
@@ -209,6 +198,14 @@ static dds_return_t dds_writer_delete (dds_entity *e) ddsrt_nonnull_all;
 static dds_return_t dds_writer_delete (dds_entity *e)
 {
   dds_writer * const wr = (dds_writer *) e;
+#ifdef DDS_HAS_SHM
+  if (wr->m_iox_pub)
+  {
+    DDS_CLOG(DDS_LC_SHM, &e->m_domain->gv.logconfig, "Release iceoryx's publisher\n");
+    iox_pub_stop_offer(wr->m_iox_pub);
+    iox_pub_deinit(wr->m_iox_pub);
+  }
+#endif
   /* FIXME: not freeing WHC here because it is owned by the DDSI entity */
   thread_state_awake (lookup_thread_state (), &e->m_domain->gv);
   nn_xpack_free (wr->m_xp);
@@ -219,15 +216,15 @@ static dds_return_t dds_writer_delete (dds_entity *e)
 
 static dds_return_t validate_writer_qos (const dds_qos_t *wqos)
 {
-#ifndef DDSI_INCLUDE_LIFESPAN
+#ifndef DDS_HAS_LIFESPAN
   if (wqos != NULL && (wqos->present & QP_LIFESPAN) && wqos->lifespan.duration != DDS_INFINITY)
     return DDS_RETCODE_BAD_PARAMETER;
 #endif
-#ifndef DDSI_INCLUDE_DEADLINE_MISSED
+#ifndef DDS_HAS_DEADLINE_MISSED
   if (wqos != NULL && (wqos->present & QP_DEADLINE) && wqos->deadline.deadline != DDS_INFINITY)
     return DDS_RETCODE_BAD_PARAMETER;
 #endif
-#if defined(DDSI_INCLUDE_LIFESPAN) && defined(DDSI_INCLUDE_DEADLINE_MISSED)
+#if defined(DDS_HAS_LIFESPAN) && defined(DDS_HAS_DEADLINE_MISSED)
   DDSRT_UNUSED_ARG (wqos);
 #endif
   return DDS_RETCODE_OK;
@@ -284,6 +281,30 @@ const struct dds_entity_deriver dds_entity_deriver_writer = {
   .refresh_statistics = dds_writer_refresh_statistics
 };
 
+#ifdef DDS_HAS_SHM
+#define DDS_WRITER_QOS_CHECK_FIELDS (QP_LIVELINESS|QP_DEADLINE|QP_RELIABILITY|QP_DURABILITY|QP_HISTORY)
+static bool dds_writer_support_shm(const struct ddsi_config* cfg, const dds_qos_t* qos, const struct dds_topic *tp)
+{
+  if (NULL == cfg ||
+      false == cfg->enable_shm)
+    return false;
+
+  if (!tp->m_stype->fixed_size)
+    return false;
+
+  uint32_t pub_history_cap = cfg->pub_history_capacity;
+
+  return (NULL != qos &&
+          DDS_WRITER_QOS_CHECK_FIELDS == (qos->present&DDS_WRITER_QOS_CHECK_FIELDS) &&
+          DDS_LIVELINESS_AUTOMATIC == qos->liveliness.kind &&
+          DDS_INFINITY == qos->deadline.deadline &&
+          DDS_RELIABILITY_RELIABLE == qos->reliability.kind &&
+          DDS_DURABILITY_VOLATILE == qos->durability.kind &&
+          DDS_HISTORY_KEEP_LAST == qos->history.kind &&
+          (int)pub_history_cap >= (int)qos->history.depth);
+}
+#endif
+
 dds_entity_t dds_create_writer (dds_entity_t participant_or_publisher, dds_entity_t topic, const dds_qos_t *qos, const dds_listener_t *listener)
 {
   dds_return_t rc;
@@ -319,7 +340,7 @@ dds_entity_t dds_create_writer (dds_entity_t participant_or_publisher, dds_entit
 
   if ((rc = dds_topic_pin (topic, &tp)) != DDS_RETCODE_OK)
     goto err_pin_topic;
-  assert (tp->m_stopic);
+  assert (tp->m_stype);
   if (dds_entity_participant (&pub->m_entity) != dds_entity_participant (&tp->m_entity))
   {
     rc = DDS_RETCODE_BAD_PARAMETER;
@@ -345,7 +366,7 @@ dds_entity_t dds_create_writer (dds_entity_t participant_or_publisher, dds_entit
     ddsi_xqos_mergein_missing (wqos, pub->m_entity.m_qos, ~(uint64_t)0);
   if (tp->m_ktopic->qos)
     ddsi_xqos_mergein_missing (wqos, tp->m_ktopic->qos, ~(uint64_t)0);
-  ddsi_xqos_mergein_missing (wqos, &gv->default_xqos_wr, ~(uint64_t)0);
+  ddsi_xqos_mergein_missing (wqos, &ddsi_default_qos_writer, ~(uint64_t)0);
 
   if ((rc = ddsi_xqos_valid (&gv->logconfig, wqos)) < 0 || (rc = validate_writer_qos(wqos)) != DDS_RETCODE_OK)
     goto err_bad_qos;
@@ -358,12 +379,12 @@ dds_entity_t dds_create_writer (dds_entity_t participant_or_publisher, dds_entit
      the publisher lock, we can assert that the participant exists. */
   assert (pp != NULL);
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   /* Check if DDS Security is enabled */
   if (q_omg_participant_is_secure (pp))
   {
     /* ask to access control security plugin for create writer permissions */
-    if (!q_omg_security_check_create_writer (pp, gv->config.domainId, tp->m_stopic->name, wqos))
+    if (!q_omg_security_check_create_writer (pp, gv->config.domainId, tp->m_name, wqos))
     {
       rc = DDS_RETCODE_NOT_ALLOWED_BY_SECURITY;
       goto err_not_allowed;
@@ -371,22 +392,43 @@ dds_entity_t dds_create_writer (dds_entity_t participant_or_publisher, dds_entit
   }
 #endif
 
+  // configure async mode
+  bool async_mode = (wqos->latency_budget.duration > 0);
+
   /* Create writer */
-  ddsi_tran_conn_t conn = gv->xmit_conn;
   struct dds_writer * const wr = dds_alloc (sizeof (*wr));
-  const dds_entity_t writer = dds_entity_init (&wr->m_entity, &pub->m_entity, DDS_KIND_WRITER, false, wqos, listener, DDS_WRITER_STATUS_MASK);
+  const dds_entity_t writer = dds_entity_init (&wr->m_entity, &pub->m_entity, DDS_KIND_WRITER, false, true, wqos, listener, DDS_WRITER_STATUS_MASK);
   wr->m_topic = tp;
   dds_entity_add_ref_locked (&tp->m_entity);
-  wr->m_xp = nn_xpack_new (conn, get_bandwidth_limit (wqos->transport_priority), gv->config.xpack_send_async);
+  wr->m_xp = nn_xpack_new (gv, get_bandwidth_limit (wqos->transport_priority), async_mode);
   wrinfo = whc_make_wrinfo (wr, wqos);
   wr->m_whc = whc_new (gv, wrinfo);
   whc_free_wrinfo (wrinfo);
   wr->whc_batch = gv->config.whc_batch;
 
-  rc = new_writer (&wr->m_wr, &wr->m_entity.m_guid, NULL, pp, tp->m_stopic, wqos, wr->m_whc, dds_writer_status_cb, wr);
+#ifdef DDS_HAS_SHM
+  assert(wqos->present & QP_LOCATOR_MASK);
+  if (!dds_writer_support_shm(&gv->config, wqos, tp))
+    wqos->ignore_locator_type |= NN_LOCATOR_KIND_SHEM;
+#endif
+
+  rc = new_writer (&wr->m_wr, &wr->m_entity.m_guid, NULL, pp, tp->m_name, tp->m_stype, wqos, wr->m_whc, dds_writer_status_cb, wr);
   assert(rc == DDS_RETCODE_OK);
   thread_state_asleep (lookup_thread_state ());
 
+#ifdef DDS_HAS_SHM
+  if (0x0 == (wqos->ignore_locator_type & NN_LOCATOR_KIND_SHEM))
+  {
+    DDS_CLOG (DDS_LC_SHM, &wr->m_entity.m_domain->gv.logconfig, "Writer's topic name will be DDS:Cyclone:%s\n", wr->m_topic->m_name);
+    iox_pub_options_t opts;
+    iox_pub_options_init(&opts);
+    opts.historyCapacity = wr->m_entity.m_domain->gv.config.pub_history_capacity;
+    wr->m_iox_pub = iox_pub_init(&wr->m_iox_pub_stor, gv->config.iceoryx_service, wr->m_topic->m_stype->type_name, wr->m_topic->m_name, &opts);
+    memset(wr->m_iox_pub_loans, 0, sizeof(wr->m_iox_pub_loans));
+    dds_sleepfor(DDS_MSECS(10));
+  }
+#endif
+
   wr->m_entity.m_iid = get_entity_instance_id (&wr->m_entity.m_domain->gv, &wr->m_entity.m_guid);
   dds_entity_register_child (&pub->m_entity, &wr->m_entity);
 
@@ -395,9 +437,17 @@ dds_entity_t dds_create_writer (dds_entity_t participant_or_publisher, dds_entit
   dds_topic_allow_set_qos (tp);
   dds_topic_unpin (tp);
   dds_publisher_unlock (pub);
+
+  // start async thread if not already started and the latency budget is non zero
+  ddsrt_mutex_lock (&gv->sendq_running_lock);
+  if (async_mode && !gv->sendq_running) {
+    nn_xpack_sendq_init(gv);
+    nn_xpack_sendq_start(gv);
+  }
+  ddsrt_mutex_unlock (&gv->sendq_running_lock);
   return writer;
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
 err_not_allowed:
   thread_state_asleep (lookup_thread_state ());
 #endif
@@ -434,6 +484,39 @@ dds_entity_t dds_get_publisher (dds_entity_t writer)
   }
 }
 
+dds_return_t dds__writer_data_allocator_init (const dds_writer *wr, dds_data_allocator_t *data_allocator)
+{
+#ifdef DDS_HAS_SHM
+  dds_iox_allocator_t *d = (dds_iox_allocator_t *) data_allocator->opaque.bytes;
+  if (NULL != wr->m_iox_pub)
+  {
+    d->kind = DDS_IOX_ALLOCATOR_KIND_PUBLISHER;
+    d->ref.pub = wr->m_iox_pub;
+  }
+  else
+  {
+    d->kind = DDS_IOX_ALLOCATOR_KIND_NONE;
+  }
+  return DDS_RETCODE_OK;
+#else
+  (void) wr;
+  (void) data_allocator;
+  return DDS_RETCODE_OK;
+#endif
+}
+
+dds_return_t dds__writer_data_allocator_fini (const dds_writer *wr, dds_data_allocator_t *data_allocator)
+{
+#ifdef DDS_HAS_SHM
+  dds_iox_allocator_t *d = (dds_iox_allocator_t *) data_allocator->opaque.bytes;
+  d->kind = DDS_IOX_ALLOCATOR_KIND_FINI;
+#else
+  (void) data_allocator;
+#endif
+  (void) wr;
+  return DDS_RETCODE_OK;
+}
+
 dds_return_t dds__writer_wait_for_acks (struct dds_writer *wr, ddsi_guid_t *rdguid, dds_time_t abstimeout)
 {
   /* during lifetime of the writer m_wr is constant, it is only during deletion that it
@@ -443,8 +526,3 @@ dds_return_t dds__writer_wait_for_acks (struct dds_writer *wr, ddsi_guid_t *rdgu
   else
     return writer_wait_for_acks (wr->m_wr, rdguid, abstimeout);
 }
-
-DDS_GET_STATUS(writer, publication_matched, PUBLICATION_MATCHED, total_count_change, current_count_change)
-DDS_GET_STATUS(writer, liveliness_lost, LIVELINESS_LOST, total_count_change)
-DDS_GET_STATUS(writer, offered_deadline_missed, OFFERED_DEADLINE_MISSED, total_count_change)
-DDS_GET_STATUS(writer, offered_incompatible_qos, OFFERED_INCOMPATIBLE_QOS, total_count_change)
