@@ -12,10 +12,12 @@
 #ifndef DDSI_PLIST_H
 #define DDSI_PLIST_H
 
+#include "dds/ddsrt/bswap.h"
 #include "dds/ddsi/q_feature_check.h"
 #include "dds/ddsi/ddsi_xqos.h"
 #include "dds/ddsi/ddsi_keyhash.h"
 #include "dds/ddsi/ddsi_tran.h" /* FIXME: eliminate */
+#include "dds/ddsi/ddsi_typelookup.h"
 
 #if defined (__cplusplus)
 extern "C" {
@@ -49,7 +51,7 @@ extern "C" {
 #define PP_ADLINK_PARTICIPANT_VERSION_INFO      ((uint64_t)1 << 26)
 #define PP_ADLINK_TYPE_DESCRIPTION              ((uint64_t)1 << 27)
 #define PP_COHERENT_SET                         ((uint64_t)1 << 28)
-#ifdef DDSI_INCLUDE_SSM
+#ifdef DDS_HAS_SSM
 #define PP_READER_FAVOURS_SSM                   ((uint64_t)1 << 29)
 #endif
 #define PP_DOMAIN_ID                            ((uint64_t)1 << 30)
@@ -61,7 +63,12 @@ extern "C" {
 #define PP_PARTICIPANT_SECURITY_INFO            ((uint64_t)1 << 35)
 #define PP_IDENTITY_STATUS_TOKEN                ((uint64_t)1 << 36)
 #define PP_DATA_TAGS                            ((uint64_t)1 << 37)
+/* Other stuff */
 #define PP_CYCLONE_RECEIVE_BUFFER_SIZE          ((uint64_t)1 << 38)
+#define PP_CYCLONE_TOPIC_GUID                   ((uint64_t)1 << 39)
+#define PP_CYCLONE_REQUESTS_KEYHASH             ((uint64_t)1 << 40)
+#define PP_CYCLONE_REDUNDANT_NETWORKING         ((uint64_t)1 << 41)
+
 /* Set for unrecognized parameters that are in the reserved space or
    in our own vendor-specific space that have the
    PID_UNRECOGNIZED_INCOMPATIBLE_FLAG set (see DDSI 2.1 9.6.2.2.1) */
@@ -86,7 +93,7 @@ extern "C" {
    the same thing. */
 struct nn_locators_one {
   struct nn_locators_one *next;
-  nn_locator_t loc;
+  ddsi_locator_t loc;
 };
 
 typedef struct nn_locators {
@@ -99,7 +106,7 @@ typedef uint32_t nn_ipv4address_t;
 
 typedef uint32_t nn_port_t;
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
 typedef struct nn_tag {
   char *name;
   char *value;
@@ -115,13 +122,13 @@ typedef struct nn_datatags {
 } nn_datatags_t;
 #endif
 
-#ifdef DDSI_INCLUDE_SSM
+#ifdef DDS_HAS_SSM
 typedef struct nn_reader_favours_ssm {
   uint32_t state; /* default is false */
 } nn_reader_favours_ssm_t;
 #endif
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
 typedef struct nn_dataholder
 {
   char *class_id;
@@ -203,6 +210,7 @@ typedef struct ddsi_plist {
   ddsi_guid_t participant_guid;
   ddsi_guid_t endpoint_guid;
   ddsi_guid_t group_guid;
+  ddsi_guid_t topic_guid;
 #if 0 /* reserved, rather than NIY */
   nn_entityid_t participant_entityid;
   nn_entityid_t group_entityid;
@@ -215,7 +223,7 @@ typedef struct ddsi_plist {
   nn_adlink_participant_version_info_t adlink_participant_version_info;
   char *type_description;
   nn_sequence_number_t coherent_set_seqno;
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   nn_token_t identity_token;
   nn_token_t permissions_token;
   nn_security_info_t endpoint_security_info;
@@ -223,12 +231,14 @@ typedef struct ddsi_plist {
   nn_token_t identity_status_token;
   nn_datatags_t data_tags;
 #endif
-#ifdef DDSI_INCLUDE_SSM
+#ifdef DDS_HAS_SSM
   nn_reader_favours_ssm_t reader_favours_ssm;
 #endif
   uint32_t domain_id;
   char *domain_tag;
   uint32_t cyclone_receive_buffer_size;
+  unsigned char cyclone_requests_keyhash;
+  unsigned char cyclone_redundant_networking;
 } ddsi_plist_t;
 
 
@@ -241,10 +251,10 @@ typedef struct ddsi_plist_src {
   const unsigned char *buf;               /**< input buffer */
   size_t bufsz;                           /**< size of input buffer */
   bool strict;                            /**< whether to be strict in checking */
-  ddsi_tran_factory_t factory;            /**< transport; eliminate this */
-  struct ddsrt_log_cfg *logconfig;        /**< logging configuration */
 } ddsi_plist_src_t;
 
+DDS_EXPORT extern const ddsi_plist_t ddsi_default_plist_participant;
+
 /**
  * @brief Initialize global parameter-list parsing indices.
  *
@@ -315,6 +325,8 @@ DDS_EXPORT ddsi_plist_t *ddsi_plist_dup (const ddsi_plist_t *src);
  *               - encoding is PL_CDR_LE or PL_CDR_BE
  *               - buf is a pointer to the first parameter header
  *               - bufsz is the size in bytes of the input buffer
+ * @param[in]  gv
+ *               Global context, used for locator kind lookups and tracing
  * @param[out] dest
  *               Filled with the recognized parameters in the input if successful, otherwise
  *               initialized to an empty parameter list.  Where possible, pointers alias the
@@ -341,7 +353,7 @@ DDS_EXPORT ddsi_plist_t *ddsi_plist_dup (const ddsi_plist_t *src);
  *               Input contained an unrecognized parameter with the "incompatible-if-unknown"
  *               flag set; dest is cleared, *nextafterplist is NULL.
  */
-DDS_EXPORT dds_return_t ddsi_plist_init_frommsg (ddsi_plist_t *dest, char **nextafterplist, uint64_t pwanted, uint64_t qwanted, const ddsi_plist_src_t *src);
+DDS_EXPORT dds_return_t ddsi_plist_init_frommsg (ddsi_plist_t *dest, char **nextafterplist, uint64_t pwanted, uint64_t qwanted, const ddsi_plist_src_t *src, struct ddsi_domaingv const * const gv);
 
 /**
  * @brief Free memory owned by "ps"
@@ -411,16 +423,9 @@ DDS_EXPORT void ddsi_plist_addtomsg (struct nn_xmsg *m, const ddsi_plist_t *ps,
  * @param[in]     ps       source
  * @param[in]     pwanted  subset of non-QoS part of ps (if PP_X is set, add X if present)
  * @param[in]     qwanted  subset of QoS part of ps (if QP_X is set, add X if present)
- * @param[in]     be       use native endianness if false, big-endian if true
+ * @param[in]     bo       byte order
  */
-DDS_EXPORT void ddsi_plist_addtomsg_bo (struct nn_xmsg *m, const ddsi_plist_t *ps, uint64_t pwanted, uint64_t qwanted, bool be);
-
-/**
- * @brief Initialize plist to match default settings for a participant
- *
- * @param[out] plist    plist to contain the default settings.
- */
-DDS_EXPORT void ddsi_plist_init_default_participant (ddsi_plist_t *plist);
+DDS_EXPORT void ddsi_plist_addtomsg_bo (struct nn_xmsg *m, const ddsi_plist_t *ps, uint64_t pwanted, uint64_t qwanted, enum ddsrt_byte_order_selector bo);
 
 /**
  * @brief Determine the set of entries in which "x" differs from "y"
@@ -481,13 +486,15 @@ struct nn_rsample_info;
  * It clears the `bswap` flag in `dest` if the input is in native endianness, and sets it
  * otherwise.
  *
- * @param[in]  src      input description (see `ddsi_plist_init_frommsg`)
  * @param[out] dest     internal sample info of which some fields will be set
+ * @param[in]  src      input description (see `ddsi_plist_init_frommsg`)
+ * @param[out] keyhashp set to point to keyhash in inline QoS if present, else to NULL
+ * @param[in]  gv       global context (for logging)
  *
  * @return pointer to the first byte following the sentinel if the input is well-formed, a
  * null pointer if it is not.
 */
-DDS_EXPORT unsigned char *ddsi_plist_quickscan (struct nn_rsample_info *dest, const ddsi_plist_src_t *src);
+DDS_EXPORT unsigned char *ddsi_plist_quickscan (struct nn_rsample_info *dest, const ddsi_keyhash_t **keyhashp, const ddsi_plist_src_t *src, struct ddsi_domaingv const * const gv);
 
 /**
  * @brief Locate a specific parameter in a PL_CDR-serialized parameter list
