@@ -12,6 +12,8 @@
 #ifndef DDSI_XQOS_H
 #define DDSI_XQOS_H
 
+#include "dds/features.h"
+
 #include "dds/ddsc/dds_public_qosdefs.h"
 /*XXX*/
 #include "dds/ddsi/q_protocol.h"
@@ -217,6 +219,17 @@ typedef struct dds_ignorelocal_qospolicy {
   dds_ignorelocal_kind_t value;
 } dds_ignorelocal_qospolicy_t;
 
+typedef struct dds_type_consistency_enforcement_qospolicy {
+  dds_type_consistency_kind_t kind;
+  bool ignore_sequence_bounds;
+  bool ignore_string_bounds;
+  bool ignore_member_names;
+  bool prevent_type_widening;
+  bool force_type_validation;
+} dds_type_consistency_enforcement_qospolicy_t;
+
+typedef uint32_t dds_locator_mask_t;
+
 /***/
 
 /* Qos Present bit indices */
@@ -241,13 +254,16 @@ typedef struct dds_ignorelocal_qospolicy {
 #define QP_OWNERSHIP                         ((uint64_t)1 << 18)
 #define QP_OWNERSHIP_STRENGTH                ((uint64_t)1 << 19)
 #define QP_TIME_BASED_FILTER                 ((uint64_t)1 << 20)
-#define QP_ADLINK_WRITER_DATA_LIFECYCLE   ((uint64_t)1 << 21)
-#define QP_ADLINK_READER_DATA_LIFECYCLE   ((uint64_t)1 << 22)
-#define QP_ADLINK_READER_LIFESPAN         ((uint64_t)1 << 24)
-#define QP_ADLINK_SUBSCRIPTION_KEYS       ((uint64_t)1 << 25)
-#define QP_ADLINK_ENTITY_FACTORY          ((uint64_t)1 << 27)
+#define QP_ADLINK_WRITER_DATA_LIFECYCLE      ((uint64_t)1 << 21)
+#define QP_ADLINK_READER_DATA_LIFECYCLE      ((uint64_t)1 << 22)
+#define QP_ADLINK_READER_LIFESPAN            ((uint64_t)1 << 24)
+#define QP_ADLINK_SUBSCRIPTION_KEYS          ((uint64_t)1 << 25)
+#define QP_ADLINK_ENTITY_FACTORY             ((uint64_t)1 << 27)
 #define QP_CYCLONE_IGNORELOCAL               ((uint64_t)1 << 30)
 #define QP_PROPERTY_LIST                     ((uint64_t)1 << 31)
+#define QP_TYPE_CONSISTENCY_ENFORCEMENT      ((uint64_t)1 << 32)
+#define QP_CYCLONE_TYPE_INFORMATION          ((uint64_t)1 << 33)
+#define QP_LOCATOR_MASK                      ((uint64_t)1 << 34)
 
 /* Partition QoS is not RxO according to the specification (DDS 1.2,
    section 7.1.3), but communication will not take place unless it
@@ -271,6 +287,9 @@ struct dds_qos {
   /*      Extras: */
   /* xx */char *topic_name;
   /* xx */char *type_name;
+#ifdef DDS_HAS_TYPE_DISCOVERY
+  /* xx */ddsi_octetseq_t type_information;
+#endif
   /*      PublisherQos, SubscriberQos: */
   /*xxx */dds_presentation_qospolicy_t presentation;
   /*xxx */dds_partition_qospolicy_t partition;
@@ -300,10 +319,17 @@ struct dds_qos {
   /*x xR*/dds_reader_lifespan_qospolicy_t reader_lifespan;
   /* x  */dds_ignorelocal_qospolicy_t ignorelocal;
   /*xxx */dds_property_qospolicy_t property;
+  /*xxxR*/dds_type_consistency_enforcement_qospolicy_t type_consistency;
+  /*xxxX*/dds_locator_mask_t ignore_locator_type;
 };
 
 struct nn_xmsg;
 
+DDS_EXPORT extern const dds_qos_t ddsi_default_qos_reader;
+DDS_EXPORT extern const dds_qos_t ddsi_default_qos_writer;
+DDS_EXPORT extern const dds_qos_t ddsi_default_qos_topic;
+DDS_EXPORT extern const dds_qos_t ddsi_default_qos_publisher_subscriber;
+
 /**
  * @brief Initialize a new empty dds_qos_t as an empty object
  *
@@ -314,63 +340,6 @@ struct nn_xmsg;
  */
 DDS_EXPORT void ddsi_xqos_init_empty (dds_qos_t *xqos);
 
-/**
- * @brief Initialize xqos to match default QoS settings for a reader
- *
- * @param[out] xqos    qos object to contain the default settings.
- */
-DDS_EXPORT void ddsi_xqos_init_default_reader (dds_qos_t *xqos);
-
-/**
- * @brief Initialize xqos to match default QoS settings for a writer
- *
- * @param[out] xqos    qos object to contain the default settings.
- */
-DDS_EXPORT void ddsi_xqos_init_default_writer (dds_qos_t *xqos);
-
-/**
- * @brief Initialize xqos to match default QoS settings for a non-autodispose writer
- *
- * The default writer QoS has "auto dispose unregistered instances" set to true, and
- * opinions differ about what the correct behaviour is, but the setting doesn't really
- * make any sense if it is treated as a macro on the writing such, such that each
- * "unregister" call also performs a dispose.  Cyclone DDS implements the interpretation
- * that matches the name (whenever no registrations are left, make it disposed), which is
- * a refinement of OpenSplice's interpretation (whenever an unregister event occurs, make
- * it disposed).  All these differences originate in the DCPS spec not being clear enough
- * and the complicating factor is that the two different interpretations existed before
- * the introduction of the DDSI specification, and yet the DDSI specification doesn't
- * provide a standard way of discovering the setting for remote writers.
- *
- * Cyclone DDS uses the same vendor-specific extension as OpenSplice for communicating it,
- * and uses a different default for different implementations.  So this is the default
- * used for the "RTI interpretation".
- *
- * @param[out] xqos    qos object to contain the default settings.
- */
-DDS_EXPORT void ddsi_xqos_init_default_writer_noautodispose (dds_qos_t *xqos);
-
-/**
- * @brief Initialize xqos to match default QoS settings for a subscriber
- *
- * @param[out] xqos    qos object to contain the default settings.
- */
-DDS_EXPORT void ddsi_xqos_init_default_subscriber (dds_qos_t *xqos);
-
-/**
- * @brief Initialize xqos to match default QoS settings for a publisher
- *
- * @param[out] xqos    qos object to contain the default settings.
- */
-DDS_EXPORT void ddsi_xqos_init_default_publisher (dds_qos_t *xqos);
-
-/**
- * @brief Initialize xqos to match default QoS settings for a topic
- *
- * @param[out] xqos    qos object to contain the default settings.
- */
-DDS_EXPORT void ddsi_xqos_init_default_topic (dds_qos_t *xqos);
-
 /**
  * @brief Copy "src" to "dst"
  *
@@ -540,9 +509,9 @@ DDS_EXPORT bool ddsi_xqos_has_prop_prefix (const dds_qos_t *xqos, const char *na
  */
 DDS_EXPORT bool ddsi_xqos_find_prop (const dds_qos_t *xqos, const char *name, const char **value);
 
-#ifdef DDSI_INCLUDE_SECURITY
-struct omg_security_configuration_type;
-DDS_EXPORT void ddsi_xqos_mergein_security_config (dds_qos_t *xqos, const struct omg_security_configuration_type *cfg);
+#ifdef DDS_HAS_SECURITY
+struct ddsi_config_omg_security;
+DDS_EXPORT void ddsi_xqos_mergein_security_config (dds_qos_t *xqos, const struct ddsi_config_omg_security *cfg);
 #endif
 
 #if defined (__cplusplus)
