@@ -16,6 +16,7 @@
 #include "dds/ddsrt/endian.h"
 #include "dds/ddsrt/md5.h"
 #include "dds/ddsrt/heap.h"
+#include "dds/ddsrt/static_assert.h"
 #include "dds/ddsi/q_bswap.h"
 #include "dds/ddsi/q_config.h"
 #include "dds/ddsi/ddsi_cdrstream.h"
@@ -210,9 +211,10 @@ static uint32_t get_type_size (enum dds_stream_typecode type)
   return (uint32_t)1 << ((uint32_t) type - 1);
 }
 
-static size_t dds_stream_check_optimize1 (const struct ddsi_sertopic_default_desc * __restrict desc)
+static size_t dds_stream_check_optimize1 (const struct ddsi_sertype_default_desc * __restrict desc)
 {
-  const uint32_t *ops = desc->m_ops;
+  const uint32_t *ops = desc->ops.ops;
+  size_t off = 0, size;
   uint32_t insn;
   while ((insn = *ops) != DDS_OP_RTS)
   {
@@ -225,8 +227,12 @@ static size_t dds_stream_check_optimize1 (const struct ddsi_sertopic_default_des
       case DDS_OP_VAL_2BY:
       case DDS_OP_VAL_4BY:
       case DDS_OP_VAL_8BY:
-        if ((ops[1] % get_type_size (DDS_OP_TYPE (insn))) != 0)
+        size = get_type_size (DDS_OP_TYPE (insn));
+        if (off % size)
+          off += size - (off % size);
+        if (ops[1] != off)
           return 0;
+        off += size;
         ops += 2;
         break;
 
@@ -234,8 +240,12 @@ static size_t dds_stream_check_optimize1 (const struct ddsi_sertopic_default_des
         switch (DDS_OP_SUBTYPE (insn))
         {
           case DDS_OP_VAL_1BY: case DDS_OP_VAL_2BY: case DDS_OP_VAL_4BY: case DDS_OP_VAL_8BY:
-            if ((ops[1] % get_type_size (DDS_OP_SUBTYPE (insn))) != 0)
+            size = get_type_size (DDS_OP_SUBTYPE (insn));
+            if (off % size)
+              off += size - (off % size);
+            if (ops[1] != off)
               return 0;
+            off += size * ops[2];
             ops += 3;
             break;
           default:
@@ -248,10 +258,12 @@ static size_t dds_stream_check_optimize1 (const struct ddsi_sertopic_default_des
     }
   }
 
-  return desc->m_size;
+  // off < desc can occur if desc->size includes "trailing" padding
+  assert (off <= desc->size);
+  return off;
 }
 
-size_t dds_stream_check_optimize (const struct ddsi_sertopic_default_desc * __restrict desc)
+size_t dds_stream_check_optimize (const struct ddsi_sertype_default_desc * __restrict desc)
 {
   return dds_stream_check_optimize1 (desc);
 }
@@ -1170,12 +1182,12 @@ static bool stream_normalize (char * __restrict data, uint32_t * __restrict off,
   return true;
 }
 
-static bool stream_normalize_key (void * __restrict data, uint32_t size, bool bswap, const struct ddsi_sertopic_default_desc * __restrict desc)
+static bool stream_normalize_key (void * __restrict data, uint32_t size, bool bswap, const struct ddsi_sertype_default_desc * __restrict desc)
 {
   uint32_t off = 0;
-  for (uint32_t i = 0; i < desc->m_nkeys; i++)
+  for (uint32_t i = 0; i < desc->keys.nkeys; i++)
   {
-    const uint32_t *op = desc->m_ops + desc->m_keys[i];
+    const uint32_t *op = desc->ops.ops + desc->keys.keys[i];
     assert (insn_key_ok_p (*op));
     switch (DDS_OP_TYPE (*op))
     {
@@ -1194,7 +1206,7 @@ static bool stream_normalize_key (void * __restrict data, uint32_t size, bool bs
   return true;
 }
 
-bool dds_stream_normalize (void * __restrict data, uint32_t size, bool bswap, const struct ddsi_sertopic_default * __restrict topic, bool just_key)
+bool dds_stream_normalize (void * __restrict data, uint32_t size, bool bswap, const struct ddsi_sertype_default * __restrict topic, bool just_key)
 {
   if (size > CDR_SIZE_MAX)
     return false;
@@ -1203,7 +1215,7 @@ bool dds_stream_normalize (void * __restrict data, uint32_t size, bool bswap, co
   else
   {
     uint32_t off = 0;
-    return stream_normalize (data, &off, size, bswap, topic->type.m_ops);
+    return stream_normalize (data, &off, size, bswap, topic->type.ops.ops);
   }
 }
 
@@ -1410,14 +1422,19 @@ void dds_stream_free_sample (void *vdata, const uint32_t * ops)
  **
  *******************************************************************************************/
 
-void dds_stream_read_sample (dds_istream_t * __restrict is, void * __restrict data, const struct ddsi_sertopic_default * __restrict topic)
+void dds_stream_read_sample (dds_istream_t * __restrict is, void * __restrict data, const struct ddsi_sertype_default * __restrict type)
 {
-  const struct ddsi_sertopic_default_desc *desc = &topic->type;
-  if (topic->opt_size)
-    dds_is_get_bytes (is, data, desc->m_size, 1);
+  const struct ddsi_sertype_default_desc *desc = &type->type;
+  if (type->opt_size)
+  {
+    /* Layout of struct & CDR is the same, but sizeof(struct) may include padding at
+       the end that is not present in CDR, so we must use type->opt_size to avoid a
+       potential out-of-bounds read */
+    dds_is_get_bytes (is, data, (uint32_t) type->opt_size, 1);
+  }
   else
   {
-    if (desc->m_flagset & DDS_TOPIC_CONTAINS_UNION)
+    if (desc->flagset & DDS_TOPIC_CONTAINS_UNION)
     {
       /* Switching union cases causes big trouble if some cases have sequences or strings,
          and other cases have other things mapped to those addresses.  So, pretend to be
@@ -1425,28 +1442,28 @@ void dds_stream_read_sample (dds_istream_t * __restrict is, void * __restrict da
          make any preallocated buffers go to waste, but it does allow reusing the message
          from read-to-read, at the somewhat reasonable price of a slower deserialization
          and not being able to use preallocated sequences in topics containing unions. */
-      dds_stream_free_sample (data, desc->m_ops);
-      memset (data, 0, desc->m_size);
+      dds_stream_free_sample (data, desc->ops.ops);
+      memset (data, 0, desc->size);
     }
-    dds_stream_read (is, data, desc->m_ops);
+    dds_stream_read (is, data, desc->ops.ops);
   }
 }
 
-void dds_stream_write_sample (dds_ostream_t * __restrict os, const void * __restrict data, const struct ddsi_sertopic_default * __restrict topic)
+void dds_stream_write_sample (dds_ostream_t * __restrict os, const void * __restrict data, const struct ddsi_sertype_default * __restrict type)
 {
-  const struct ddsi_sertopic_default_desc *desc = &topic->type;
-  if (topic->opt_size && desc->m_align && (os->m_index % desc->m_align) == 0)
-    dds_os_put_bytes (os, data, desc->m_size);
+  const struct ddsi_sertype_default_desc *desc = &type->type;
+  if (type->opt_size && desc->align && (os->m_index % desc->align) == 0)
+    dds_os_put_bytes (os, data, (uint32_t) type->opt_size);
   else
-    dds_stream_write (os, data, desc->m_ops);
+    dds_stream_write (os, data, desc->ops.ops);
 }
 
-void dds_stream_read_key (dds_istream_t * __restrict is, char * __restrict sample, const struct ddsi_sertopic_default * __restrict topic)
+void dds_stream_read_key (dds_istream_t * __restrict is, char * __restrict sample, const struct ddsi_sertype_default * __restrict type)
 {
-  const struct ddsi_sertopic_default_desc *desc = &topic->type;
-  for (uint32_t i = 0; i < desc->m_nkeys; i++)
+  const struct ddsi_sertype_default_desc *desc = &type->type;
+  for (uint32_t i = 0; i < desc->keys.nkeys; i++)
   {
-    const uint32_t *op = desc->m_ops + desc->m_keys[i];
+    const uint32_t *op = desc->ops.ops + desc->keys.keys[i];
     char *dst = sample + op[1];
     assert (insn_key_ok_p (*op));
     switch (DDS_OP_TYPE (*op))
@@ -1467,12 +1484,12 @@ void dds_stream_read_key (dds_istream_t * __restrict is, char * __restrict sampl
   }
 }
 
-void dds_stream_write_key (dds_ostream_t * __restrict os, const char * __restrict sample, const struct ddsi_sertopic_default * __restrict topic)
+void dds_stream_write_key (dds_ostream_t * __restrict os, const char * __restrict sample, const struct ddsi_sertype_default * __restrict type)
 {
-  const struct ddsi_sertopic_default_desc *desc = &topic->type;
-  for (uint32_t i = 0; i < desc->m_nkeys; i++)
+  const struct ddsi_sertype_default_desc *desc = &type->type;
+  for (uint32_t i = 0; i < desc->keys.nkeys; i++)
   {
-    const uint32_t *insnp = desc->m_ops + desc->m_keys[i];
+    const uint32_t *insnp = desc->ops.ops + desc->keys.keys[i];
     const void *src = sample + insnp[1];
     assert (insn_key_ok_p (*insnp));
     switch (DDS_OP_TYPE (*insnp))
@@ -1527,12 +1544,12 @@ static void dds_stream_swap_insitu (void * __restrict vbuf, uint32_t size, uint3
   }
 }
 
-void dds_stream_write_keyBE (dds_ostreamBE_t * __restrict os, const char * __restrict sample, const struct ddsi_sertopic_default * __restrict topic)
+void dds_stream_write_keyBE (dds_ostreamBE_t * __restrict os, const char * __restrict sample, const struct ddsi_sertype_default * __restrict type)
 {
-  const struct ddsi_sertopic_default_desc *desc = &topic->type;
-  for (uint32_t i = 0; i < desc->m_nkeys; i++)
+  const struct ddsi_sertype_default_desc *desc = &type->type;
+  for (uint32_t i = 0; i < desc->keys.nkeys; i++)
   {
-    const uint32_t *insnp = desc->m_ops + desc->m_keys[i];
+    const uint32_t *insnp = desc->ops.ops + desc->keys.keys[i];
     const void *src = sample + insnp[1];
     assert (insn_key_ok_p (*insnp));
     switch (DDS_OP_TYPE (*insnp))
@@ -1560,9 +1577,9 @@ void dds_stream_write_keyBE (dds_ostreamBE_t * __restrict os, const char * __res
   }
 }
 #elif DDSRT_ENDIAN == DDSRT_BIG_ENDIAN
-void dds_stream_write_keyBE (dds_ostreamBE_t * __restrict os, const char * __restrict sample, const struct ddsi_sertopic_default * __restrict topic)
+void dds_stream_write_keyBE (dds_ostreamBE_t * __restrict os, const char * __restrict sample, const struct ddsi_sertype_default * __restrict type)
 {
-  dds_stream_write_key (&os->x, sample, topic);
+  dds_stream_write_key (&os->x, sample, type);
 }
 #else
 #error "DDSRT_ENDIAN neither LITTLE nor BIG"
@@ -1603,7 +1620,6 @@ static void dds_stream_extract_key_from_key_prim_op (dds_istream_t * __restrict
       void * const dst = os->m_buffer + os->m_index;
       dds_is_get_bytes (is, dst, num, align);
       os->m_index += num * align;
-      is->m_index += num * align;
       break;
     }
     case DDS_OP_VAL_SEQ: case DDS_OP_VAL_UNI: case DDS_OP_VAL_STU: {
@@ -1688,12 +1704,12 @@ static void dds_stream_extract_keyBE_from_key_prim_op (dds_istream_t * __restric
   }
 }
 
-static void dds_stream_extract_keyBE_from_key (dds_istream_t * __restrict is, dds_ostreamBE_t * __restrict os, const struct ddsi_sertopic_default * __restrict topic)
+static void dds_stream_extract_keyBE_from_key (dds_istream_t * __restrict is, dds_ostreamBE_t * __restrict os, const struct ddsi_sertype_default * __restrict type)
 {
-  const struct ddsi_sertopic_default_desc *desc = &topic->type;
-  for (uint32_t i = 0; i < desc->m_nkeys; i++)
+  const struct ddsi_sertype_default_desc *desc = &type->type;
+  for (uint32_t i = 0; i < desc->keys.nkeys; i++)
   {
-    uint32_t const * const op = desc->m_ops + desc->m_keys[i];
+    uint32_t const * const op = desc->ops.ops + desc->keys.keys[i];
     dds_stream_extract_keyBE_from_key_prim_op (is, os, op);
   }
 }
@@ -1731,7 +1747,7 @@ static const uint32_t *dds_stream_extract_key_from_data_skip_array (dds_istream_
   assert (DDS_OP_TYPE (op) == DDS_OP_VAL_ARR);
   const uint32_t subtype = DDS_OP_SUBTYPE (op);
   const uint32_t num = ops[2];
-  if (subtype > DDS_OP_VAL_BST)
+  if (subtype >= DDS_OP_VAL_BST)
   {
     const uint32_t *jsr_ops = ops + DDS_OP_ADR_JSR (ops[3]);
     const uint32_t jmp = DDS_OP_ADR_JMP (ops[3]);
@@ -1748,12 +1764,23 @@ static const uint32_t *dds_stream_extract_key_from_data_skip_array (dds_istream_
 static const uint32_t *dds_stream_extract_key_from_data_skip_sequence (dds_istream_t * __restrict is, const uint32_t * __restrict ops)
 {
   const uint32_t op = *ops;
-  const enum dds_stream_typecode type = DDS_OP_TYPE (op);
-  assert (type == DDS_OP_VAL_SEQ);
+  assert (DDS_OP_TYPE (op) == DDS_OP_VAL_SEQ);
   const uint32_t subtype = DDS_OP_SUBTYPE (op);
   const uint32_t num = dds_is_get4 (is);
-  if (num == 0)
-    return ops + 2 + (type == DDS_OP_VAL_BST || type == DDS_OP_VAL_ARR);
+  if (num == 0) {
+    switch(subtype) {
+      case DDS_OP_VAL_BST:
+        return ops + 3;
+      break;
+      case DDS_OP_VAL_SEQ: case DDS_OP_VAL_ARR: case DDS_OP_VAL_UNI: case DDS_OP_VAL_STU: {
+        const uint32_t jmp = DDS_OP_ADR_JMP (ops[3]);
+        return ops + (jmp ? jmp : 4);/* FIXME: why would jmp be 0? */
+      }
+      break;
+      default:
+        return ops + 2;
+    }
+  }
   else if (subtype > DDS_OP_VAL_BST)
   {
     const uint32_t * jsr_ops = ops + DDS_OP_ADR_JSR (ops[3]);
@@ -1764,7 +1791,7 @@ static const uint32_t *dds_stream_extract_key_from_data_skip_sequence (dds_istre
   else
   {
     dds_stream_extract_key_from_data_skip_subtype (is, num, subtype, NULL);
-    return ops + 2 + (type == DDS_OP_VAL_BST || type == DDS_OP_VAL_ARR);
+    return ops + 2 + (subtype == DDS_OP_VAL_BST);
   }
 }
 
@@ -1891,30 +1918,30 @@ static void dds_stream_extract_keyBE_from_data1 (dds_istream_t * __restrict is,
   }
 }
 
-void dds_stream_extract_key_from_data (dds_istream_t * __restrict is, dds_ostream_t * __restrict os, const struct ddsi_sertopic_default * __restrict topic)
+void dds_stream_extract_key_from_data (dds_istream_t * __restrict is, dds_ostream_t * __restrict os, const struct ddsi_sertype_default * __restrict type)
 {
-  const struct ddsi_sertopic_default_desc *desc = &topic->type;
-  uint32_t keys_remaining = desc->m_nkeys;
-  dds_stream_extract_key_from_data1 (is, os, desc->m_ops, &keys_remaining);
+  const struct ddsi_sertype_default_desc *desc = &type->type;
+  uint32_t keys_remaining = desc->keys.nkeys;
+  dds_stream_extract_key_from_data1 (is, os, desc->ops.ops, &keys_remaining);
 }
 
-void dds_stream_extract_keyBE_from_data (dds_istream_t * __restrict is, dds_ostreamBE_t * __restrict os, const struct ddsi_sertopic_default * __restrict topic)
+void dds_stream_extract_keyBE_from_data (dds_istream_t * __restrict is, dds_ostreamBE_t * __restrict os, const struct ddsi_sertype_default * __restrict type)
 {
-  const struct ddsi_sertopic_default_desc *desc = &topic->type;
-  uint32_t keys_remaining = desc->m_nkeys;
-  dds_stream_extract_keyBE_from_data1 (is, os, desc->m_ops, &keys_remaining);
+  const struct ddsi_sertype_default_desc *desc = &type->type;
+  uint32_t keys_remaining = desc->keys.nkeys;
+  dds_stream_extract_keyBE_from_data1 (is, os, desc->ops.ops, &keys_remaining);
 }
 
-void dds_stream_extract_keyhash (dds_istream_t * __restrict is, dds_keyhash_t * __restrict kh, const struct ddsi_sertopic_default * __restrict topic, const bool just_key)
+void dds_stream_extract_keyhash (dds_istream_t * __restrict is, dds_keyhash_t * __restrict kh, const struct ddsi_sertype_default * __restrict type, const bool just_key)
 {
-  const struct ddsi_sertopic_default_desc *desc = &topic->type;
+  const struct ddsi_sertype_default_desc *desc = &type->type;
   kh->m_set = 1;
-  if (desc->m_nkeys == 0)
+  if (desc->keys.nkeys == 0)
   {
     kh->m_iskey = 1;
     kh->m_keysize = 0;
   }
-  else if (desc->m_flagset & DDS_TOPIC_FIXED_KEY)
+  else if (desc->flagset & DDS_TOPIC_FIXED_KEY)
   {
     dds_ostreamBE_t os;
     kh->m_iskey = 1;
@@ -1922,9 +1949,9 @@ void dds_stream_extract_keyhash (dds_istream_t * __restrict is, dds_keyhash_t *
     os.x.m_buffer = kh->m_hash;
     os.x.m_size = 16;
     if (just_key)
-      dds_stream_extract_keyBE_from_key (is, &os, topic);
+      dds_stream_extract_keyBE_from_key (is, &os, type);
     else
-      dds_stream_extract_keyBE_from_data (is, &os, topic);
+      dds_stream_extract_keyBE_from_data (is, &os, type);
     assert (os.x.m_index <= 16);
     kh->m_keysize = (unsigned)os.x.m_index & 0x1f;
   }
@@ -1936,9 +1963,9 @@ void dds_stream_extract_keyhash (dds_istream_t * __restrict is, dds_keyhash_t *
     kh->m_keysize = 16;
     dds_ostreamBE_init (&os, 0);
     if (just_key)
-      dds_stream_extract_keyBE_from_key (is, &os, topic);
+      dds_stream_extract_keyBE_from_key (is, &os, type);
     else
-      dds_stream_extract_keyBE_from_data (is, &os, topic);
+      dds_stream_extract_keyBE_from_data (is, &os, type);
     ddsrt_md5_init (&md5st);
     ddsrt_md5_append (&md5st, os.x.m_buffer, os.x.m_index);
     ddsrt_md5_finish (&md5st, kh->m_hash);
@@ -1953,6 +1980,9 @@ void dds_stream_extract_keyhash (dds_istream_t * __restrict is, dds_keyhash_t *
  *******************************************************************************************/
 
 /* Returns true if buffer not yet exhausted, false otherwise */
+static bool prtf (char * __restrict *buf, size_t * __restrict bufsize, const char *fmt, ...)
+  ddsrt_attribute_format_printf(3, 4);
+
 static bool prtf (char * __restrict *buf, size_t * __restrict bufsize, const char *fmt, ...)
 {
   va_list ap;
@@ -2228,19 +2258,19 @@ static bool dds_stream_print_sample1 (char * __restrict *buf, size_t * __restric
   return cont;
 }
 
-size_t dds_stream_print_sample (dds_istream_t * __restrict is, const struct ddsi_sertopic_default * __restrict topic, char * __restrict buf, size_t bufsize)
+size_t dds_stream_print_sample (dds_istream_t * __restrict is, const struct ddsi_sertype_default * __restrict type, char * __restrict buf, size_t bufsize)
 {
-  (void) dds_stream_print_sample1 (&buf, &bufsize, is, topic->type.m_ops, true);
+  (void) dds_stream_print_sample1 (&buf, &bufsize, is, type->type.ops.ops, true);
   return bufsize;
 }
 
-size_t dds_stream_print_key (dds_istream_t * __restrict is, const struct ddsi_sertopic_default * __restrict topic, char * __restrict buf, size_t bufsize)
+size_t dds_stream_print_key (dds_istream_t * __restrict is, const struct ddsi_sertype_default * __restrict type, char * __restrict buf, size_t bufsize)
 {
-  const struct ddsi_sertopic_default_desc *desc = &topic->type;
+  const struct ddsi_sertype_default_desc *desc = &type->type;
   bool cont = prtf (&buf, &bufsize, ":k:{");
-  for (uint32_t i = 0; cont && i < desc->m_nkeys; i++)
+  for (uint32_t i = 0; cont && i < desc->keys.nkeys; i++)
   {
-    const uint32_t *op = desc->m_ops + desc->m_keys[i];
+    const uint32_t *op = desc->ops.ops + desc->keys.keys[i];
     assert (insn_key_ok_p (*op));
     switch (DDS_OP_TYPE (*op))
     {
