@@ -10,8 +10,10 @@
  * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
  */
 #include "ddsi_eth.h"
+#include "dds/ddsi/q_protocol.h" // for NN_LOCATOR_KIND_...
+#include "dds/ddsi/q_config.h" // for transport_selector
 
-int ddsi_eth_enumerate_interfaces (ddsi_tran_factory_t fact, enum transport_selector transport_selector, ddsrt_ifaddrs_t **ifs)
+int ddsi_eth_enumerate_interfaces (ddsi_tran_factory_t fact, enum ddsi_transport_selector transport_selector, ddsrt_ifaddrs_t **ifs)
 {
     int afs[] = { AF_INET, DDSRT_AF_TERM };
 
@@ -19,8 +21,8 @@ int ddsi_eth_enumerate_interfaces (ddsi_tran_factory_t fact, enum transport_sele
     (void)transport_selector;
 
 #if DDSRT_HAVE_IPV6
-    if (transport_selector == TRANS_TCP6 ||
-        transport_selector == TRANS_UDP6)
+    if (transport_selector == DDSI_TRANS_TCP6 ||
+        transport_selector == DDSI_TRANS_UDP6)
     {
       afs[0] = AF_INET6;
     }
