@@ -36,7 +36,7 @@
 #include "dds/ddsi/q_lease.h"
 #include "dds/ddsi/ddsi_tkmap.h"
 #include "dds/ddsi/ddsi_serdata.h"
-#include "dds/ddsi/ddsi_sertopic.h"
+#include "dds/ddsi/ddsi_sertype.h"
 #include "dds/ddsi/ddsi_security_omg.h"
 
 #include "dds/ddsi/sysdeps.h"
@@ -194,9 +194,6 @@ struct nn_xmsg *writer_hbcontrol_create_heartbeat (struct writer *wr, const stru
   if (prd_guid == NULL)
   {
     nn_xmsg_setdstN (msg, wr->as, wr->as_group);
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-    nn_xmsg_setencoderid (msg, wr->partition_id);
-#endif
     add_Heartbeat (msg, wr, whcst, hbansreq, 0, to_entityid (NN_ENTITYID_UNKNOWN), issync);
   }
   else
@@ -211,9 +208,6 @@ struct nn_xmsg *writer_hbcontrol_create_heartbeat (struct writer *wr, const stru
     /* set the destination explicitly to the unicast destination and the fourth
        param of add_Heartbeat needs to be the guid of the reader */
     nn_xmsg_setdstPRD (msg, prd);
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-    nn_xmsg_setencoderid (msg, wr->partition_id);
-#endif
     // send to all readers in the participant: whether or not the entityid is set affects
     // the retransmit requests
     add_Heartbeat (msg, wr, whcst, hbansreq, 0, to_entityid (NN_ENTITYID_UNKNOWN), issync);
@@ -277,10 +271,12 @@ struct nn_xmsg *writer_hbcontrol_piggyback (struct writer *wr, const struct whc_
   struct hbcontrol * const hbc = &wr->hbcontrol;
   uint32_t last_packetid;
   ddsrt_mtime_t tlast;
+  ddsrt_mtime_t t_of_last_hb;
   struct nn_xmsg *msg;
 
   tlast = hbc->t_of_last_write;
   last_packetid = hbc->last_packetid;
+  t_of_last_hb = hbc->t_of_last_hb;
 
   hbc->t_of_last_write = tnow;
   hbc->last_packetid = packetid;
@@ -295,12 +291,18 @@ struct nn_xmsg *writer_hbcontrol_piggyback (struct writer *wr, const struct whc_
     /* So we force a heartbeat in - but we also rely on our caller to
        send the packet out */
     msg = writer_hbcontrol_create_heartbeat (wr, whcst, tnow, *hbansreq, 1);
-  } else if (last_packetid != packetid) {
+  } else if (last_packetid != packetid && tnow.v - t_of_last_hb.v > DDS_USECS (100)) {
     /* If we crossed a packet boundary since the previous write,
        piggyback a heartbeat, with *hbansreq determining whether or
        not an ACK is needed.  We don't force the packet out either:
        this is just to ensure a regular flow of ACKs for cleaning up
-       the WHC & for allowing readers to NACK missing samples. */
+       the WHC & for allowing readers to NACK missing samples.
+
+       Still rate-limit: if there are new readers that haven't sent an
+       an ACK yet, the FINAL flag will be cleared and so we get an ACK
+       storm if writing at a high rate without batching which eats up
+       a *large* amount of time because there are out-of-order readers
+       present. */
     msg = writer_hbcontrol_create_heartbeat (wr, whcst, tnow, *hbansreq, 1);
   } else {
     *hbansreq = 0;
@@ -321,7 +323,7 @@ struct nn_xmsg *writer_hbcontrol_piggyback (struct writer *wr, const struct whc_
   return msg;
 }
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
 struct nn_xmsg *writer_hbcontrol_p2p(struct writer *wr, const struct whc_state *whcst, int hbansreq, struct proxy_reader *prd)
 {
   struct ddsi_domaingv const * const gv = wr->e.gv;
@@ -343,9 +345,6 @@ struct nn_xmsg *writer_hbcontrol_p2p(struct writer *wr, const struct whc_state *
   /* set the destination explicitly to the unicast destination and the fourth
      param of add_Heartbeat needs to be the guid of the reader */
   nn_xmsg_setdstPRD (msg, prd);
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-  nn_xmsg_setencoderid (msg, wr->partition_id);
-#endif
   add_Heartbeat (msg, wr, whcst, hbansreq, 0, prd->e.guid.entityid, 1);
 
   if (nn_xmsg_size(msg) == 0)
@@ -463,11 +462,6 @@ static dds_return_t create_fragment_message_simple (struct writer *wr, seqno_t s
   if ((*pmsg = nn_xmsg_new (gv->xmsgpool, &wr->e.guid, wr->c.pp, sizeof (InfoTimestamp_t) + sizeof (Data_t) + expected_inline_qos_size, NN_XMSG_KIND_DATA)) == NULL)
     return DDS_RETCODE_OUT_OF_RESOURCES;
 
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-  /* use the partition_id from the writer to select the proper encoder */
-  nn_xmsg_setencoderid (*pmsg, wr->partition_id);
-#endif
-
   nn_xmsg_setdstN (*pmsg, wr->as, wr->as_group);
   nn_xmsg_setmaxdelay (*pmsg, wr->xqos->latency_budget.duration);
   nn_xmsg_add_timestamp (*pmsg, serdata->timestamp);
@@ -485,7 +479,7 @@ static dds_return_t create_fragment_message_simple (struct writer *wr, seqno_t s
     nn_xmsg_setwriterseq (*pmsg, &wr->e.guid, seq);
 
   /* Adding parameters means potential reallocing, so sm, ddcmn now likely become invalid */
-  if (wr->include_keyhash)
+  if (wr->num_readers_requesting_keyhash > 0)
     nn_xmsg_addpar_keyhash (*pmsg, serdata, wr->force_md5_keyhash);
   if (serdata->statusinfo)
     nn_xmsg_addpar_statusinfo (*pmsg, serdata->statusinfo);
@@ -549,11 +543,6 @@ dds_return_t create_fragment_message (struct writer *wr, seqno_t seq, const stru
   if ((*pmsg = nn_xmsg_new (gv->xmsgpool, &wr->e.guid, wr->c.pp, sizeof (InfoTimestamp_t) + sizeof (DataFrag_t) + expected_inline_qos_size, xmsg_kind)) == NULL)
     return DDS_RETCODE_OUT_OF_RESOURCES;
 
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-  /* use the partition_id from the writer to select the proper encoder */
-  nn_xmsg_setencoderid (*pmsg, wr->partition_id);
-#endif
-
   if (prd)
   {
     nn_xmsg_setdstPRD (*pmsg, prd);
@@ -639,7 +628,7 @@ dds_return_t create_fragment_message (struct writer *wr, seqno_t seq, const stru
   {
     int rc;
     /* Adding parameters means potential reallocing, so sm, ddcmn now likely become invalid */
-    if (wr->include_keyhash)
+    if (wr->num_readers_requesting_keyhash > 0)
     {
       nn_xmsg_addpar_keyhash (*pmsg, serdata, wr->force_md5_keyhash);
     }
@@ -683,9 +672,6 @@ static void create_HeartbeatFrag (struct writer *wr, seqno_t seq, unsigned fragn
   ASSERT_MUTEX_HELD (&wr->e.lock);
   if ((*pmsg = nn_xmsg_new (gv->xmsgpool, &wr->e.guid, wr->c.pp, sizeof (HeartbeatFrag_t), NN_XMSG_KIND_CONTROL)) == NULL)
     return; /* ignore out-of-memory: HeartbeatFrag is only advisory anyway */
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-  nn_xmsg_setencoderid (*pmsg, wr->partition_id);
-#endif
   if (prd)
     nn_xmsg_setdstPRD (*pmsg, prd);
   else
@@ -735,9 +721,6 @@ dds_return_t write_hb_liveliness (struct ddsi_domaingv * const gv, struct ddsi_g
     return DDS_RETCODE_OUT_OF_RESOURCES;
   ddsrt_mutex_lock (&wr->e.lock);
   nn_xmsg_setdstN (msg, wr->as, wr->as_group);
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-  nn_xmsg_setencoderid (msg, wr->partition_id);
-#endif
   whc_get_state (wr->whc, &whcst);
   add_Heartbeat (msg, wr, &whcst, 0, 1, to_entityid (NN_ENTITYID_UNKNOWN), 1);
   ddsrt_mutex_unlock (&wr->e.lock);
@@ -940,11 +923,11 @@ static int insert_sample_in_whc (struct writer *wr, seqno_t seq, struct ddsi_pli
     ETRACE (wr, "write_sample "PGUIDFMT" #%"PRId64, PGUID (wr->e.guid), seq);
     if (plist != 0 && (plist->present & PP_COHERENT_SET))
       ETRACE (wr, " C#%"PRId64"", fromSN (plist->coherent_set_seqno));
-    ETRACE (wr, ": ST%"PRIu32" %s/%s:%s%s\n", serdata->statusinfo, wr->topic->name, wr->topic->type_name, ppbuf, tmp < (int) sizeof (ppbuf) ? "" : " (trunc)");
+    ETRACE (wr, ": ST%"PRIu32" %s/%s:%s%s\n", serdata->statusinfo, wr->xqos->topic_name, wr->type->type_name, ppbuf, tmp < (int) sizeof (ppbuf) ? "" : " (trunc)");
   }
 
   assert (wr->reliable || have_reliable_subs (wr) == 0);
-#ifdef DDSI_INCLUDE_DEADLINE_MISSED
+#ifdef DDS_HAS_DEADLINE_MISSED
   /* If deadline missed duration is not infinite, the sample is inserted in
      the whc so that the instance is created (or renewed) in the whc and the deadline
      missed event is registered. The sample is removed immediately after inserting it
@@ -955,7 +938,7 @@ static int insert_sample_in_whc (struct writer *wr, seqno_t seq, struct ddsi_pli
   if ((wr->reliable && have_reliable_subs (wr)) || wr_deadline || wr->handle_as_transient_local)
   {
     ddsrt_mtime_t exp = DDSRT_MTIME_NEVER;
-#ifdef DDSI_INCLUDE_LIFESPAN
+#ifdef DDS_HAS_LIFESPAN
     /* Don't set expiry for samples with flags unregister or dispose, because these are required
      * for sample lifecycle and should always be delivered to the reader so that is can clean up
      * its history cache. */
@@ -964,7 +947,7 @@ static int insert_sample_in_whc (struct writer *wr, seqno_t seq, struct ddsi_pli
 #endif
     res = ((insres = whc_insert (wr->whc, writer_max_drop_seq (wr), seq, exp, plist, serdata, tk)) < 0) ? insres : 1;
 
-#ifdef DDSI_INCLUDE_DEADLINE_MISSED
+#ifdef DDS_HAS_DEADLINE_MISSED
     if (!(wr->reliable && have_reliable_subs (wr)) && !wr->handle_as_transient_local)
     {
       /* Sample was inserted only because writer has deadline, so we'll remove the sample from whc */
@@ -1202,7 +1185,7 @@ static int write_sample_eot (struct thread_state1 * const ts1, struct nn_xpack *
   assert (gc_allowed || (wr->xqos->history.kind == DDS_HISTORY_KEEP_LAST && wr->whc_low == INT32_MAX));
   (void) gc_allowed;
 
-  if (ddsi_serdata_size (serdata) > gv->config.max_sample_size)
+  if (gv->config.max_sample_size < (uint32_t) INT32_MAX && ddsi_serdata_size (serdata) > gv->config.max_sample_size)
   {
     char ppbuf[1024];
     int tmp;
@@ -1210,7 +1193,7 @@ static int write_sample_eot (struct thread_state1 * const ts1, struct nn_xpack *
     tmp = sizeof (ppbuf) - 1;
     GVWARNING ("dropping oversize (%"PRIu32" > %"PRIu32") sample from local writer "PGUIDFMT" %s/%s:%s%s\n",
                ddsi_serdata_size (serdata), gv->config.max_sample_size,
-               PGUID (wr->e.guid), wr->topic->name, wr->topic->type_name, ppbuf,
+               PGUID (wr->e.guid), wr->xqos->topic_name, wr->type->type_name, ppbuf,
                tmp < (int) sizeof (ppbuf) ? "" : " (trunc)");
     r = DDS_RETCODE_BAD_PARAMETER;
     goto drop;
