@@ -24,7 +24,6 @@
 #include "dds/ddsrt/random.h"
 
 #include "dds/ddsrt/avl.h"
-#include "dds/ddsrt/thread_pool.h"
 
 #include "dds/ddsi/q_protocol.h"
 #include "dds/ddsi/ddsi_xqos.h"
@@ -63,7 +62,8 @@ struct nn_xmsg_chain_elem {
 enum nn_xmsg_dstmode {
   NN_XMSG_DST_UNSET,
   NN_XMSG_DST_ONE,
-  NN_XMSG_DST_ALL
+  NN_XMSG_DST_ALL,
+  NN_XMSG_DST_ALL_UC
 };
 
 struct nn_xmsg {
@@ -73,15 +73,12 @@ struct nn_xmsg {
   int have_params;
   struct ddsi_serdata *refd_payload;
   ddsrt_iovec_t refd_payload_iov;
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   /* Used as pointer to contain encoded payload to which iov can alias. */
   unsigned char *refd_payload_encoded;
   nn_msg_sec_info_t sec_info;
 #endif
   int64_t maxdelay;
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-  uint32_t encoderid;
-#endif
 
   /* Backref for late updating of available sequence numbers, and
      merging of retransmits. */
@@ -102,12 +99,15 @@ struct nn_xmsg {
   enum nn_xmsg_dstmode dstmode;
   union {
     struct {
-      nn_locator_t loc;  /* send just to this locator */
+      ddsi_xlocator_t loc;  /* send just to this locator */
     } one;
     struct {
       struct addrset *as;       /* send to all addresses in set */
       struct addrset *as_group; /* send to one address in set */
     } all;
+    struct {
+      struct addrset *as;       /* send to all unicast addresses in set */
+    } all_uc;
   } dstaddr;
 
   struct nn_xmsg_chain_elem link;
@@ -136,7 +136,7 @@ struct nn_xmsg_chain {
   struct nn_xmsg_chain_elem *latest;
 };
 
-#ifdef DDSI_INCLUDE_BANDWIDTH_LIMITING
+#ifdef DDS_HAS_BANDWIDTH_LIMITING
 #define NN_BW_UNLIMITED (0)
 
 struct nn_bw_limiter {
@@ -146,51 +146,6 @@ struct nn_bw_limiter {
 };
 #endif
 
-///////////////////////////
-typedef struct {
-  ddsrt_mutex_t mtx;
-  uint32_t value;
-  ddsrt_cond_t cv;
-} ddsi_sem_t;
-
-static dds_return_t
-ddsi_sem_init (ddsi_sem_t *sem, uint32_t value)
-{
-  sem->value = value;
-  ddsrt_mutex_init (&sem->mtx);
-  ddsrt_cond_init (&sem->cv);
-  return DDS_RETCODE_OK;
-}
-
-static dds_return_t
-ddsi_sem_destroy (ddsi_sem_t *sem)
-{
-  ddsrt_cond_destroy (&sem->cv);
-  ddsrt_mutex_destroy (&sem->mtx);
-  return DDS_RETCODE_OK;
-}
-
-static dds_return_t
-ddsi_sem_post (ddsi_sem_t *sem)
-{
-  ddsrt_mutex_lock (&sem->mtx);
-  if (sem->value++ == 0)
-    ddsrt_cond_signal (&sem->cv);
-  ddsrt_mutex_unlock (&sem->mtx);
-  return DDS_RETCODE_OK;
-}
-
-static dds_return_t
-ddsi_sem_wait (ddsi_sem_t *sem)
-{
-  ddsrt_mutex_lock (&sem->mtx);
-  while (sem->value == 0)
-    ddsrt_cond_wait (&sem->cv, &sem->mtx);
-  ddsrt_mutex_unlock (&sem->mtx);
-  return DDS_RETCODE_OK;
-}
-///////////////////////////
-
 struct nn_xpack
 {
   struct nn_xpack *sendq_next;
@@ -203,8 +158,6 @@ struct nn_xpack
   unsigned packetid;
   ddsrt_atomic_uint32_t calls;
   uint32_t call_flags;
-  ddsi_tran_conn_t conn;
-  ddsi_sem_t sem;
   size_t niov;
   ddsrt_iovec_t *iov;
   enum nn_xmsg_dstmode dstmode;
@@ -212,25 +165,29 @@ struct nn_xpack
 
   union
   {
-    nn_locator_t loc; /* send just to this locator */
+    ddsi_xlocator_t loc; /* send just to this locator */
     struct
     {
       struct addrset *as;        /* send to all addresses in set */
       struct addrset *as_group;  /* send to one address in set */
     } all;
+    struct
+    {
+      struct addrset *as;        /* send to all unicast addresses in set */
+    } all_uc;
   } dstaddr;
 
   bool includes_rexmit;
   struct nn_xmsg_chain included_msgs;
 
-#ifdef DDSI_INCLUDE_BANDWIDTH_LIMITING
+#ifdef DDS_HAS_BANDWIDTH_LIMITING
   struct nn_bw_limiter limiter;
 #endif
 
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
+#ifdef DDS_HAS_NETWORK_PARTITIONS
   uint32_t encoderId;
-#endif /* DDSI_INCLUDE_NETWORK_PARTITIONS */
-#ifdef DDSI_INCLUDE_SECURITY
+#endif /* DDS_HAS_NETWORK_PARTITIONS */
+#ifdef DDS_HAS_SECURITY
   nn_msg_sec_info_t sec_info;
 #endif
 };
@@ -293,14 +250,11 @@ static void nn_xmsg_reinit (struct nn_xmsg *m, enum nn_xmsg_kind kind)
   m->dstmode = NN_XMSG_DST_UNSET;
   m->kind = kind;
   m->maxdelay = 0;
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   m->refd_payload_encoded = NULL;
   m->sec_info.use_rtps_encoding = 0;
   m->sec_info.src_pp_handle = 0;
   m->sec_info.dst_pp_handle = 0;
-#endif
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-  m->encoderid = 0;
 #endif
   memset (&m->kindspecific, 0, sizeof (m->kindspecific));
 }
@@ -347,7 +301,7 @@ struct nn_xmsg *nn_xmsg_new (struct nn_xmsgpool *pool, const ddsi_guid_t *src_gu
     return NULL;
   m->data->src.guid_prefix = nn_hton_guid_prefix (src_guid->prefix);
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   m->sec_info.use_rtps_encoding = 0;
   if (pp && q_omg_participant_is_secure(pp))
   {
@@ -375,13 +329,21 @@ void nn_xmsg_free (struct nn_xmsg *m)
   struct nn_xmsgpool *pool = m->pool;
   if (m->refd_payload)
     ddsi_serdata_to_ser_unref (m->refd_payload, &m->refd_payload_iov);
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   ddsrt_free(m->refd_payload_encoded);
 #endif
-  if (m->dstmode == NN_XMSG_DST_ALL)
+  switch (m->dstmode)
   {
-    unref_addrset (m->dstaddr.all.as);
-    unref_addrset (m->dstaddr.all.as_group);
+    case NN_XMSG_DST_UNSET:
+    case NN_XMSG_DST_ONE:
+      break;
+    case NN_XMSG_DST_ALL:
+      unref_addrset (m->dstaddr.all.as);
+      unref_addrset (m->dstaddr.all.as_group);
+      break;
+    case NN_XMSG_DST_ALL_UC:
+      unref_addrset (m->dstaddr.all_uc.as);
+      break;
   }
   /* Only cache the smallest xmsgs; data messages store the payload by reference and are small */
   if (m->maxsz > NN_XMSG_CHUNK_SIZE || !nn_freelist_push (&pool->freelist, m))
@@ -541,7 +503,7 @@ void nn_xmsg_submsg_setnext (struct nn_xmsg *msg, struct nn_xmsg_marker marker)
     ((unsigned)(msg->data->payload + msg->sz + plsize - (char *) hdr) - RTPS_SUBMESSAGE_HEADER_SIZE);
 }
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
 
 size_t nn_xmsg_submsg_size (struct nn_xmsg *msg, struct nn_xmsg_marker marker)
 {
@@ -630,7 +592,7 @@ void nn_xmsg_submsg_append_refd_payload(struct nn_xmsg *msg, struct nn_xmsg_mark
   }
 }
 
-#endif /* DDSI_INCLUDE_SECURITY */
+#endif /* DDS_HAS_SECURITY */
 
 void *nn_xmsg_submsg_from_marker (struct nn_xmsg *msg, struct nn_xmsg_marker marker)
 {
@@ -706,7 +668,7 @@ void nn_xmsg_serdata (struct nn_xmsg *m, struct ddsi_serdata *serdata, size_t of
     assert (m->refd_payload == NULL);
     m->refd_payload = ddsi_serdata_to_ser_ref (serdata, off, len4, &m->refd_payload_iov);
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
     assert (m->refd_payload_encoded == NULL);
     /* When encoding is necessary, m->refd_payload_encoded will be allocated
      * and m->refd_payload_iov contents will change to point to that buffer.
@@ -726,13 +688,10 @@ void nn_xmsg_serdata (struct nn_xmsg *m, struct ddsi_serdata *serdata, size_t of
   }
 }
 
-void nn_xmsg_setdst1 (struct ddsi_domaingv *gv, struct nn_xmsg *m, const ddsi_guid_prefix_t *gp, const nn_locator_t *loc)
+static void nn_xmsg_setdst1_common (struct ddsi_domaingv *gv, struct nn_xmsg *m, const ddsi_guid_prefix_t *gp)
 {
-  assert (m->dstmode == NN_XMSG_DST_UNSET);
-  m->dstmode = NN_XMSG_DST_ONE;
-  m->dstaddr.one.loc = *loc;
   m->data->dst.guid_prefix = nn_hton_guid_prefix (*gp);
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   if (m->sec_info.use_rtps_encoding && !m->sec_info.dst_pp_handle)
   {
     struct proxy_participant *proxypp;
@@ -746,10 +705,18 @@ void nn_xmsg_setdst1 (struct ddsi_domaingv *gv, struct nn_xmsg *m, const ddsi_gu
       m->sec_info.dst_pp_handle = q_omg_security_get_remote_participant_handle(proxypp);
   }
 #else
-    DDSRT_UNUSED_ARG(gv);
+  DDSRT_UNUSED_ARG(gv);
 #endif
 }
 
+void nn_xmsg_setdst1 (struct ddsi_domaingv *gv, struct nn_xmsg *m, const ddsi_guid_prefix_t *gp, const ddsi_xlocator_t *loc)
+{
+  assert (m->dstmode == NN_XMSG_DST_UNSET);
+  m->dstmode = NN_XMSG_DST_ONE;
+  m->dstaddr.one.loc = *loc;
+  nn_xmsg_setdst1_common (gv, m, gp);
+}
+
 bool nn_xmsg_getdst1prefix (struct nn_xmsg *m, ddsi_guid_prefix_t *gp)
 {
   if (m->dstmode == NN_XMSG_DST_ONE)
@@ -762,18 +729,44 @@ bool nn_xmsg_getdst1prefix (struct nn_xmsg *m, ddsi_guid_prefix_t *gp)
 
 void nn_xmsg_setdstPRD (struct nn_xmsg *m, const struct proxy_reader *prd)
 {
-  nn_locator_t loc;
   // only accepting endpoints that have an address
-  addrset_any_uc_else_mc_nofail (prd->c.as, &loc);
-  nn_xmsg_setdst1 (prd->e.gv, m, &prd->e.guid.prefix, &loc);
+  assert (m->dstmode == NN_XMSG_DST_UNSET);
+  if (!prd->redundant_networking)
+  {
+    ddsi_xlocator_t loc;
+    addrset_any_uc_else_mc_nofail (prd->c.as, &loc);
+    nn_xmsg_setdst1 (prd->e.gv, m, &prd->e.guid.prefix, &loc);
+  }
+  else
+  {
+    // FIXME: maybe I should work on the merging instead ...
+    if (m->kind == NN_XMSG_KIND_DATA_REXMIT)
+      m->kind = NN_XMSG_KIND_DATA_REXMIT_NOMERGE;
+    m->dstmode = NN_XMSG_DST_ALL_UC;
+    m->dstaddr.all_uc.as = ref_addrset (prd->c.as);
+    nn_xmsg_setdst1_common (prd->e.gv, m, &prd->e.guid.prefix);
+  }
 }
 
 void nn_xmsg_setdstPWR (struct nn_xmsg *m, const struct proxy_writer *pwr)
 {
-  nn_locator_t loc;
   // only accepting endpoints that have an address
-  addrset_any_uc_else_mc_nofail (pwr->c.as, &loc);
-  nn_xmsg_setdst1 (pwr->e.gv, m, &pwr->e.guid.prefix, &loc);
+  assert (m->dstmode == NN_XMSG_DST_UNSET);
+  if (!pwr->redundant_networking)
+  {
+    ddsi_xlocator_t loc;
+    addrset_any_uc_else_mc_nofail (pwr->c.as, &loc);
+    nn_xmsg_setdst1 (pwr->e.gv, m, &pwr->e.guid.prefix, &loc);
+  }
+  else
+  {
+    // FIXME: maybe I should work on the merging instead ...
+    if (m->kind == NN_XMSG_KIND_DATA_REXMIT)
+      m->kind = NN_XMSG_KIND_DATA_REXMIT_NOMERGE;
+    m->dstmode = NN_XMSG_DST_ALL_UC;
+    m->dstaddr.all_uc.as = ref_addrset (pwr->c.as);
+    nn_xmsg_setdst1_common (pwr->e.gv, m, &pwr->e.guid.prefix);
+  }
 }
 
 void nn_xmsg_setdstN (struct nn_xmsg *m, struct addrset *as, struct addrset *as_group)
@@ -786,7 +779,7 @@ void nn_xmsg_setdstN (struct nn_xmsg *m, struct addrset *as, struct addrset *as_
 
 void nn_xmsg_set_data_readerId (struct nn_xmsg *m, ddsi_entityid_t *readerId)
 {
-  assert (m->kind == NN_XMSG_KIND_DATA_REXMIT);
+  assert (m->kind == NN_XMSG_KIND_DATA_REXMIT || m->kind == NN_XMSG_KIND_DATA_REXMIT_NOMERGE);
   assert (m->kindspecific.data.readerId_off == 0);
   assert ((char *) readerId > m->data->payload);
   assert ((char *) readerId < m->data->payload + m->sz);
@@ -795,7 +788,7 @@ void nn_xmsg_set_data_readerId (struct nn_xmsg *m, ddsi_entityid_t *readerId)
 
 static void clear_readerId (struct nn_xmsg *m)
 {
-  assert (m->kind == NN_XMSG_KIND_DATA_REXMIT);
+  assert (m->kind == NN_XMSG_KIND_DATA_REXMIT || m->kind == NN_XMSG_KIND_DATA_REXMIT_NOMERGE);
   assert (m->kindspecific.data.readerId_off != 0);
   *((ddsi_entityid_t *) (m->data->payload + m->kindspecific.data.readerId_off)) =
     nn_hton_entityid (to_entityid (NN_ENTITYID_UNKNOWN));
@@ -803,7 +796,7 @@ static void clear_readerId (struct nn_xmsg *m)
 
 static ddsi_entityid_t load_readerId (const struct nn_xmsg *m)
 {
-  assert (m->kind == NN_XMSG_KIND_DATA_REXMIT);
+  assert (m->kind == NN_XMSG_KIND_DATA_REXMIT || m->kind == NN_XMSG_KIND_DATA_REXMIT_NOMERGE);
   assert (m->kindspecific.data.readerId_off != 0);
   return nn_ntoh_entityid (*((ddsi_entityid_t *) (m->data->payload + m->kindspecific.data.readerId_off)));
 }
@@ -840,6 +833,10 @@ int nn_xmsg_merge_rexmit_destinations_wrlock_held (struct ddsi_domaingv *gv, str
       GVTRACE ("*->*)");
       return 1;
 
+    case NN_XMSG_DST_ALL_UC:
+      GVTRACE ("all-uc)");
+      return 0;
+
     case NN_XMSG_DST_ONE:
       switch (madd->dstmode)
       {
@@ -847,6 +844,10 @@ int nn_xmsg_merge_rexmit_destinations_wrlock_held (struct ddsi_domaingv *gv, str
           assert (0);
           return 0;
 
+        case NN_XMSG_DST_ALL_UC:
+          GVTRACE ("all-uc)");
+          return 0;
+
         case NN_XMSG_DST_ALL:
           GVTRACE ("1+*->*)");
           clear_readerId (m);
@@ -905,15 +906,6 @@ int nn_xmsg_setmaxdelay (struct nn_xmsg *msg, int64_t maxdelay)
   return 0;
 }
 
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-int nn_xmsg_setencoderid (struct nn_xmsg *msg, uint32_t encoderid)
-{
-  assert (msg->encoderid == 0);
-  msg->encoderid = encoderid;
-  return 0;
-}
-#endif
-
 void nn_xmsg_setwriterseq (struct nn_xmsg *msg, const ddsi_guid_t *wrguid, seqno_t wrseq)
 {
   msg->kindspecific.data.wrguid = *wrguid;
@@ -926,31 +918,27 @@ void nn_xmsg_setwriterseq_fragid (struct nn_xmsg *msg, const ddsi_guid_t *wrguid
   msg->kindspecific.data.wrfragid = wrfragid;
 }
 
-void *nn_xmsg_addpar_bo (struct nn_xmsg *m, nn_parameterid_t pid, size_t len, bool be)
+void *nn_xmsg_addpar_bo (struct nn_xmsg *m, nn_parameterid_t pid, size_t len, enum ddsrt_byte_order_selector bo)
 {
-#define BO2U(x)  (be ? ddsrt_toBE2u((x)) : (x))
-
   const size_t len4 = (len + 3) & ~(size_t)3; /* must alloc a multiple of 4 */
   nn_parameter_t *phdr;
   char *p;
   assert (len4 < UINT16_MAX); /* FIXME: return error */
   m->have_params = 1;
   phdr = nn_xmsg_append (m, NULL, sizeof (nn_parameter_t) + len4);
-  phdr->parameterid = BO2U(pid);
-  phdr->length = BO2U((uint16_t) len4);
+  phdr->parameterid = ddsrt_toBO2u(bo, pid);
+  phdr->length = ddsrt_toBO2u(bo, (uint16_t) len4);
   p = (char *) (phdr + 1);
   /* zero out padding bytes added to satisfy parameter alignment: this way
      valgrind can tell us where we forgot to initialize something */
   while (len < len4)
     p[len++] = 0;
   return p;
-
-#undef BO2U
 }
 
 void *nn_xmsg_addpar (struct nn_xmsg *m, nn_parameterid_t pid, size_t len)
 {
-  return nn_xmsg_addpar_bo(m, pid, len, false);
+  return nn_xmsg_addpar_bo(m, pid, len, DDSRT_BOSEL_NATIVE);
 }
 
 void nn_xmsg_addpar_keyhash (struct nn_xmsg *m, const struct ddsi_serdata *serdata, bool force_md5)
@@ -989,9 +977,9 @@ void nn_xmsg_addpar_sentinel (struct nn_xmsg * m)
   nn_xmsg_addpar (m, PID_SENTINEL, 0);
 }
 
-void nn_xmsg_addpar_sentinel_bo (struct nn_xmsg * m, bool be)
+void nn_xmsg_addpar_sentinel_bo (struct nn_xmsg * m, enum ddsrt_byte_order_selector bo)
 {
-  nn_xmsg_addpar_bo (m, PID_SENTINEL, 0, be);
+  nn_xmsg_addpar_bo (m, PID_SENTINEL, 0, bo);
 }
 
 int nn_xmsg_addpar_sentinel_ifparam (struct nn_xmsg * m)
@@ -1059,7 +1047,7 @@ static void nn_xmsg_chain_add (struct nn_xmsg_chain *chain, struct nn_xmsg *m)
   chain->latest = &m->link;
 }
 
-#ifdef DDSI_INCLUDE_BANDWIDTH_LIMITING
+#ifdef DDS_HAS_BANDWIDTH_LIMITING
 /* BW_LIMITER ----------------------------------------------------------
 
    Helper for XPACKS, that contain the configuration and state to handle Bandwidth limitation.*/
@@ -1121,7 +1109,7 @@ static void nn_bw_limit_init (struct nn_bw_limiter *limiter, uint32_t bandwidth_
   else
     limiter->last_update.v = 0;
 }
-#endif /* DDSI_INCLUDE_BANDWIDTH_LIMITING */
+#endif /* DDS_HAS_BANDWIDTH_LIMITING */
 
 /* XPACK ---------------------------------------------------------------
 
@@ -1138,28 +1126,24 @@ static void nn_xpack_reinit (struct nn_xpack *xp)
   xp->includes_rexmit = false;
   xp->included_msgs.latest = NULL;
   xp->maxdelay = DDS_INFINITY;
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   xp->sec_info.use_rtps_encoding = 0;
 #endif
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
+#ifdef DDS_HAS_NETWORK_PARTITIONS
   xp->encoderId = 0;
 #endif
   xp->packetid++;
 }
 
-struct nn_xpack * nn_xpack_new (ddsi_tran_conn_t conn, uint32_t bw_limit, bool async_mode)
+struct nn_xpack * nn_xpack_new (struct ddsi_domaingv *gv, uint32_t bw_limit, bool async_mode)
 {
   struct nn_xpack *xp;
 
-  /* Disallow setting async_mode if not configured to enable async mode: this way we
-     can avoid starting the async send thread altogether */
-  assert (!async_mode || conn->m_base.gv->config.xpack_send_async);
-
   xp = ddsrt_malloc (sizeof (*xp));
   memset (xp, 0, sizeof (*xp));
   xp->async_mode = async_mode;
   xp->iov = NULL;
-  xp->gv = conn->m_base.gv;
+  xp->gv = gv;
 
   /* Fixed header fields, initialized just once */
   xp->hdr.protocol.id[0] = 'R';
@@ -1176,13 +1160,9 @@ struct nn_xpack * nn_xpack_new (ddsi_tran_conn_t conn, uint32_t bw_limit, bool a
   xp->msg_len.smhdr.flags = (DDSRT_ENDIAN == DDSRT_LITTLE_ENDIAN ? SMFLAG_ENDIANNESS : 0);
   xp->msg_len.smhdr.octetsToNextHeader = 4;
 
-  xp->conn = conn;
   nn_xpack_reinit (xp);
 
-  if (xp->gv->thread_pool)
-    ddsi_sem_init (&xp->sem, 0);
-
-#ifdef DDSI_INCLUDE_BANDWIDTH_LIMITING
+#ifdef DDS_HAS_BANDWIDTH_LIMITING
   nn_bw_limit_init (&xp->limiter, bw_limit);
 #else
   (void) bw_limit;
@@ -1194,42 +1174,40 @@ void nn_xpack_free (struct nn_xpack *xp)
 {
   assert (xp->niov == 0);
   assert (xp->included_msgs.latest == NULL);
-  if (xp->gv->thread_pool)
-    ddsi_sem_destroy (&xp->sem);
   ddsrt_free (xp->iov);
   ddsrt_free (xp);
 }
 
-static ssize_t nn_xpack_send_rtps(struct nn_xpack * xp, const nn_locator_t *loc)
+static ssize_t nn_xpack_send_rtps(struct nn_xpack * xp, const ddsi_xlocator_t *loc)
 {
   ssize_t ret = -1;
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   /* Only encode when needed. */
   if (xp->sec_info.use_rtps_encoding)
   {
     ret = secure_conn_write(
                       xp->gv,
-                      xp->conn,
-                      loc,
+                      loc->conn,
+                      &loc->c,
                       xp->niov,
                       xp->iov,
                       xp->call_flags,
                       &(xp->msg_len),
-                      (xp->dstmode == NN_XMSG_DST_ONE),
+                      (xp->dstmode == NN_XMSG_DST_ONE || xp->dstmode == NN_XMSG_DST_ALL_UC),
                       &(xp->sec_info),
                       ddsi_conn_write);
   }
   else
-#endif /* DDSI_INCLUDE_SECURITY */
+#endif /* DDS_HAS_SECURITY */
   {
-    ret = ddsi_conn_write (xp->conn, loc, xp->niov, xp->iov, xp->call_flags);
+    ret = ddsi_conn_write (loc->conn, &loc->c, xp->niov, xp->iov, xp->call_flags);
   }
 
   return ret;
 }
 
-static ssize_t nn_xpack_send1 (const nn_locator_t *loc, void * varg)
+static ssize_t nn_xpack_send1 (const ddsi_xlocator_t *loc, void * varg)
 {
   struct nn_xpack *xp = varg;
   struct ddsi_domaingv const * const gv = xp->gv;
@@ -1238,7 +1216,7 @@ static ssize_t nn_xpack_send1 (const nn_locator_t *loc, void * varg)
   if (gv->logconfig.c.mask & DDS_LC_TRACE)
   {
     char buf[DDSI_LOCSTRLEN];
-    GVTRACE (" %s", ddsi_locator_to_string (buf, sizeof(buf), loc));
+    GVTRACE (" %s", ddsi_xlocator_to_string (buf, sizeof(buf), loc));
   }
 
   if (gv->config.xmit_lossiness > 0)
@@ -1252,8 +1230,13 @@ static ssize_t nn_xpack_send1 (const nn_locator_t *loc, void * varg)
       return 0;
     }
   }
-
+#ifdef DDS_HAS_SHM
+  // SHM_TODO: We avoid sending packet while data is SHMEM.
+  //           I'm not sure whether this is correct or not.
+  if (!gv->mute && loc->c.kind != NN_LOCATOR_KIND_SHEM)
+#else
   if (!gv->mute)
+#endif
   {
     nbytes = nn_xpack_send_rtps(xp, loc);
 
@@ -1279,7 +1262,7 @@ static ssize_t nn_xpack_send1 (const nn_locator_t *loc, void * varg)
 
   xp->call_flags = 0;
 
-#ifdef DDSI_INCLUDE_BANDWIDTH_LIMITING
+#ifdef DDS_HAS_BANDWIDTH_LIMITING
   if (nbytes > 0)
   {
     nn_bw_limit_sleep_if_needed (gv, &xp->limiter, nbytes);
@@ -1289,36 +1272,11 @@ static ssize_t nn_xpack_send1 (const nn_locator_t *loc, void * varg)
   return nbytes;
 }
 
-static void nn_xpack_send1v (const nn_locator_t *loc, void * varg)
+static void nn_xpack_send1v (const ddsi_xlocator_t *loc, void * varg)
 {
   (void) nn_xpack_send1 (loc, varg);
 }
 
-typedef struct nn_xpack_send1_thread_arg {
-  const nn_locator_t *loc;
-  struct nn_xpack *xp;
-} *nn_xpack_send1_thread_arg_t;
-
-static void nn_xpack_send1_thread (void * varg)
-{
-  nn_xpack_send1_thread_arg_t arg = varg;
-  (void) nn_xpack_send1 (arg->loc, arg->xp);
-  if (ddsrt_atomic_dec32_ov (&arg->xp->calls) == 1)
-  {
-    ddsi_sem_post (&arg->xp->sem);
-  }
-  ddsrt_free (varg);
-}
-
-static void nn_xpack_send1_threaded (const nn_locator_t *loc, void * varg)
-{
-  nn_xpack_send1_thread_arg_t arg = ddsrt_malloc (sizeof (*arg));
-  arg->xp = (struct nn_xpack *) varg;
-  arg->loc = loc;
-  ddsrt_atomic_inc32 (&arg->xp->calls);
-  ddsrt_thread_pool_submit (arg->xp->gv->thread_pool, nn_xpack_send1_thread, arg);
-}
-
 static void nn_xpack_send_real (struct nn_xpack *xp)
 {
   struct ddsi_domaingv const * const gv = xp->gv;
@@ -1357,21 +1315,7 @@ static void nn_xpack_send_real (struct nn_xpack *xp)
     calls = 0;
     if (xp->dstaddr.all.as)
     {
-      if (xp->gv->thread_pool == NULL)
-      {
-        calls = addrset_forall_count (xp->dstaddr.all.as, nn_xpack_send1v, xp);
-      }
-      else
-      {
-        ddsrt_atomic_st32 (&xp->calls, 1);
-        calls = addrset_forall_count (xp->dstaddr.all.as, nn_xpack_send1_threaded, xp);
-        /* Wait for the thread pool to complete the write; if we're the one
-           decrementing "calls" to 0, all of the work has been completed and
-           none of the threads will be posting; else some thread will be
-           posting it and we had better wait for it */
-        if (ddsrt_atomic_dec32_ov (&xp->calls) != 1)
-          ddsi_sem_wait (&xp->sem);
-      }
+      calls = addrset_forall_count (xp->dstaddr.all.as, nn_xpack_send1v, xp);
       unref_addrset (xp->dstaddr.all.as);
     }
 
@@ -1402,13 +1346,17 @@ static void nn_xpack_send_real (struct nn_xpack *xp)
 static uint32_t nn_xpack_sendq_thread (void *vgv)
 {
   struct ddsi_domaingv *gv = vgv;
+  struct thread_state1 * const ts1 = lookup_thread_state ();
+  thread_state_awake_fixed_domain (ts1);
   ddsrt_mutex_lock (&gv->sendq_lock);
   while (!(gv->sendq_stop && gv->sendq_head == NULL))
   {
     struct nn_xpack *xp;
     if ((xp = gv->sendq_head) == NULL)
     {
-      (void) ddsrt_cond_waitfor (&gv->sendq_cond, &gv->sendq_lock, 1000000);
+      thread_state_asleep (ts1);
+      (void) ddsrt_cond_wait (&gv->sendq_cond, &gv->sendq_lock);
+      thread_state_awake_fixed_domain (ts1);
     }
     else
     {
@@ -1422,6 +1370,7 @@ static uint32_t nn_xpack_sendq_thread (void *vgv)
     }
   }
   ddsrt_mutex_unlock (&gv->sendq_lock);
+  thread_state_asleep (ts1);
   return 0;
 }
 
@@ -1437,8 +1386,9 @@ void nn_xpack_sendq_init (struct ddsi_domaingv *gv)
 
 void nn_xpack_sendq_start (struct ddsi_domaingv *gv)
 {
-  if (create_thread (&gv->sendq_ts, gv, "sendq", nn_xpack_sendq_thread, NULL) != DDS_RETCODE_OK)
+  if (create_thread (&gv->sendq_ts, gv, "sendq", nn_xpack_sendq_thread, gv) != DDS_RETCODE_OK)
     GVERROR ("nn_xpack_sendq_start: can't create nn_xpack_sendq_thread\n");
+  gv->sendq_running = true;
 }
 
 void nn_xpack_sendq_stop (struct ddsi_domaingv *gv)
@@ -1451,8 +1401,8 @@ void nn_xpack_sendq_stop (struct ddsi_domaingv *gv)
 
 void nn_xpack_sendq_fini (struct ddsi_domaingv *gv)
 {
-  assert (gv->sendq_head == NULL);
   join_thread (gv->sendq_ts);
+  assert (gv->sendq_head == NULL);
   ddsrt_cond_destroy (&gv->sendq_cond);
   ddsrt_mutex_destroy (&gv->sendq_lock);
 }
@@ -1466,16 +1416,20 @@ void nn_xpack_send (struct nn_xpack *xp, bool immediately)
   else
   {
     struct ddsi_domaingv * const gv = xp->gv;
+    // copy xp
     struct nn_xpack *xp1 = ddsrt_malloc (sizeof (*xp));
-    memcpy (xp1, xp, sizeof (*xp1));
+    memcpy(xp1, xp, sizeof(*xp1));
+    if (xp->iov != NULL) {
+      xp1->iov = ddsrt_malloc(xp->niov * sizeof(*xp->iov));
+      memcpy(xp1->iov, xp->iov, (xp->niov * sizeof(*xp->iov)));
+    }
     nn_xpack_reinit (xp);
     xp1->sendq_next = NULL;
     ddsrt_mutex_lock (&gv->sendq_lock);
-    if (immediately || gv->sendq_length == SENDQ_HW)
+    if (immediately || gv->sendq_length > SENDQ_LW)
       ddsrt_cond_broadcast (&gv->sendq_cond);
     if (gv->sendq_length >= SENDQ_MAX)
     {
-      while (gv->sendq_length > SENDQ_LW)
         ddsrt_cond_wait (&gv->sendq_cond, &gv->sendq_lock);
     }
     if (gv->sendq_head)
@@ -1505,6 +1459,9 @@ static void copy_addressing_info (struct nn_xpack *xp, const struct nn_xmsg *m)
       xp->dstaddr.all.as = ref_addrset (m->dstaddr.all.as);
       xp->dstaddr.all.as_group = ref_addrset (m->dstaddr.all.as_group);
       break;
+    case NN_XMSG_DST_ALL_UC:
+      xp->dstaddr.all_uc.as = ref_addrset (m->dstaddr.all_uc.as);
+      break;
   }
 }
 
@@ -1521,6 +1478,8 @@ static int addressing_info_eq_onesidederr (const struct nn_xpack *xp, const stru
     case NN_XMSG_DST_ALL:
       return (addrset_eq_onesidederr (xp->dstaddr.all.as, m->dstaddr.all.as) &&
               addrset_eq_onesidederr (xp->dstaddr.all.as_group, m->dstaddr.all.as_group));
+    case NN_XMSG_DST_ALL_UC:
+      return addrset_eq_onesidederr (xp->dstaddr.all_uc.as, m->dstaddr.all_uc.as);
   }
   assert (0);
   return 0;
@@ -1568,13 +1527,7 @@ static int nn_xpack_mayaddmsg (const struct nn_xpack *xp, const struct nn_xmsg *
     return 0;
   }
 
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-  /* Don't mix up xmsg for different encoders */
-  if (xp->encoderId != m->encoderid)
-    return 0;
-#endif
-
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
   /* Don't mix up encoded and plain rtps messages */
   if (xp->sec_info.use_rtps_encoding != m->sec_info.use_rtps_encoding)
     return 0;
@@ -1661,7 +1614,7 @@ int nn_xpack_addmsg (struct nn_xpack *xp, struct nn_xmsg *m, const uint32_t flag
 
     /* Add MSG_LEN sub message for stream based transports */
 
-    if (xp->conn->m_stream)
+    if (!gv->m_factory->m_connless)
     {
       xp->iov[niov].iov_base = (void*) &xp->msg_len;
       xp->iov[niov].iov_len = sizeof (xp->msg_len);
@@ -1669,11 +1622,8 @@ int nn_xpack_addmsg (struct nn_xpack *xp, struct nn_xmsg *m, const uint32_t flag
       niov++;
     }
 
-#ifdef DDSI_INCLUDE_SECURITY
+#ifdef DDS_HAS_SECURITY
     xp->sec_info = m->sec_info;
-#endif
-#ifdef DDSI_INCLUDE_NETWORK_PARTITIONS
-    xp->encoderId = m->encoderid;
 #endif
     xp->last_src = &xp->hdr.guid_prefix;
     xp->last_dst = NULL;
@@ -1708,8 +1658,8 @@ int nn_xpack_addmsg (struct nn_xpack *xp, struct nn_xmsg *m, const uint32_t flag
      packed message, add an InfoDST note. Note that neither has to
      have a dst set. */
   if (xp->last_dst == NULL)
-    dst = (m->dstmode == NN_XMSG_DST_ONE) ? &m->data->dst : NULL;
-  else if (m->dstmode != NN_XMSG_DST_ONE)
+    dst = (m->dstmode == NN_XMSG_DST_ONE || m->dstmode == NN_XMSG_DST_ALL_UC) ? &m->data->dst : NULL;
+  else if (m->dstmode != NN_XMSG_DST_ONE && m->dstmode != NN_XMSG_DST_ALL_UC)
     dst = &static_zero_dst;
   else
     dst = guid_prefix_eq (&xp->last_dst->guid_prefix, &m->data->dst.guid_prefix) ? NULL : &m->data->dst;
@@ -1720,7 +1670,7 @@ int nn_xpack_addmsg (struct nn_xpack *xp, struct nn_xmsg *m, const uint32_t flag
        than many small ones */
     if ((char *) xp->iov[niov-1].iov_base + xp->iov[niov-1].iov_len == (char *) dst)
     {
-      xp->iov[niov-1].iov_len += sizeof (*dst);
+      xp->iov[niov-1].iov_len += (ddsrt_iov_len_t)sizeof (*dst);
     }
     else
     {
@@ -1765,7 +1715,7 @@ int nn_xpack_addmsg (struct nn_xpack *xp, struct nn_xmsg *m, const uint32_t flag
   xp->niov = niov;
 
   const bool rexmit = xp->includes_rexmit || nn_xmsg_is_rexmit (m);
-  const unsigned max_msg_size = rexmit ? xp->gv->config.max_rexmit_msg_size : xp->gv->config.max_msg_size;
+  const uint32_t max_msg_size = rexmit ? xp->gv->config.max_rexmit_msg_size : xp->gv->config.max_msg_size;
   if (xpo_niov > 0 && sz > max_msg_size)
   {
     GVTRACE (" => now niov %d sz %"PRIuSIZE" > max_msg_size %"PRIu32", nn_xpack_send niov %d sz %"PRIu32" now\n",
@@ -1778,15 +1728,8 @@ int nn_xpack_addmsg (struct nn_xpack *xp, struct nn_xmsg *m, const uint32_t flag
   else
   {
     xp->call_flags = flags;
-    switch (m->kind)
-    {
-      case NN_XMSG_KIND_DATA:
-      case NN_XMSG_KIND_CONTROL:
-        break;
-      case NN_XMSG_KIND_DATA_REXMIT:
-      case NN_XMSG_KIND_DATA_REXMIT_NOMERGE:
-        xp->includes_rexmit = true;
-    }
+    if (nn_xmsg_is_rexmit (m))
+      xp->includes_rexmit = true;
     nn_xmsg_chain_add (&xp->included_msgs, m);
     GVTRACE (" => now niov %d sz %"PRIuSIZE"\n", (int) niov, sz);
   }
