@@ -9,11 +9,12 @@
  *
  * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
  */
+#include "dds/export.h"
 #include "dds/ddsrt/bswap.h"
 
-extern inline uint16_t ddsrt_bswap2u (uint16_t x);
-extern inline uint32_t ddsrt_bswap4u (uint32_t x);
-extern inline uint64_t ddsrt_bswap8u (uint64_t x);
-extern inline int16_t ddsrt_bswap2 (int16_t x);
-extern inline int32_t ddsrt_bswap4 (int32_t x);
-extern inline int64_t ddsrt_bswap8 (int64_t x);
+DDS_EXPORT extern inline uint16_t ddsrt_bswap2u (uint16_t x);
+DDS_EXPORT extern inline uint32_t ddsrt_bswap4u (uint32_t x);
+DDS_EXPORT extern inline uint64_t ddsrt_bswap8u (uint64_t x);
+DDS_EXPORT extern inline int16_t ddsrt_bswap2 (int16_t x);
+DDS_EXPORT extern inline int32_t ddsrt_bswap4 (int32_t x);
+DDS_EXPORT extern inline int64_t ddsrt_bswap8 (int64_t x);
