@@ -17,28 +17,35 @@
 #include "dds/ddsrt/io.h"
 
 int
-ddsrt_asprintf(
+ddsrt_vasprintf(
   char **strp,
   const char *fmt,
-  ...)
+  va_list ap)
 {
   int ret;
   unsigned int len;
+#if !defined(_WIN32)
   char buf[1] = { '\0' };
+#endif
   char *str = NULL;
-  va_list args1, args2;
+  va_list ap2;
 
   assert(strp != NULL);
   assert(fmt != NULL);
 
-  va_start(args1, fmt);
-  va_copy(args2, args1); /* va_list cannot be reused */
+  va_copy(ap2, ap); /* va_list cannot be reused */
 
-  if ((ret = vsnprintf(buf, sizeof(buf), fmt, args1)) >= 0) {
-    len = (unsigned int)ret; /* +1 for null byte */
+#if defined(_WIN32)
+  /* mingw-w64 maps vsnprintf to _vsnprint which returns -1 if the buffer is
+     not sufficiently large enough */
+  if ((ret = _vscprintf(fmt, ap)) >= 0) {
+#else
+  if ((ret = vsnprintf(buf, sizeof(buf), fmt, ap)) >= 0) {
+#endif
+    len = (unsigned int)ret;
     if ((str = ddsrt_malloc(len + 1)) == NULL) {
       ret = -1;
-    } else if ((ret = vsnprintf(str, len + 1, fmt, args2)) >= 0) {
+    } else if ((ret = vsnprintf(str, len + 1, fmt, ap2)) >= 0) {
       assert(((unsigned int)ret) == len);
       *strp = str;
     } else {
@@ -46,8 +53,26 @@ ddsrt_asprintf(
     }
   }
 
-  va_end(args1);
-  va_end(args2);
+  va_end(ap2);
+
+  return ret;
+}
+
+int
+ddsrt_asprintf(
+  char **strp,
+  const char *fmt,
+  ...)
+{
+  int ret;
+  va_list args;
+
+  assert(strp != NULL);
+  assert(fmt != NULL);
+
+  va_start(args, fmt);
+  ret = ddsrt_vasprintf(strp, fmt, args);
+  va_end(args);
 
   return ret;
 }
