@@ -21,6 +21,7 @@
 #include "dds/ddsrt/atomics.h"
 #include "dds/ddsrt/string.h"
 #include "dds/ddsrt/io.h"
+#include "dds/ddsrt/static_assert.h"
 #include "dds/security/dds_security_api_defs.h"
 #include "dds/security/core/dds_security_utils.h"
 #include "dds/security/openssl_support.h"
@@ -28,7 +29,19 @@
 
 #define MAX_TRUSTED_CA 100
 
-char *get_openssl_error_message(void)
+typedef enum {
+    AUTH_CONF_ITEM_PREFIX_UNKNOWN,
+    AUTH_CONF_ITEM_PREFIX_FILE,
+    AUTH_CONF_ITEM_PREFIX_DATA,
+    AUTH_CONF_ITEM_PREFIX_PKCS11
+} AuthConfItemPrefix_t;
+
+/* Return a string that contains an openssl error description
+ * When a openssl function returns an error this function can be
+ * used to retrieve a descriptive error string.
+ * Note that the returned string should be freed.
+ */
+static char *get_openssl_error_message(void)
 {
   char *msg, *buf = NULL;
   size_t len;
@@ -242,7 +255,8 @@ static BIO *load_file_into_BIO (const char *filename, DDS_Security_SecurityExcep
     goto err_get_length;
   }
   long max = ftell (fp);
-  if (max < 0 || (unsigned long) max > SIZE_MAX)
+  DDSRT_STATIC_ASSERT(ULONG_MAX <= SIZE_MAX);
+  if (max < 0)
   {
     DDS_Security_Exception_set (ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, DDS_SECURITY_VALIDATION_FAILED, "load_file_into_BIO: ftell failed");
     goto err_get_length;
@@ -366,14 +380,17 @@ static DDS_Security_ValidationResult_t load_private_key_from_file(const char *fi
  * Gets the URI string (as referred in DDS Security spec) and returns the URI type
  * data: data part of the URI. Typically It contains different format according to URI type.
  */
-AuthConfItemPrefix_t get_conf_item_type(const char *str, char **data)
+static AuthConfItemPrefix_t get_conf_item_type(const char *str, char **data)
 {
   const char *f = "file:", *d = "data:,", *p = "pkcs11:";
   size_t sf = strlen(f), sd = strlen(d), sp = strlen(p);
+  const char *ptr;
   assert(str);
   assert(data);
 
-  char *ptr = ddssec_strchrs(str, " \t", false);
+  for (ptr = str; *ptr == ' ' || *ptr == '\t'; ptr++)
+    /* ignore leading whitespace */;
+
   if (strncmp(ptr, f, sf) == 0)
   {
     size_t e = strncmp(ptr + sf, "//", 2) == 0 ? 2 : 0;
@@ -470,15 +487,85 @@ DDS_Security_ValidationResult_t load_X509_private_key(const char *data, const ch
   return result;
 }
 
-DDS_Security_ValidationResult_t verify_certificate(X509 *identityCert, X509 *identityCa, DDS_Security_SecurityException *ex)
+static DDS_Security_ValidationResult_t load_CRL_from_file(const char *filepath, X509_CRL **crl, DDS_Security_SecurityException *ex)
+{
+  BIO *bio;
+  assert(filepath);
+  assert(crl);
+
+  if ((bio = load_file_into_BIO(filepath, ex)) == NULL)
+  {
+    return DDS_SECURITY_VALIDATION_FAILED;
+  }
+
+  *crl = PEM_read_bio_X509_CRL(bio, NULL, NULL, NULL);
+  BIO_free(bio);
+  if (*crl == NULL)
+  {
+    DDS_Security_Exception_set_with_openssl_error(ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, DDS_SECURITY_VALIDATION_FAILED, "Failed to read CRL: ");
+    return DDS_SECURITY_VALIDATION_FAILED;
+  }
+
+  return DDS_SECURITY_VALIDATION_OK;
+}
+
+static DDS_Security_ValidationResult_t load_CRL_from_data(const char *data, X509_CRL **crl, DDS_Security_SecurityException *ex)
+{
+  BIO *bio;
+  assert(data);
+  assert(crl);
+
+  if (!(bio = BIO_new_mem_buf((void *)data, -1)))
+  {
+    DDS_Security_Exception_set(ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, DDS_SECURITY_VALIDATION_FAILED, "BIO_new_mem_buf failed");
+    return DDS_SECURITY_VALIDATION_FAILED;
+  }
+
+  *crl = PEM_read_bio_X509_CRL(bio, NULL, NULL, NULL);
+  BIO_free(bio);
+  if (*crl == NULL)
+  {
+    DDS_Security_Exception_set_with_openssl_error(ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, DDS_SECURITY_VALIDATION_FAILED, "Failed to read CRL: ");
+    return DDS_SECURITY_VALIDATION_FAILED;
+  }
+
+  return DDS_SECURITY_VALIDATION_OK;
+}
+
+DDS_Security_ValidationResult_t load_X509_CRL(const char *data, X509_CRL **crl, DDS_Security_SecurityException *ex)
+{
+  DDS_Security_ValidationResult_t result;
+  char *contents = NULL;
+  assert(data);
+  assert(crl);
+
+  switch (get_conf_item_type(data, &contents))
+  {
+  case AUTH_CONF_ITEM_PREFIX_FILE:
+    result = load_CRL_from_file(contents, crl, ex);
+    break;
+  case AUTH_CONF_ITEM_PREFIX_DATA:
+    result = load_CRL_from_data(contents, crl, ex);
+    break;
+  default:
+    result = DDS_SECURITY_VALIDATION_FAILED;
+    DDS_Security_Exception_set(ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, (int)result, "Specified CRL has wrong format:\n%s", data);
+    break;
+  }
+  ddsrt_free(contents);
+
+  return result;
+}
+
+DDS_Security_ValidationResult_t verify_certificate(X509 *identityCert, X509 *identityCa, X509_CRL *crl, DDS_Security_SecurityException *ex)
 {
   X509_STORE_CTX *ctx;
   X509_STORE *store;
+  unsigned long verify_flags = 0;
   assert(identityCert);
   assert(identityCa);
 
-  /* Currently only a self signed indentiyCa is supported. Verification of against a certificate chain is not yet supported */
-  /* Verification of the certificate expiry using a CRL is not yet supported */
+  /* Currently only a self signed identityCa is supported. Verification against a certificate chain is not yet supported */
 
   if (!(store = X509_STORE_new()))
   {
@@ -490,6 +577,16 @@ DDS_Security_ValidationResult_t verify_certificate(X509 *identityCert, X509 *ide
     DDS_Security_Exception_set_with_openssl_error(ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, DDS_SECURITY_VALIDATION_FAILED, "X509_STORE_add_cert failed : ");
     goto err_add_cert;
   }
+
+  if (crl != NULL)
+  {
+    if (X509_STORE_add_crl(store, crl) == 0)
+    {
+      goto err_add_cert;
+    }
+    verify_flags = X509_V_FLAG_CRL_CHECK;
+  }
+
   if (!(ctx = X509_STORE_CTX_new()))
   {
     DDS_Security_Exception_set_with_openssl_error(ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, DDS_SECURITY_VALIDATION_FAILED, "X509_STORE_CTX_new failed : ");
@@ -500,10 +597,12 @@ DDS_Security_ValidationResult_t verify_certificate(X509 *identityCert, X509 *ide
     DDS_Security_Exception_set_with_openssl_error(ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, DDS_SECURITY_VALIDATION_FAILED, "X509_STORE_CTX_init failed : ");
     goto err_ctx_init;
   }
+  /* X509_STORE_CTX_set_flags is mis-named; it doesn't actually set flags, it ORs them in.  So it is always safe to do this. */
+  X509_STORE_CTX_set_flags(ctx, verify_flags);
   if (X509_verify_cert(ctx) != 1)
   {
     const char *msg = X509_verify_cert_error_string(X509_STORE_CTX_get_error(ctx));
-    char *subject = get_certificate_subject_name(identityCert, NULL);
+    char *subject = get_certificate_subject_name(identityCert, ex);
     DDS_Security_Exception_set(ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, DDS_SECURITY_VALIDATION_FAILED, "Certificate not valid: error: %s; subject: %s", msg, subject ? subject : "[not found]");
     ddsrt_free(subject);
     goto err_ctx_init;
@@ -821,7 +920,7 @@ static DDS_Security_ValidationResult_t dh_oct_to_public_key_modp(EVP_PKEY **pkey
     DDS_Security_Exception_set_with_openssl_error(ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, DDS_SECURITY_VALIDATION_FAILED, "Failed to convert octet sequence to ASN1 integer: ");
     goto fail_alloc_pkey;
   }
-  if (!(asn1int = d2i_ASN1_INTEGER(NULL, (const unsigned char **)&keystr, size)))
+  if (!(asn1int = d2i_ASN1_INTEGER(NULL, (const unsigned char **)&keystr, (long)size)))
   {
     DDS_Security_Exception_set_with_openssl_error(ex, DDS_AUTH_PLUGIN_CONTEXT, DDS_SECURITY_ERR_UNDEFINED_CODE, DDS_SECURITY_VALIDATION_FAILED, "Failed to convert octet sequence to ASN1 integer: ");
     goto fail_get_asn1int;
