@@ -221,7 +221,7 @@ static void prepare_endpoint_security_attributes_and_properties(DDS_Security_End
     properties->_maximum = properties->_length = 1;
     properties->_buffer = ddsrt_malloc(sizeof(DDS_Security_Property_t));
 
-    properties->_buffer[0].name = ddsrt_strdup("dds.sec.crypto.keysize");
+    properties->_buffer[0].name = ddsrt_strdup(DDS_SEC_PROP_CRYPTO_KEYSIZE);
 
     if (transformation_kind == CRYPTO_TRANSFORMATION_KIND_AES128_GCM || transformation_kind == CRYPTO_TRANSFORMATION_KIND_AES128_GMAC)
     {
@@ -433,7 +433,8 @@ static void suite_decode_datareader_submessage_init(void)
   CU_ASSERT_FATAL ((plugins = load_plugins(
                       NULL    /* Access Control */,
                       NULL    /* Authentication */,
-                      &crypto /* Cryptograpy    */)) != NULL);
+                      &crypto /* Cryptograpy    */,
+                      NULL)) != NULL);
 
   CU_ASSERT_EQUAL_FATAL (register_local_participant(), 0);
   CU_ASSERT_EQUAL_FATAL (register_remote_participant(), 0);
@@ -864,66 +865,6 @@ CU_Test(ddssec_builtin_decode_datareader_submessage, invalid_args, .init = suite
 
   reset_exception(&exception);
 
-  /* decoded buffer NULL */
-  result = crypto->crypto_transform->decode_datareader_submessage(
-      crypto->crypto_transform,
-      NULL,
-      &encoded_buffer,
-      local_writer_crypto,
-      remote_reader_crypto,
-      &exception);
-
-  if (!result)
-  {
-    printf("decode_datareader_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
-  /* encoded buffer NULL */
-  result = crypto->crypto_transform->decode_datareader_submessage(
-      crypto->crypto_transform,
-      &decoded_buffer,
-      NULL,
-      local_writer_crypto,
-      remote_reader_crypto,
-      &exception);
-
-  if (!result)
-  {
-    printf("decode_datareader_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
-  /* empty encoded buffer */
-  result = crypto->crypto_transform->decode_datareader_submessage(
-      crypto->crypto_transform,
-      &decoded_buffer,
-      &empty_buffer,
-      local_writer_crypto,
-      remote_reader_crypto,
-      &exception);
-
-  if (!result)
-  {
-    printf("decode_datareader_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
   /* local writer crypto 0 */
   result = crypto->crypto_transform->decode_datareader_submessage(
       crypto->crypto_transform,
@@ -1648,13 +1589,13 @@ CU_Test(ddssec_builtin_decode_datareader_submessage, volatile_sec, .init = suite
 
   datareader_properties._length = datareader_properties._maximum = 1;
   datareader_properties._buffer = DDS_Security_PropertySeq_allocbuf(1);
-  datareader_properties._buffer[0].name = ddsrt_strdup("dds.sec.builtin_endpoint_name");
+  datareader_properties._buffer[0].name = ddsrt_strdup(DDS_SEC_PROP_BUILTIN_ENDPOINT_NAME);
   datareader_properties._buffer[0].value = ddsrt_strdup("BuiltinParticipantVolatileMessageSecureReader");
   datareader_properties._buffer[0].propagate = false;
 
   datawriter_properties._length = datawriter_properties._maximum = 1;
   datawriter_properties._buffer = DDS_Security_PropertySeq_allocbuf(1);
-  datawriter_properties._buffer[0].name = ddsrt_strdup("dds.sec.builtin_endpoint_name");
+  datawriter_properties._buffer[0].name = ddsrt_strdup(DDS_SEC_PROP_BUILTIN_ENDPOINT_NAME);
   datawriter_properties._buffer[0].value = ddsrt_strdup("BuiltinParticipantVolatileMessageSecureWriter");
   datawriter_properties._buffer[0].propagate = false;
 
