@@ -17,6 +17,7 @@
 #include "dds/ddsrt/endian.h"
 #include "dds/ddsrt/types.h"
 #include "dds/ddsrt/environ.h"
+#include "dds/ddsrt/static_assert.h"
 #include "dds/security/dds_security_api.h"
 #include "dds/security/core/dds_security_serialize.h"
 #include "dds/security/core/dds_security_utils.h"
@@ -154,7 +155,7 @@ static void prepare_endpoint_security_attributes_and_properties(DDS_Security_End
     properties->_maximum = properties->_length = 1;
     properties->_buffer = ddsrt_malloc(sizeof(DDS_Security_Property_t));
 
-    properties->_buffer[0].name = ddsrt_strdup("dds.sec.crypto.keysize");
+    properties->_buffer[0].name = ddsrt_strdup(DDS_SEC_PROP_CRYPTO_KEYSIZE);
 
     if (transformation_kind == CRYPTO_TRANSFORMATION_KIND_AES128_GCM || transformation_kind == CRYPTO_TRANSFORMATION_KIND_AES128_GMAC)
     {
@@ -407,7 +408,8 @@ static void suite_decode_datawriter_submessage_init(void)
   CU_ASSERT_FATAL ((plugins = load_plugins(
                       NULL    /* Access Control */,
                       NULL    /* Authentication */,
-                      &crypto /* Cryptograpy    */)) != NULL);
+                      &crypto /* Cryptograpy    */,
+                      NULL)) != NULL);
 
   CU_ASSERT_EQUAL_FATAL (register_local_participant(), 0);
   CU_ASSERT_EQUAL_FATAL (register_remote_participant(), 0);
@@ -421,23 +423,26 @@ static void suite_decode_datawriter_submessage_fini(void)
   unload_plugins(plugins);
 }
 
-static void initialize_data_submessage(DDS_Security_OctetSeq *submsg, bool be)
+static unsigned char submsg_header_endianness_flag (enum ddsrt_byte_order_selector bo)
+{
+#if DDSRT_ENDIAN == DDSRT_LITTLE_ENDIAN
+  return (unsigned char) ((bo == DDSRT_BOSEL_BE) ? 0 : SMFLAG_ENDIANNESS);
+#else
+  return (unsigned char) ((bo == DDSRT_BO_LE) ? SMFLAG_ENDIANNESS : 0);
+#endif
+}
+
+static void initialize_data_submessage(DDS_Security_OctetSeq *submsg, enum ddsrt_byte_order_selector bo)
 {
   size_t length = strlen(sample_test_data) + 1;
   struct submsg_header *header;
-  int swap;
   unsigned char *buffer, *ptr;
 
-  if (be)
-    swap = (DDSRT_ENDIAN == DDSRT_LITTLE_ENDIAN);
-  else
-    swap = (DDSRT_ENDIAN != DDSRT_LITTLE_ENDIAN);
-
   buffer = ddsrt_malloc(length + sizeof(struct submsg_header));
   header = (struct submsg_header *)buffer;
   header->id = 0x15;
-  header->flags = be ? 0x00 : 0x01;
-  header->length = swap ? ddsrt_bswap2u((uint16_t)length) : (uint16_t)length;
+  header->flags = submsg_header_endianness_flag(bo);
+  header->length = ddsrt_toBO2u(bo, (uint16_t)length);
   ptr = (unsigned char *)(header + 1);
 
   memcpy((char *)ptr, sample_test_data, length);
@@ -511,7 +516,7 @@ static void decode_datawriter_submessage_not_signed(DDS_Security_CryptoTransform
   prepare_endpoint_security_attributes_and_properties(&datareader_security_attributes, &datareader_properties, transformation_kind, false);
   prepare_endpoint_security_attributes_and_properties(&datawriter_security_attributes, &datawriter_properties, transformation_kind, false);
 
-  initialize_data_submessage(&plain_buffer, false);
+  initialize_data_submessage(&plain_buffer, DDSRT_BOSEL_NATIVE);
 
   local_writer_crypto = register_local_datawriter(&datawriter_security_attributes, &datawriter_properties);
   CU_ASSERT_FATAL(local_writer_crypto != 0);
@@ -648,7 +653,7 @@ static void decode_datawriter_submessage_signed(DDS_Security_CryptoTransformKind
   prepare_endpoint_security_attributes_and_properties(&datareader_security_attributes, &datareader_properties, transformation_kind, true);
   prepare_endpoint_security_attributes_and_properties(&datawriter_security_attributes, &datawriter_properties, transformation_kind, true);
 
-  initialize_data_submessage(&plain_buffer, false);
+  initialize_data_submessage(&plain_buffer, DDSRT_BOSEL_NATIVE);
 
   local_writer_crypto = register_local_datawriter(&datawriter_security_attributes, &datawriter_properties);
   CU_ASSERT_FATAL(local_writer_crypto != 0);
@@ -817,7 +822,7 @@ CU_Test(ddssec_builtin_decode_datawriter_submessage, invalid_args, .init = suite
   prepare_endpoint_security_attributes_and_properties(&datareader_security_attributes, &datareader_properties, CRYPTO_TRANSFORMATION_KIND_AES256_GCM, true);
   prepare_endpoint_security_attributes_and_properties(&datawriter_security_attributes, &datawriter_properties, CRYPTO_TRANSFORMATION_KIND_AES256_GCM, true);
 
-  initialize_data_submessage(&plain_buffer, false);
+  initialize_data_submessage(&plain_buffer, DDSRT_BOSEL_NATIVE);
 
   memset(&empty_buffer, 0, sizeof(empty_buffer));
 
@@ -862,87 +867,6 @@ CU_Test(ddssec_builtin_decode_datawriter_submessage, invalid_args, .init = suite
 
   reset_exception(&exception);
 
-#if 0
-    /*  */
-    result = crypto->crypto_transform->decode_datawriter_submessage(
-            crypto->crypto_transform,
-            &decoded_buffer,
-            &encoded_buffer,
-            local_reader_crypto,
-            remote_writer_crypto,
-            &exception);
-
-    if (!result) {
-        printf("decode_datawriter_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-    }
-
-    CU_ASSERT(!result);
-    CU_ASSERT(exception.code != 0);
-    CU_ASSERT(exception.message != NULL);
-
-    reset_exception(&exception);
-#endif
-
-  /* decoded buffer NULL */
-  result = crypto->crypto_transform->decode_datawriter_submessage(
-      crypto->crypto_transform,
-      NULL,
-      &encoded_buffer,
-      local_reader_crypto,
-      remote_writer_crypto,
-      &exception);
-
-  if (!result)
-  {
-    printf("decode_datawriter_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
-  /* encoded buffer NULL */
-  result = crypto->crypto_transform->decode_datawriter_submessage(
-      crypto->crypto_transform,
-      &decoded_buffer,
-      NULL,
-      local_reader_crypto,
-      remote_writer_crypto,
-      &exception);
-
-  if (!result)
-  {
-    printf("decode_datawriter_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
-  /* empty encoded buffer */
-  result = crypto->crypto_transform->decode_datawriter_submessage(
-      crypto->crypto_transform,
-      &decoded_buffer,
-      &empty_buffer,
-      local_reader_crypto,
-      remote_writer_crypto,
-      &exception);
-
-  if (!result)
-  {
-    printf("decode_datawriter_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
   /* local reader crypto 0 */
   result = crypto->crypto_transform->decode_datawriter_submessage(
       crypto->crypto_transform,
@@ -1071,7 +995,7 @@ CU_Test(ddssec_builtin_decode_datawriter_submessage, invalid_data, .init = suite
   prepare_endpoint_security_attributes_and_properties(&datareader_security_attributes, &datareader_properties, CRYPTO_TRANSFORMATION_KIND_AES256_GCM, true);
   prepare_endpoint_security_attributes_and_properties(&datawriter_security_attributes, &datawriter_properties, CRYPTO_TRANSFORMATION_KIND_AES256_GCM, true);
 
-  initialize_data_submessage(&plain_buffer, false);
+  initialize_data_submessage(&plain_buffer, DDSRT_BOSEL_NATIVE);
 
   local_writer_crypto = register_local_datawriter(&datawriter_security_attributes, &datawriter_properties);
   CU_ASSERT_FATAL(local_writer_crypto != 0);
@@ -1667,17 +1591,17 @@ CU_Test(ddssec_builtin_decode_datawriter_submessage, volatile_sec, .init = suite
   prepare_endpoint_security_attributes_and_properties(&datareader_security_attributes, NULL, CRYPTO_TRANSFORMATION_KIND_AES256_GCM, false);
   prepare_endpoint_security_attributes_and_properties(&datawriter_security_attributes, NULL, CRYPTO_TRANSFORMATION_KIND_AES256_GCM, false);
 
-  initialize_data_submessage(&plain_buffer, false);
+  initialize_data_submessage(&plain_buffer, DDSRT_BOSEL_NATIVE);
 
   datareader_properties._length = datareader_properties._maximum = 1;
   datareader_properties._buffer = DDS_Security_PropertySeq_allocbuf(1);
-  datareader_properties._buffer[0].name = ddsrt_strdup("dds.sec.builtin_endpoint_name");
+  datareader_properties._buffer[0].name = ddsrt_strdup(DDS_SEC_PROP_BUILTIN_ENDPOINT_NAME);
   datareader_properties._buffer[0].value = ddsrt_strdup("BuiltinParticipantVolatileMessageSecureReader");
   datareader_properties._buffer[0].propagate = false;
 
   datawriter_properties._length = datawriter_properties._maximum = 1;
   datawriter_properties._buffer = DDS_Security_PropertySeq_allocbuf(1);
-  datawriter_properties._buffer[0].name = ddsrt_strdup("dds.sec.builtin_endpoint_name");
+  datawriter_properties._buffer[0].name = ddsrt_strdup(DDS_SEC_PROP_BUILTIN_ENDPOINT_NAME);
   datawriter_properties._buffer[0].value = ddsrt_strdup("BuiltinParticipantVolatileMessageSecureWriter");
   datawriter_properties._buffer[0].propagate = false;
 
