@@ -25,6 +25,7 @@
 #include "CUnit/Test.h"
 #include "common/src/loader.h"
 #include "crypto_objects.h"
+#include "crypto_utils.h"
 
 #define TEST_SHARED_SECRET_SIZE 32
 
@@ -392,7 +393,8 @@ static void suite_decode_serialized_payload_init(void)
   CU_ASSERT_FATAL ((plugins = load_plugins(
                       NULL    /* Access Control */,
                       NULL    /* Authentication */,
-                      &crypto /* Cryptograpy    */)) != NULL);
+                      &crypto /* Cryptograpy    */,
+                      NULL)) != NULL);
   CU_ASSERT_EQUAL_FATAL (register_local_participant(), 0);
   CU_ASSERT_EQUAL_FATAL (register_remote_participant(), 0);
 }
@@ -623,69 +625,6 @@ CU_Test(ddssec_builtin_decode_serialized_payload, invalid_args, .init = suite_de
 
   reset_exception(&exception);
 
-  /* no decoded data buffer specified */
-  result = crypto->crypto_transform->decode_serialized_payload(
-      crypto->crypto_transform,
-      NULL,
-      &encoded_buffer,
-      &extra_inline_qos,
-      local_reader_crypto,
-      remote_writer_crypto,
-      &exception);
-
-  if (!result)
-  {
-    printf("decode_serialized_payload: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
-  /* no encrypted data buffer specified */
-  result = crypto->crypto_transform->decode_serialized_payload(
-      crypto->crypto_transform,
-      &decoded_buffer,
-      NULL,
-      &extra_inline_qos,
-      local_reader_crypto,
-      remote_writer_crypto,
-      &exception);
-
-  if (!result)
-  {
-    printf("decode_serialized_payload: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
-  /* empty encrypted data buffer specified */
-  result = crypto->crypto_transform->decode_serialized_payload(
-      crypto->crypto_transform,
-      &decoded_buffer,
-      &empty_buffer,
-      &extra_inline_qos,
-      local_reader_crypto,
-      remote_writer_crypto,
-      &exception);
-
-  if (!result)
-  {
-    printf("decode_serialized_payload: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
   /* unknown local reader crypto handle specified */
   result = crypto->crypto_transform->decode_serialized_payload(
       crypto->crypto_transform,
@@ -885,7 +824,7 @@ CU_Test(ddssec_builtin_decode_serialized_payload, invalid_data, .init = suite_de
   /* use incorrect transformation key id */
   {
     unsigned char key[4];
-    uint32_t val = (uint32_t) rand();
+    uint32_t val = crypto_get_random_uint32();
 
     memcpy(key, header->transform_identifier.transformation_key_id, 4);
     memcpy(header->transform_identifier.transformation_key_id, &val, 4);
@@ -916,7 +855,7 @@ CU_Test(ddssec_builtin_decode_serialized_payload, invalid_data, .init = suite_de
   /* use incorrect session id*/
   {
     unsigned char sid[4];
-    uint32_t val = (uint32_t) rand();
+    uint32_t val = crypto_get_random_uint32();
 
     memcpy(sid, header->session_id, 4);
     memcpy(header->session_id, &val, 4);
@@ -953,8 +892,8 @@ CU_Test(ddssec_builtin_decode_serialized_payload, invalid_data, .init = suite_de
       uint32_t l;
     } val;
 
-    val.h = (uint32_t) rand();
-    val.l = (uint32_t) rand();
+    val.h = crypto_get_random_uint32();
+    val.l = crypto_get_random_uint32();
 
     memcpy(iv, header->init_vector_suffix, 8);
     memcpy(header->init_vector_suffix, &val, 8);
