@@ -291,16 +291,18 @@ static bool read_prefix(unsigned char **ptr, uint32_t *remain)
 
   prefix = (struct submsg_header *)(*ptr);
 
-  if (prefix->id != SMID_SEC_PREFIX_KIND)
+  if (prefix->id != SMID_SEC_PREFIX)
   {
     printf("check_encoded_data: prefix incorrect smid 0x%x02\n", prefix->id);
     return false;
   }
 
+  DDSRT_WARNING_MSVC_OFF(6326)
   if (prefix->flags & 0x01)
     swap = (DDSRT_ENDIAN != DDSRT_LITTLE_ENDIAN);
   else
     swap = (DDSRT_ENDIAN == DDSRT_LITTLE_ENDIAN);
+  DDSRT_WARNING_MSVC_ON(6326)
 
   hlen = swap ? ddsrt_bswap2u(prefix->length) : prefix->length;
 
@@ -326,7 +328,6 @@ static bool read_header(struct crypto_header **header, unsigned char **ptr, uint
 
   *header = ddsrt_malloc(sizeof(struct crypto_header));
   memcpy(*header, *ptr, sizeof(struct crypto_header));
-
   *ptr += sizeof(struct crypto_header);
   *remain -= (uint32_t)sizeof(struct crypto_header);
 
@@ -346,10 +347,12 @@ static bool read_body(DDS_Security_OctetSeq *contents, bool encrypted, unsigned
 
   body = (struct submsg_header *)(*ptr);
 
+  DDSRT_WARNING_MSVC_OFF(6326)
   if (body->flags & 0x01)
     swap = (DDSRT_ENDIAN != DDSRT_LITTLE_ENDIAN);
   else
     swap = (DDSRT_ENDIAN == DDSRT_LITTLE_ENDIAN);
+  DDSRT_WARNING_MSVC_ON(6326)
 
   hlen = swap ? ddsrt_bswap2u(body->length) : body->length;
 
@@ -357,7 +360,7 @@ static bool read_body(DDS_Security_OctetSeq *contents, bool encrypted, unsigned
   {
     struct encrypted_data *enc;
 
-    if (body->id != SMID_SEC_BODY_KIND)
+    if (body->id != SMID_SEC_BODY)
     {
       printf("check_encoded_data: submessage SEC_BODY missing\n");
       return false;
@@ -370,7 +373,7 @@ static bool read_body(DDS_Security_OctetSeq *contents, bool encrypted, unsigned
   }
   else
   {
-    if (body->id == SMID_SEC_BODY_KIND)
+    if (body->id == SMID_SEC_BODY)
     {
       printf("check_encoded_data: submessage SEC_BODY not expected\n");
       return false;
@@ -399,7 +402,7 @@ static bool read_postfix(unsigned char **ptr, uint32_t *remain)
 
   postfix = (struct submsg_header *)(*ptr);
 
-  if (postfix->id != SMID_SEC_POSTFIX_KIND)
+  if (postfix->id != SMID_SEC_POSTFIX)
   {
     printf("check_encoded_data: postfix invalid smid\n");
     return false;
@@ -478,6 +481,12 @@ static bool cipher_sign_data(const unsigned char *session_key, uint32_t key_size
       goto fail_encrypt;
     }
   }
+  else
+  {
+    assert (0);
+    goto fail_encrypt;
+  }
+
 
   /* Initialise key and IV */
   if (!EVP_EncryptInit_ex(ctx, NULL, NULL, session_key, iv))
@@ -547,6 +556,15 @@ static bool crypto_decrypt_data(uint32_t session_id, unsigned char *iv, DDS_Secu
         result = false;
       }
     }
+    else
+    {
+      assert (0);
+      return false;
+    }
+  }
+  else
+  {
+    result = false;
   }
 
   if (result)
@@ -635,7 +653,7 @@ static void prepare_endpoint_security_attributes_and_properties(DDS_Security_End
   properties->_maximum = properties->_length = 1;
   properties->_buffer = ddsrt_malloc(sizeof(DDS_Security_Property_t));
 
-  properties->_buffer[0].name = ddsrt_strdup("dds.sec.crypto.keysize");
+  properties->_buffer[0].name = ddsrt_strdup(DDS_SEC_PROP_CRYPTO_KEYSIZE);
   if (transformation_kind == CRYPTO_TRANSFORMATION_KIND_AES128_GCM || transformation_kind == CRYPTO_TRANSFORMATION_KIND_AES128_GMAC)
   {
     properties->_buffer[0].value = ddsrt_strdup("128");
@@ -759,7 +777,8 @@ static void suite_encode_datareader_submessage_init(void)
   CU_ASSERT_FATAL ((plugins = load_plugins(
                       NULL    /* Access Control */,
                       NULL    /* Authentication */,
-                      &crypto /* Cryptograpy    */)) != NULL);
+                      &crypto /* Cryptograpy    */,
+                      NULL)) != NULL);
   CU_ASSERT_EQUAL_FATAL (register_local_participant(), 0);
   CU_ASSERT_EQUAL_FATAL (register_remote_participant(), 0);
 }
@@ -1116,46 +1135,6 @@ CU_Test(ddssec_builtin_encode_datareader_submessage, invalid_args, .init = suite
   writer_list._buffer = DDS_Security_DatareaderCryptoHandleSeq_allocbuf(1);
   writer_list._buffer[0] = writer_crypto;
 
-  /* encoded_buffer NULL */
-  result = crypto->crypto_transform->encode_datareader_submessage(
-      crypto->crypto_transform,
-      NULL,
-      &plain_buffer,
-      reader_crypto,
-      &writer_list,
-      &exception);
-
-  if (!result)
-  {
-    printf("encode_datareader_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
-  /* plain_buffer NULL */
-  result = crypto->crypto_transform->encode_datareader_submessage(
-      crypto->crypto_transform,
-      &encoded_buffer,
-      NULL,
-      writer_crypto,
-      &writer_list,
-      &exception);
-
-  if (!result)
-  {
-    printf("encode_datareader_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
   /* writer crypto 0 */
   result = crypto->crypto_transform->encode_datareader_submessage(
       crypto->crypto_transform,
@@ -1196,46 +1175,6 @@ CU_Test(ddssec_builtin_encode_datareader_submessage, invalid_args, .init = suite
 
   reset_exception(&exception);
 
-  /* reader crypto list NULL*/
-  result = crypto->crypto_transform->encode_datareader_submessage(
-      crypto->crypto_transform,
-      &encoded_buffer,
-      &plain_buffer,
-      writer_crypto,
-      NULL,
-      &exception);
-
-  if (!result)
-  {
-    printf("encode_datareader_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
-  /* empty reader crypto list */
-  result = crypto->crypto_transform->encode_datareader_submessage(
-      crypto->crypto_transform,
-      &encoded_buffer,
-      &plain_buffer,
-      writer_crypto,
-      &empty_writer_list,
-      &exception);
-
-  if (!result)
-  {
-    printf("encode_datareader_submessage: %s\n", exception.message ? exception.message : "Error message missing");
-  }
-
-  CU_ASSERT(!result);
-  CU_ASSERT(exception.code != 0);
-  CU_ASSERT(exception.message != NULL);
-
-  reset_exception(&exception);
-
   /* writer crypto list with invalid writer crypto */
   writer_list._buffer[0] = 0;
   result = crypto->crypto_transform->encode_datareader_submessage(
