@@ -27,31 +27,16 @@
 #include "CUnit/Test.h"
 #include "common/src/loader.h"
 #include "common/src/handshake_helper.h"
+#include "auth_tokens.h"
+#include "ac_tokens.h"
 
 #define HANDSHAKE_SIGNATURE_SIZE 6
 
-static const char * AUTH_PROTOCOL_CLASS_ID          = "DDS:Auth:PKI-DH:1.0";
-static const char * PERM_ACCESS_CLASS_ID            = "DDS:Access:Permissions:1.0";
-
-static const char * PROPERTY_IDENTITY_CA            = "dds.sec.auth.identity_ca";
-static const char * PROPERTY_PRIVATE_KEY            = "dds.sec.auth.private_key";
-static const char * PROPERTY_IDENTITY_CERT          = "dds.sec.auth.identity_certificate";
-
-static const char * PROPERTY_CERT_SUBJECT_NAME      = "dds.cert.sn";
-static const char * PROPERTY_CERT_ALGORITHM         = "dds.cert.algo";
-static const char * PROPERTY_CA_SUBJECT_NAME        = "dds.ca.sn";
-static const char * PROPERTY_CA_ALGORITHM           = "dds.ca.aglo";
-
-static const char * PROPERTY_PERM_CA_SUBJECT_NAME   = "ds.perm_ca.sn";
-
 static const char * SUBJECT_NAME_IDENTITY_CERT      = "CN=CHAM-574 client,O=Some Company,ST=Some-State,C=NL";
 static const char * SUBJECT_NAME_IDENTITY_CA        = "CN=CHAM-574 authority,O=Some Company,ST=Some-State,C=NL";
 
 static const char * RSA_2048_ALGORITHM_NAME         = "RSA-2048";
 
-static const char * AUTH_REQUEST_TOKEN_CLASS_ID         = "DDS:Auth:PKI-DH:1.0+AuthReq";
-static const char * AUTH_REQUEST_TOKEN_FUTURE_PROP_NAME = "future_challenge";
-
 static const char * AUTH_HANDSHAKE_REQUEST_TOKEN_CLASS_ID = "DDS:Auth:PKI-DH:1.0+Req";
 static const char * AUTH_HANDSHAKE_REPLY_TOKEN_CLASS_ID   = "DDS:Auth:PKI-DH:1.0+Reply";
 static const char * AUTH_HANDSHAKE_FINAL_TOKEN_CLASS_ID   = "DDS:Auth:PKI-DH:1.0+Final";
@@ -303,24 +288,24 @@ initialize_identity_token(
 {
     memset(token, 0, sizeof(*token));
 
-    token->class_id = ddsrt_strdup(AUTH_PROTOCOL_CLASS_ID);
+    token->class_id = ddsrt_strdup(DDS_AUTHTOKEN_CLASS_ID);
     token->properties._maximum = 4;
     token->properties._length  = 4;
     token->properties._buffer = DDS_Security_PropertySeq_allocbuf(4);
 
-    token->properties._buffer[0].name = ddsrt_strdup(PROPERTY_CERT_SUBJECT_NAME);
+    token->properties._buffer[0].name = ddsrt_strdup(DDS_AUTHTOKEN_PROP_CERT_SN);
     token->properties._buffer[0].value = ddsrt_strdup(SUBJECT_NAME_IDENTITY_CERT);
     token->properties._buffer[0].propagate = true;
 
-    token->properties._buffer[1].name = ddsrt_strdup(PROPERTY_CERT_ALGORITHM);
+    token->properties._buffer[1].name = ddsrt_strdup(DDS_AUTHTOKEN_PROP_CERT_ALGO);
     token->properties._buffer[1].value = ddsrt_strdup(certAlgo);
     token->properties._buffer[1].propagate = true;
 
-    token->properties._buffer[2].name = ddsrt_strdup(PROPERTY_CA_SUBJECT_NAME);
+    token->properties._buffer[2].name = ddsrt_strdup(DDS_AUTHTOKEN_PROP_CA_SN);
     token->properties._buffer[2].value = ddsrt_strdup(SUBJECT_NAME_IDENTITY_CA);
     token->properties._buffer[2].propagate = true;
 
-    token->properties._buffer[3].name = ddsrt_strdup(PROPERTY_CA_ALGORITHM);
+    token->properties._buffer[3].name = ddsrt_strdup(DDS_AUTHTOKEN_PROP_CA_ALGO);
     token->properties._buffer[3].value = ddsrt_strdup(caAlgo);
     token->properties._buffer[3].propagate = true;
 }
@@ -330,13 +315,13 @@ initialize_permissions_token(
     DDS_Security_PermissionsToken *token,
     const char *caAlgo)
 {
-    token->class_id = ddsrt_strdup(PERM_ACCESS_CLASS_ID);
+    token->class_id = ddsrt_strdup(DDS_ACTOKEN_PERMISSIONS_CLASS_ID);
     token->properties._length = 2;
     token->properties._maximum = 2;
     token->properties._buffer = DDS_Security_PropertySeq_allocbuf(4);
-    token->properties._buffer[0].name = ddsrt_strdup(PROPERTY_CERT_SUBJECT_NAME);
+    token->properties._buffer[0].name = ddsrt_strdup(DDS_AUTHTOKEN_PROP_CERT_SN);
     token->properties._buffer[0].value = ddsrt_strdup(SUBJECT_NAME_IDENTITY_CA);
-    token->properties._buffer[1].name = ddsrt_strdup(PROPERTY_PERM_CA_SUBJECT_NAME);
+    token->properties._buffer[1].name = ddsrt_strdup(DDS_ACTOKEN_PROP_PERM_CA_SN);
     token->properties._buffer[1].value = ddsrt_strdup(caAlgo);
 }
 
@@ -358,11 +343,11 @@ fill_auth_request_token(
 
     memset(token, 0, sizeof(*token));
 
-    token->class_id = ddsrt_strdup(AUTH_REQUEST_TOKEN_CLASS_ID);
+    token->class_id = ddsrt_strdup(DDS_SECURITY_AUTH_REQUEST_TOKEN_CLASS_ID);
     token->binary_properties._maximum = 1;
     token->binary_properties._length = 1;
     token->binary_properties._buffer = DDS_Security_BinaryPropertySeq_allocbuf(1);
-    token->binary_properties._buffer->name = ddsrt_strdup(AUTH_REQUEST_TOKEN_FUTURE_PROP_NAME);
+    token->binary_properties._buffer->name = ddsrt_strdup(DDS_AUTHTOKEN_PROP_FUTURE_CHALLENGE);
 
     token->binary_properties._buffer->value._maximum = (DDS_Security_unsigned_long) len;
     token->binary_properties._buffer->value._length = (DDS_Security_unsigned_long) len;
@@ -433,11 +418,11 @@ validate_local_identity(void)
 
     memset(&participant_qos, 0, sizeof(participant_qos));
     dds_security_property_init(&participant_qos.property.value, 3);
-    participant_qos.property.value._buffer[0].name = ddsrt_strdup(PROPERTY_IDENTITY_CERT);
+    participant_qos.property.value._buffer[0].name = ddsrt_strdup(DDS_SEC_PROP_AUTH_IDENTITY_CERT);
     participant_qos.property.value._buffer[0].value = ddsrt_strdup(identity_certificate);
-    participant_qos.property.value._buffer[1].name = ddsrt_strdup(PROPERTY_IDENTITY_CA);
+    participant_qos.property.value._buffer[1].name = ddsrt_strdup(DDS_SEC_PROP_AUTH_IDENTITY_CA);
     participant_qos.property.value._buffer[1].value = ddsrt_strdup(identity_ca);
-    participant_qos.property.value._buffer[2].name = ddsrt_strdup(PROPERTY_PRIVATE_KEY);
+    participant_qos.property.value._buffer[2].name = ddsrt_strdup(DDS_SEC_PROP_AUTH_PRIV_KEY);
     participant_qos.property.value._buffer[2].value = ddsrt_strdup(private_key);
 
     /* Now call the function. */
@@ -846,7 +831,7 @@ validate_remote_identities (const char *remote_id_certificate)
     g_remote_participant_data2->security_info.participant_security_attributes = 0x01;
     g_remote_participant_data2->security_info.plugin_participant_security_attributes = 0x02;
 
-    g_challenge1_predefined_glb = find_binary_property(&g_remote_auth_request_token, AUTH_REQUEST_TOKEN_FUTURE_PROP_NAME);
+    g_challenge1_predefined_glb = find_binary_property(&g_remote_auth_request_token, DDS_AUTHTOKEN_PROP_FUTURE_CHALLENGE);
     g_challenge2_predefined_glb = g_challenge1_predefined_glb;
 
     return res;
@@ -880,7 +865,8 @@ CU_Init(ddssec_builtin_get_authenticated_peer_credential)
     /* Only need the authentication plugin. */
     g_plugins = load_plugins(NULL   /* Access Control */,
                              &g_auth  /* Authentication */,
-                             NULL   /* Cryptograpy    */);
+                             NULL   /* Cryptograpy    */,
+                             &(const struct ddsi_domaingv){ .handshake_include_optional = true });
     if (g_plugins) {
         result = validate_local_identity();
         if (result >= 0) {
@@ -1028,30 +1014,30 @@ fill_handshake_message_token(
 
        /* Store the Identity Certificate associated with the local identify in c.id property */
        if (certificate) {
-           set_binary_property_string(c_id, "c.id", certificate);
+           set_binary_property_string(c_id, DDS_AUTHTOKEN_PROP_C_ID, certificate);
        } else {
-           set_binary_property_string(c_id, "c.idx", "rubbish");
+           set_binary_property_string(c_id, DDS_AUTHTOKEN_PROP_C_ID "x", "rubbish");
        }
 
        /* Store the permission document in the c.perm property */
-       set_binary_property_string(c_perm, "c.perm", PERMISSIONS_DOCUMENT);
+       set_binary_property_string(c_perm, DDS_AUTHTOKEN_PROP_C_PERM, PERMISSIONS_DOCUMENT);
 
        /* Store the provided g_local_participant_data in the c.pdata property */
-       set_binary_property_value(c_pdata, "c.pdata", serialized_local_participant_data, serialized_local_participant_data_size);
+       set_binary_property_value(c_pdata, DDS_AUTHTOKEN_PROP_C_PDATA, serialized_local_participant_data, serialized_local_participant_data_size);
        ddsrt_free(serialized_local_participant_data);
 
        /* Set the used signing algorithm descriptor in c.dsign_algo */
        if (dsign) {
-           set_binary_property_string(c_dsign_algo, "c.dsign_algo", dsign);
+           set_binary_property_string(c_dsign_algo, DDS_AUTHTOKEN_PROP_C_DSIGN_ALGO, dsign);
        } else {
-           set_binary_property_string(c_dsign_algo, "c.dsign_algox", "rubbish");
+           set_binary_property_string(c_dsign_algo, DDS_AUTHTOKEN_PROP_C_DSIGN_ALGO "x", "rubbish");
        }
 
        /* Set the used key algorithm descriptor in c.kagree_algo */
        if (kagree) {
-           set_binary_property_string(c_kagree_algo, "c.kagree_algo", kagree);
+           set_binary_property_string(c_kagree_algo, DDS_AUTHTOKEN_PROP_C_KAGREE_ALGO, kagree);
        } else {
-           set_binary_property_string(c_kagree_algo, "c.kagree_algox", "rubbish");
+           set_binary_property_string(c_kagree_algo, DDS_AUTHTOKEN_PROP_C_KAGREE_ALGO "x", "rubbish");
        }
 
        /* Calculate the hash_c1 */
@@ -1073,21 +1059,21 @@ fill_handshake_message_token(
            ddsrt_free(buffer);
            DDS_Security_Serializer_free(serializer);
 
-           set_binary_property_value(hash_c1, "hash_c1", hash1_sentrequest_arr, sizeof(hash1_sentrequest_arr));
+           set_binary_property_value(hash_c1, DDS_AUTHTOKEN_PROP_HASH_C1, hash1_sentrequest_arr, sizeof(hash1_sentrequest_arr));
        }
 
        /* Set the DH public key associated with the local participant in dh1 property */
        if (diffie_hellman1) {
-           set_binary_property_value(dh1, "dh1", diffie_hellman1->data, diffie_hellman1->length);
+           set_binary_property_value(dh1, DDS_AUTHTOKEN_PROP_DH1, diffie_hellman1->data, diffie_hellman1->length);
        } else {
-           set_binary_property_string(dh1, "dh1x", "rubbish");
+           set_binary_property_string(dh1, DDS_AUTHTOKEN_PROP_DH1 "x", "rubbish");
        }
 
        /* Set the challenge in challenge1 property */
        if (challengeData) {
-           set_binary_property_value(challenge1, "challenge1", challengeData, challengeDataSize);
+           set_binary_property_value(challenge1, DDS_AUTHTOKEN_PROP_CHALLENGE1, challengeData, challengeDataSize);
        } else {
-           set_binary_property_value(challenge1, "challenge1x", g_challenge1_predefined_glb->value._buffer, g_challenge1_predefined_glb->value._length);
+           set_binary_property_value(challenge1, DDS_AUTHTOKEN_PROP_CHALLENGE1 "x", g_challenge1_predefined_glb->value._buffer, g_challenge1_predefined_glb->value._length);
        }
 
        token->class_id = ddsrt_strdup(AUTH_HANDSHAKE_REQUEST_TOKEN_CLASS_ID);
@@ -1115,35 +1101,35 @@ fill_handshake_message_token(
 
         /* Store the Identity Certificate associated with the local identify in c.id property */
         if (certificate) {
-           set_binary_property_string(c_id, "c.id", certificate);
+           set_binary_property_string(c_id, DDS_AUTHTOKEN_PROP_C_ID, certificate);
         } else {
-           set_binary_property_string(c_id, "c.idx", "rubbish");
+           set_binary_property_string(c_id, DDS_AUTHTOKEN_PROP_C_ID "x", "rubbish");
         }
 
         /* Store the permission document in the c.perm property */
-        set_binary_property_string(c_perm, "c.perm", PERMISSIONS_DOCUMENT);
+        set_binary_property_string(c_perm, DDS_AUTHTOKEN_PROP_C_PERM, PERMISSIONS_DOCUMENT);
 
         /* Store the provided g_local_participant_data in the c.pdata property */
-        set_binary_property_value(c_pdata, "c.pdata", serialized_local_participant_data, serialized_local_participant_data_size);
+        set_binary_property_value(c_pdata, DDS_AUTHTOKEN_PROP_C_PDATA, serialized_local_participant_data, serialized_local_participant_data_size);
         ddsrt_free(serialized_local_participant_data);
 
         /* Set the used signing algorithm descriptor in c.dsign_algo */
         if (dsign) {
-           set_binary_property_string(c_dsign_algo, "c.dsign_algo", dsign);
+           set_binary_property_string(c_dsign_algo, DDS_AUTHTOKEN_PROP_C_DSIGN_ALGO, dsign);
         } else {
-           set_binary_property_string(c_dsign_algo, "c.dsign_algox", "rubbish");
+           set_binary_property_string(c_dsign_algo, DDS_AUTHTOKEN_PROP_C_DSIGN_ALGO "x", "rubbish");
         }
 
         /* Set the used key algorithm descriptor in c.kagree_algo */
         if (kagree) {
-           set_binary_property_string(c_kagree_algo, "c.kagree_algo", kagree);
+           set_binary_property_string(c_kagree_algo, DDS_AUTHTOKEN_PROP_C_KAGREE_ALGO, kagree);
         } else {
-           set_binary_property_string(c_kagree_algo, "c.kagree_algox", "rubbish");
+           set_binary_property_string(c_kagree_algo, DDS_AUTHTOKEN_PROP_C_KAGREE_ALGO "x", "rubbish");
         }
 
         CU_ASSERT(hash1_from_request != NULL);
 
-        set_binary_property_value(hash_c1, "hash_c1", hash1_from_request->value._buffer, hash1_from_request->value._length);
+        set_binary_property_value(hash_c1, DDS_AUTHTOKEN_PROP_HASH_C1, hash1_from_request->value._buffer, hash1_from_request->value._length);
 
         /* Calculate the hash_c2 */
         {
@@ -1165,36 +1151,36 @@ fill_handshake_message_token(
            ddsrt_free(buffer);
            DDS_Security_Serializer_free(serializer);
 
-           set_binary_property_value(hash_c2, "hash_c2", hash2_sentreply_arr, sizeof(hash2_sentreply_arr));
+           set_binary_property_value(hash_c2, DDS_AUTHTOKEN_PROP_HASH_C2, hash2_sentreply_arr, sizeof(hash2_sentreply_arr));
         }
 
         /* Set the challenge in challenge1 property */
         if (challengeData) {
-          set_binary_property_value(challenge1, "challenge1", challengeData, challengeDataSize);
+          set_binary_property_value(challenge1, DDS_AUTHTOKEN_PROP_CHALLENGE1, challengeData, challengeDataSize);
         } else {
-          set_binary_property_value(challenge1, "challenge1x", challenge2->value._buffer, challenge2->value._length);
+          set_binary_property_value(challenge1, DDS_AUTHTOKEN_PROP_CHALLENGE1 "x", challenge2->value._buffer, challenge2->value._length);
         }
 
         /* Set the challenge in challenge2 property */
         if (challengeData2) {
-           set_binary_property_value(challenge2, "challenge2", challengeData2, challengeDataSize2);
+           set_binary_property_value(challenge2, DDS_AUTHTOKEN_PROP_CHALLENGE2, challengeData2, challengeDataSize2);
         } else {
-           set_binary_property_value(challenge2, "challenge2x", challenge2->value._buffer, challenge2->value._length);
+           set_binary_property_value(challenge2, DDS_AUTHTOKEN_PROP_CHALLENGE2 "x", challenge2->value._buffer, challenge2->value._length);
         }
 
 
         /* Set the DH public key associated with the local participant in dh1 property */
         if (diffie_hellman1) {
-          set_binary_property_value(dh1, "dh1", diffie_hellman1->data, diffie_hellman1->length);
+          set_binary_property_value(dh1, DDS_AUTHTOKEN_PROP_DH1, diffie_hellman1->data, diffie_hellman1->length);
         } else {
-          set_binary_property_string(dh1, "dh1x", "rubbish");
+          set_binary_property_string(dh1, DDS_AUTHTOKEN_PROP_DH1 "x", "rubbish");
         }
 
         /* Set the DH public key associated with the local participant in dh2 property */
         if (diffie_hellman2) {
-          set_binary_property_value(dh2, "dh2", diffie_hellman2->data, diffie_hellman2->length);
+          set_binary_property_value(dh2, DDS_AUTHTOKEN_PROP_DH2, diffie_hellman2->data, diffie_hellman2->length);
         } else {
-          set_binary_property_string(dh2, "dh2x", "rubbish");
+          set_binary_property_string(dh2, DDS_AUTHTOKEN_PROP_DH2 "x", "rubbish");
         }
 
         /* Calculate the signature */
@@ -1223,7 +1209,7 @@ fill_handshake_message_token(
             {
                 printf("Exception: %s\n", exception.message);
             }
-            set_binary_property_value(signature, "signature", sign, signlen);
+            set_binary_property_value(signature, DDS_AUTHTOKEN_PROP_SIGNATURE, sign, signlen);
 
             ddsrt_free(sign);
             BIO_free(bio);
@@ -1250,38 +1236,38 @@ fill_handshake_message_token(
         CU_ASSERT(hash2_from_reply != NULL);
         assert(hash1_from_request && hash2_from_reply); // for Clang's static analyzer
 
-        set_binary_property_value(hash_c1, "hash_c1", hash1_from_request->value._buffer, hash1_from_request->value._length);
-        set_binary_property_value(hash_c2, "hash_c2", hash2_from_reply->value._buffer, hash2_from_reply->value._length);
+        set_binary_property_value(hash_c1, DDS_AUTHTOKEN_PROP_HASH_C1, hash1_from_request->value._buffer, hash1_from_request->value._length);
+        set_binary_property_value(hash_c2, DDS_AUTHTOKEN_PROP_HASH_C2, hash2_from_reply->value._buffer, hash2_from_reply->value._length);
 
         printf("process: %s\n", hash_c1->name);
 
         /* Set the challenge in challenge1 property */
         if (challengeData) {
-          set_binary_property_value(challenge1, "challenge1", challengeData, challengeDataSize);
+          set_binary_property_value(challenge1, DDS_AUTHTOKEN_PROP_CHALLENGE1, challengeData, challengeDataSize);
         } else {
-          set_binary_property_value(challenge1, "challenge1x", challenge2->value._buffer, challenge2->value._length);
+          set_binary_property_value(challenge1, DDS_AUTHTOKEN_PROP_CHALLENGE1 "x", challenge2->value._buffer, challenge2->value._length);
         }
 
         /* Set the challenge in challenge2 property */
         if (challengeData2) {
-           set_binary_property_value(challenge2, "challenge2", challengeData2, challengeDataSize2);
+           set_binary_property_value(challenge2, DDS_AUTHTOKEN_PROP_CHALLENGE2, challengeData2, challengeDataSize2);
         } else {
-           set_binary_property_value(challenge2, "challenge2x", challenge2->value._buffer, challenge2->value._length);
+           set_binary_property_value(challenge2, DDS_AUTHTOKEN_PROP_CHALLENGE2 "x", challenge2->value._buffer, challenge2->value._length);
         }
 
 
         /* Set the DH public key associated with the local participant in dh1 property */
         if (diffie_hellman1) {
-          set_binary_property_value(dh1, "dh1", diffie_hellman1->data, diffie_hellman1->length);
+          set_binary_property_value(dh1, DDS_AUTHTOKEN_PROP_DH1, diffie_hellman1->data, diffie_hellman1->length);
         } else {
-          set_binary_property_string(dh1, "dh1x", "rubbish");
+          set_binary_property_string(dh1, DDS_AUTHTOKEN_PROP_DH1 "x", "rubbish");
         }
 
         /* Set the DH public key associated with the local participant in dh2 property */
         if (diffie_hellman2) {
-          set_binary_property_value(dh2, "dh2", diffie_hellman2->data, diffie_hellman2->length);
+          set_binary_property_value(dh2, DDS_AUTHTOKEN_PROP_DH2, diffie_hellman2->data, diffie_hellman2->length);
         } else {
-          set_binary_property_string(dh2, "dh2x", "rubbish");
+          set_binary_property_string(dh2, DDS_AUTHTOKEN_PROP_DH2 "x", "rubbish");
         }
 
         /* Calculate the signature */
@@ -1309,7 +1295,7 @@ fill_handshake_message_token(
             {
                 printf("Exception: %s\n", exception.message);
             }
-            set_binary_property_value(signature, "signature", sign, signlen);
+            set_binary_property_value(signature, DDS_AUTHTOKEN_PROP_SIGNATURE, sign, signlen);
 
             ddsrt_free(sign);
             BIO_free(bio);
@@ -1378,11 +1364,11 @@ CU_Test(ddssec_builtin_get_authenticated_peer_credential,token_after_request )
     assert(result == DDS_SECURITY_VALIDATION_PENDING_HANDSHAKE_MESSAGE); // for Clang's static analyzer
 
     /* mock reply */
-    dh1 = find_binary_property(&handshake_token_out, "dh1");
+    dh1 = find_binary_property(&handshake_token_out, DDS_AUTHTOKEN_PROP_DH1);
     dh1_pub_key.data = dh1->value._buffer;
     dh1_pub_key.length = dh1->value._length;
-    challenge1_glb = find_binary_property(&handshake_token_out, "challenge1");
-    hash1_sentrequest = find_binary_property(&handshake_token_out, "hash_c1");
+    challenge1_glb = find_binary_property(&handshake_token_out, DDS_AUTHTOKEN_PROP_CHALLENGE1);
+    hash1_sentrequest = find_binary_property(&handshake_token_out, DDS_AUTHTOKEN_PROP_HASH_C1);
     fill_handshake_message_token(
                     &handshake_reply_token_in,
                     g_remote_participant_data2,
@@ -1424,18 +1410,19 @@ CU_Test(ddssec_builtin_get_authenticated_peer_credential,token_after_request )
 
     CU_ASSERT_FATAL(credential_token.class_id != NULL);
     assert(credential_token.class_id); // for Clang's static analyzer
-    CU_ASSERT(strcmp(credential_token.class_id, AUTH_PROTOCOL_CLASS_ID) == 0);
+    CU_ASSERT(strcmp(credential_token.class_id, DDS_AUTHTOKEN_CLASS_ID) == 0);
     CU_ASSERT(credential_token.properties._length == 2);
     CU_ASSERT(credential_token.binary_properties._length == 0);
 
-    c_id = find_property(&credential_token, "c.id");
+    c_id = find_property(&credential_token, DDS_AUTHTOKEN_PROP_C_ID);
     CU_ASSERT_FATAL(c_id != NULL);
+    assert(c_id); // for GCC's static analyzer
     CU_ASSERT_FATAL(c_id->value != NULL);
     assert(c_id && c_id->value); // for Clang's static analyzer
     //printf("c_id->value: %s\n", c_id->value);
     CU_ASSERT(strcmp(c_id->value, REMOTE_IDENTITY_CERTIFICATE) == 0);
 
-    c_perm = find_property(&credential_token, "c.perm");
+    c_perm = find_property(&credential_token, DDS_AUTHTOKEN_PROP_C_PERM);
     CU_ASSERT_FATAL(c_perm != NULL);
     CU_ASSERT_FATAL(c_perm->value != NULL);
     assert(c_perm && c_perm->value); // for Clang's static analyzer
@@ -1508,12 +1495,12 @@ CU_Test(ddssec_builtin_get_authenticated_peer_credential,token_after_reply )
     assert(result == DDS_SECURITY_VALIDATION_PENDING_HANDSHAKE_MESSAGE); // for Clang's static analyzer
 
     /* mock final */
-    dh2 = find_binary_property(&handshake_token_out, "dh2");
+    dh2 = find_binary_property(&handshake_token_out, DDS_AUTHTOKEN_PROP_DH2);
     dh2_pub_key.data = dh2->value._buffer;
     dh2_pub_key.length = dh2->value._length;
-    challenge2_glb = find_binary_property(&handshake_token_out, "challenge2");
-    hash1_sentrequest = find_binary_property(&handshake_token_out, "hash_c1");
-    hash2_sentreply = find_binary_property(&handshake_token_out, "hash_c2");
+    challenge2_glb = find_binary_property(&handshake_token_out, DDS_AUTHTOKEN_PROP_CHALLENGE2);
+    hash1_sentrequest = find_binary_property(&handshake_token_out, DDS_AUTHTOKEN_PROP_HASH_C1);
+    hash2_sentreply = find_binary_property(&handshake_token_out, DDS_AUTHTOKEN_PROP_HASH_C2);
     fill_handshake_message_token(
                 &handshake_final_token_in,
                 NULL,
@@ -1554,18 +1541,19 @@ CU_Test(ddssec_builtin_get_authenticated_peer_credential,token_after_reply )
     assert(success); // for Clang's static analyzer
 
     CU_ASSERT_FATAL(credential_token.class_id != NULL);
-    CU_ASSERT(strcmp(credential_token.class_id, AUTH_PROTOCOL_CLASS_ID) == 0);
+    CU_ASSERT(strcmp(credential_token.class_id, DDS_AUTHTOKEN_CLASS_ID) == 0);
     CU_ASSERT(credential_token.properties._length == 2);
     CU_ASSERT(credential_token.binary_properties._length == 0);
 
-    c_id = find_property(&credential_token, "c.id");
+    c_id = find_property(&credential_token, DDS_AUTHTOKEN_PROP_C_ID);
     CU_ASSERT_FATAL(c_id != NULL);
+    assert(c_id); // for GCC's static analyzer
     CU_ASSERT_FATAL(c_id->value != NULL);
     assert(c_id && c_id->value); // for Clang's static analyzer
     //printf("c_id->value: %s\n", c_id->value);
     CU_ASSERT(strcmp(c_id->value, REMOTE_IDENTITY_CERTIFICATE) == 0);
 
-    c_perm = find_property(&credential_token, "c.perm");
+    c_perm = find_property(&credential_token, DDS_AUTHTOKEN_PROP_C_PERM);
     CU_ASSERT_FATAL(c_perm != NULL);
     CU_ASSERT_FATAL(c_perm->value != NULL);
     assert(c_perm && c_perm->value); // for Clang's static analyzer
