@@ -118,6 +118,8 @@ void hist_record(struct hist *h, uint64_t x, unsigned weight) {
         h->bins[(x - h->bin0) / h->binwidth] += weight;
 }
 
+static void xsnprintf(char *buf, size_t bufsz, size_t *p, const char *fmt, ...) ddsrt_attribute_format_printf(4, 5);
+
 static void xsnprintf(char *buf, size_t bufsz, size_t *p, const char *fmt, ...) {
     if (*p < bufsz) {
         int n;
@@ -217,6 +219,7 @@ void hist_print(struct hist *h, dds_time_t dt, int reset) {
         hist_reset(h);
 }
 
+void error(const char *fmt, ...) ddsrt_attribute_format_printf(1, 2);
 void error(const char *fmt, ...) {
     va_list ap;
     fprintf (stderr, "%s: error: ", saved_argv0);
@@ -957,7 +960,7 @@ bool dds_err_check (dds_return_t err, unsigned flags, const char * where)
     if (flags & (DDS_CHECK_REPORT | DDS_CHECK_FAIL))
     {
       char msg[DDS_ERR_MSG_MAX];
-      (void) snprintf (msg, DDS_ERR_MSG_MAX, "Error %"PRId32":M%"PRId32":%s", dds_err_file_id(err), dds_err_line(err), dds_err_str(err));
+      (void) snprintf (msg, DDS_ERR_MSG_MAX, "Error:%s", dds_err_str(err));
       if (flags & DDS_CHECK_REPORT)
       {
         printf ("%s: %s\n", where, msg);
